package keenGame;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.awt.GraphicsConfiguration;
import java.awt.Image;
import java.awt.Point;
import java.awt.Transparency;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.lang.reflect.Method;
import java.util.Hashtable;
import java.util.Random;

import javax.swing.ImageIcon;
import keenGame.sprite.tile.PolygonTileFile;
import keenGame.sprite.tile.RectangleTileFile;


public class Cache
{
	//Used for cacheing methods
    private static Class [] classArray = new Class[1];
    private static Hashtable methodOwners = new Hashtable();
    private static Hashtable handleMethods;
    
    //Used for cacheing images
    private static Hashtable Images = new Hashtable();
    private static GraphicsConfiguration graphConfig;
    private static Image tempImage;

	//Used for cacheing PolygonTiles and RectangleTiles
    private static Hashtable PolygonTiles = new Hashtable();
    private static Hashtable RectangleTiles = new Hashtable();
    
    //Our random numbers
    public static Random rNumbers = new Random();
    
    //Used to load up resources, because java sucks
    private static ClassLoader classLoader;

	private static Hashtable scaledImages;
    
    /**
     * Black magic to discover the correct hasHit method to call when a collision
     * occurs.  It will return the method in class 'methodOwner', for dealing with
     * 'type', or failing that with return the method for dealing with the closest 
     * superclass of 'type'
     *
     * @param methodOwner - The class we are attempting to find an appropirate hasHit
	 * 						function in.
	 * @param type - The type of class we want the hasHit function to deal with.
	 */
    public static Method getHandleMethod(Class methodOwner, Class type)
   	{
   		if(methodOwners.containsKey(methodOwner))
   		{
   			handleMethods = (Hashtable)methodOwners.get(methodOwner);
   			if(handleMethods.containsKey(type))
   			{
   				return (Method)handleMethods.get(type);
   			}
   			else //we have the class but not this specific hasHit method
   			{
   				Method returnMethod = lookupHandleMethod(methodOwner,type);
   				handleMethods.put(type,returnMethod);
   				return returnMethod;
   			}
   		}
   		else //we dont have the class or method
   		{
   			//Add in the class
   			Hashtable handleMethods = new Hashtable();
   			methodOwners.put(methodOwner, handleMethods);
   			
   			//Add in the method
   			Method returnMethod = lookupHandleMethod(methodOwner,type);
   			handleMethods.put(type,returnMethod);
   			return returnMethod;
   		}
	}
	
	private static Method lookupHandleMethod(Class methodOwner, Class type)
	{
	   	Method returnMethod;
   		classArray[0] = type;
   		try
   		{
   			returnMethod = methodOwner.getMethod("hasHit", classArray);
   		}
   		catch(NoSuchMethodException e)
   		{
   			//Ok then, can the superclass handle hitting these things?
   			returnMethod = getHandleMethod(methodOwner, type.getSuperclass());
   		}
   		return returnMethod;
   	}
   	
   	public static void setGraphicsConfig(GraphicsConfiguration newGraphConfig)
    {
    	graphConfig = newGraphConfig;
    }
    
    public static void setClassLoader(ClassLoader newClassLoader)
    {
    	classLoader = newClassLoader;
    }

    public static ClassLoader getClassLoader()
    {
    	return classLoader;
    }

	public static Image getScaledImage(Image originalImage, double scale)
	{
    	if(graphConfig == null) //Incase we are trying to create sprites
       	{						//from a unit tester or something
       		System.err.println("Cache.getImage(String) No graphics config, returning null"); 
    		return null;
    	}
    	if(scaledImages == null)
    	{
    		scaledImages = new Hashtable();
    	}
    	
    	if(Images.containsKey(originalImage.toString() + Double.toString(scale)))
   		{
   			return (Image)Images.get(originalImage.toString() + Double.toString(scale));
   		}
   		else //We better load it up
   		{
   			//Get the image
			Image tempImage = originalImage.getScaledInstance((int)(originalImage.getWidth(null) * scale),
																(int)(originalImage.getHeight(null) * scale),
																Image.SCALE_SMOOTH);
        	//Add it into the hashTable
        	Images.put(originalImage.toString() + Double.toString(scale),tempImage);
        	return tempImage;
        }
	}
    
    public static Image getImage(String fileName)
    {
    	if(graphConfig == null) //Incase we are trying to create sprites
       	{						//from a unit tester or something
       		System.err.println("Cache.getImage(String) No graphics config, returning null"); 
    		return null;
    	}
    	
    	if(Images.containsKey(fileName))
   		{
   			return (Image)Images.get(fileName);
   		}
   		else //We better load it up
   		{
   			//Get the image
   			if(classLoader.getResource(fileName) == null)
   			{
   				System.err.println("Cannot find image " + fileName);
   				System.exit(0);
   			}
        	ImageIcon tempIcon = new ImageIcon(classLoader.getResource(fileName));

        	//Create our image space
        	tempImage = graphConfig.createCompatibleImage(tempIcon.getIconWidth(),
                                                  tempIcon.getIconHeight(),Transparency.BITMASK);

        	//Copy the image, to the volatile space in memory
        	tempImage.getGraphics().drawImage(tempIcon.getImage(),0,0,null);
        	
        	//Add it into the hashTable
        	Images.put(fileName,tempImage);
        	
        	
        	return tempImage;
        }
    }
    
    public static PolygonTileFile getPolygonTile(String fileName)
    {
    	if(PolygonTiles.containsKey(fileName))
   		{
   			return (PolygonTileFile)PolygonTiles.get(fileName);
   		}
   		else //We better load it up
   		{
   			//Get the image
        	PolygonTileFile tempFile=new PolygonTileFile();
        	
        	BufferedReader input;
        	String line;
        	String [] point;
        	
        	try
        	{
        		input = new BufferedReader(
        				new InputStreamReader(
        				classLoader.getResourceAsStream(fileName)));
        		
        		//Get the image
        		line = input.readLine();
        		tempFile.fileName = line;
        		tempFile.tileImage = getImage(line);
        		
        		//Get the shape
        		//Get the number of points
        		int numPoints = Integer.parseInt(input.readLine());
				tempFile.polyX = new int[numPoints];
				tempFile.polyY = new int[numPoints];
				
       			for (int i = 0; i < numPoints; i++)
				{
	        		line = input.readLine();					
	        		point = line.split(",");
	        		
					tempFile.polyX[i] = Integer.parseInt(point[0]);
					tempFile.polyY[i] = Integer.parseInt(point[1]);
				}
				
				//Now what grip points does it have
        		line = input.readLine();					
				if(line.equals("A")) //then all
				{				
	        		line = input.readLine();					
    	    		point = line.split(",");
					tempFile.rightPoint = new Point(Integer.parseInt(point[0]),Integer.parseInt(point[1]));

	        		line = input.readLine();					
   		    		point = line.split(",");
					tempFile.leftPoint = new Point(Integer.parseInt(point[0]),Integer.parseInt(point[1]));
				}
				else if(line.equals("L")) //then left
				{
	        		line = input.readLine();					
   		    		point = line.split(",");
					tempFile.leftPoint = new Point(Integer.parseInt(point[0]),Integer.parseInt(point[1]));
				}
				else if(line.equals("R")) //then right
				{
	        		line = input.readLine();					
   		    		point = line.split(",");
					tempFile.rightPoint = new Point(Integer.parseInt(point[0]),Integer.parseInt(point[1]));
				} //Else there is none
        	}
        	catch(IOException e)
        	{
        		System.err.println(e);
        		System.exit(0);
        	}
        	
        	//Add it into the hashTable
        	PolygonTiles.put(fileName,tempFile);
        	        	
        	return tempFile;
        }
    }
    
    public static RectangleTileFile getRectangleTile(String fileName)
    {
    	if(RectangleTiles.containsKey(fileName))
   		{
   			return (RectangleTileFile)RectangleTiles.get(fileName);
   		}
   		else //We better load it up
   		{
   			//Get the image
        	RectangleTileFile tempFile=new RectangleTileFile();
        	
        	BufferedReader input;
        	String line;
        	
        	try
        	{
        		input = new BufferedReader(
        				new InputStreamReader(
        				classLoader.getResourceAsStream(fileName)));
        		
        		//Get the image
        		line = input.readLine();
        		tempFile.fileName = line;
        		tempFile.tileImage = getImage(line);

        		line = input.readLine();					
				if(line.equals("A")) //then all
				{				
					tempFile.gripLeft = true;
					tempFile.gripRight = true;
					
				}
				else if(line.equals("L")) //then left
				{
					tempFile.gripLeft = true;
				}
				else if(line.equals("R")) //then right
				{
					tempFile.gripRight = true;
				}
				//else there is none
        		
        		//Get the rectangle
        		tempFile.x = Integer.parseInt(input.readLine());
        		tempFile.y = Integer.parseInt(input.readLine());
        		tempFile.width = Integer.parseInt(input.readLine());
        		tempFile.height = Integer.parseInt(input.readLine());
        	}
        	catch(IOException e)
        	{
        		System.err.println(e);
        		System.exit(0);
        	}
        	
        	//Add it into the hashTable
        	RectangleTiles.put(fileName,tempFile);
        	        	
        	return tempFile;
        }
    }

    public static void flush()
    {
    	methodOwners.clear();
    	handleMethods.clear();
    	Images.clear();
    	PolygonTiles.clear();
    	RectangleTiles.clear();
    }
}