package keenGame;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.awt.DisplayMode;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.Hashtable;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import keenGame.sprite.keenActor.KeenActorMovement;

public class ConfigMenuScreen extends Screen implements ActionListener, ListSelectionListener
{
	private JButton SetFullScreen, SetWindowed, Close , loadCustom;
	private JList ResolutionList;
	private JFrame hostApp;
	private JPanel movementPanel, movementPanel2;
	private boolean fullScreen = false;
	private DisplayMode originalMode;
	
	private JCheckBox debug;
	
   	private JFileChooser fileSelector;
   	private File fileName;

	
	private movementOption [] movement = new movementOption[16];

	public ConfigMenuScreen(Keen app)
	{
		super(app);
		hostApp = app;
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		//Box optionBox = new Box(BoxLayout.X_AXIS);
		//add(optionBox);
		JTabbedPane optionPane = new JTabbedPane();
		add(optionPane);
		
		//Set up resolution
		JPanel ResolutionPanel = new JPanel();
		ResolutionPanel.setLayout(new BoxLayout(ResolutionPanel, BoxLayout.Y_AXIS));
		//ResolutionPanel.setBorder(new TitledBorder("Set resolution"));
		
		GraphicsDevice device = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice();
		originalMode = device.getDisplayMode();
		ResolutionList = new JList(filterDisplayModes(device.getDisplayModes()));
		ResolutionList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		ResolutionList.addListSelectionListener(this);
		JScrollPane resolutionScroll = new JScrollPane(ResolutionList);
		ResolutionPanel.add(resolutionScroll);
		
		SetFullScreen = new JButton("Go to full-screen");
		SetFullScreen.setMnemonic(KeyEvent.VK_G);

		SetFullScreen.setEnabled(false);
		SetFullScreen.addActionListener(this);
		ResolutionPanel.add(SetFullScreen);
		
		SetWindowed = new JButton("Return to windowed mode");
		SetWindowed.setMnemonic(KeyEvent.VK_R);
		SetWindowed.setEnabled(false);
		SetWindowed.addActionListener(this);
		ResolutionPanel.add(SetWindowed);
		
		optionPane.addTab("Select Resolution", ResolutionPanel);
		
		//Construct movement options
		movementPanel = new JPanel();
		movementPanel.setLayout(new BoxLayout(movementPanel, BoxLayout.Y_AXIS));
		JTextArea descriptionText = new JTextArea(7, 20);
		descriptionText.setEditable(false);

		//movementPanel.setBorder(new TitledBorder("Keen movement constants"));
		
		JPanel miscMovement = new JPanel();
		miscMovement.setBorder(new TitledBorder("Misc"));
		movementPanel.add(miscMovement);
		
		String description = "The speed at which keen moves sideways, both when walking and jumping.\n" +
							 "Default = 4";
		movement[0] = new movementOption(4.0,"Walking speed", description, descriptionText);
		miscMovement.add(movement[0]);

		description = "Currently has no effect\n" +
	 					"Default = 5";
		movement[1] = new movementOption(5.0,"Scroll speed", description, descriptionText);
		miscMovement.add(movement[1]);
		
		description = "The ammount of time keen pauses when he grips a ledge.  When you grip onto\n" +
				"a ledge you cannot jump streight up.  If this time is set to zero keen will\n" +
				"always pull himself up streight away, because you wont have time to stop\n" +
				"pressing the left or right key.  A higher number means that keen will\n" +
				"hang onto a ledge for a longer number of milliseconds before he can pull\n" +
				"himself up\n" +
				"Default = 200";
		movement[2] = new movementOption(200.0,"Ledge grip time", description, descriptionText);
		miscMovement.add(movement[2]);
		
		JPanel poleMovement = new JPanel();
		poleMovement.setBorder(new TitledBorder("Pole climbing"));
		movementPanel.add(poleMovement);

		description = "The speed at which keen moves upward when he is on a pole and you press jump\n" +
						"A higher value will me keen move upward quicker, and hence jumps higher, when\n" +
						"you are griping a pole, and press the jump key\n" +
						"Default = 7";
		movement[3] = new movementOption(7.0,"Pole jump speed", description, descriptionText);
		poleMovement.add(movement[3]);
		
		description = "The speed at which keen climbs up poles\n" +
						"Default = 1";
		movement[4] = new movementOption(1.0,"Pole climbing speed", description, descriptionText);
		poleMovement.add(movement[4]);

		description = "The speed at which keen climbs down poles\n" +
						"Default = 3";
		movement[5] = new movementOption(3.0,"Pole descent speed", description, descriptionText);
		poleMovement.add(movement[5]);
		
		movementPanel.add(new JScrollPane(descriptionText));
		
		optionPane.addTab("Movement options",movementPanel);


		//Movement options 2
		movementPanel2 = new JPanel();
		movementPanel2.setLayout(new BoxLayout(movementPanel2, BoxLayout.Y_AXIS));


		JPanel jumpMovement = new JPanel();
		jumpMovement.setBorder(new TitledBorder("Jumping"));
		movementPanel2.add(jumpMovement);
		
		descriptionText = new JTextArea(7, 20);
		descriptionText.setEditable(false);


		description = "The speed at which keen begins his jumps.  Setting a higher value will\n" +
	 					"make keen jump higher, because he will have more speed at the beginning\n" + 
						"of his jump\n" +
						"Default = 9";
		movement[6] = new movementOption(9,"Jumping speed", description, descriptionText);
		jumpMovement.add(movement[6]);

		description = "The speed at which gravity pulls keen down while he is airborne.  Setting\n" +
						"a higher value will cause keens jumps to be smaller, as gravity will pull\n" +
						"him downwards quicker.\n" +
						"Default = 1.1";
		movement[7] = new movementOption(1.1,"Jump deccel", description, descriptionText);
		jumpMovement.add(movement[7]);
		
		description = "The speed at which gravity pulls keen down while he is airborne and the\n" +
						"jump key is held down.  This allows keen to jump higher if you keep the jump\n" +
						"key pressed while you are ariborne.  Setting a higher value will cause keens\n" +
						"jumps to be smaller, as gravity will pull him downwards quicker.\n" +
						"Default = 0.6";
		movement[8] = new movementOption(0.6,"Jump bost deccel", description, descriptionText);
		jumpMovement.add(movement[8]);

		description = "The speed at which keen can accelerate sideways while falling.  Setting a higher\n" +
						"value will allow keen move left and right with more responsiveness when falling\n" +
						"Default = 1.5";
		movement[9] = new movementOption(1.5,"Side acceleration", description, descriptionText);
		jumpMovement.add(movement[9]);
		
		JPanel pogoMovement = new JPanel(new GridLayout(2,6));
		pogoMovement.setBorder(new TitledBorder("Pogoing"));
		movementPanel2.add(pogoMovement);
		
		description = "The speed at which keen accelerates upwards when his pogo hits the ground\n" +
						"and the jump key is not pressed.  A higher value will make keen bounce higher\n" +
						"when he is pogoing.\n" +
						"Default = 8";
		movement[10] = new movementOption(8.0,"Pogo speed", description, descriptionText);
		pogoMovement.add(movement[10]);

		description = "The speed at which keen accelerates upwards when his pogo hits the ground\n" +
						"and the jump key is pressed.  A higher value will make keen bounce higher\n" +
						"when he is pogoing and you are pressing the jump key.\n" +
						"Default = 11";
		movement[11] = new movementOption(11.0,"Pogo boost speed", description, descriptionText);
		pogoMovement.add(movement[11]);

		description = "The speed at which gravity pulls keen downwards when he is pogoing.  A lower\n" +
						"value will make keen bounce higher because gravity will not pull him down as\n" +
						"fast\n" +
						"Default = 1.1";
		movement[12] = new movementOption(1.1,"Pogo deccel", description, descriptionText);
		pogoMovement.add(movement[12]);

		description = "The speed at which gravity pulls keen downwards when he is pogoing and the jump\n" +
						"key is pressed.  A lower value will make keen bounce higher when the jump key\n" +
						"is pressed because gravity will not pull him down as fast\n" +
						"Default = 0.5";
		movement[13] = new movementOption(0.5,"Pogo boost deccel", description, descriptionText);
		pogoMovement.add(movement[13]);

		description = "The speed at which keen acclerates left or right when he is pogoing.  Movement\n" +
						"is not instantanious when keen is on his pogo, instead he has momentum.  A\n" +
						"higher value will make give keen less momentum when he is on his pogo, and the\n" +
						"controls to move left or right will respond much more quickly.\n" +
						"Default = 1.5";
		movement[14] = new movementOption(1.5,"Pogo side accel", description, descriptionText);
		pogoMovement.add(movement[14]);
		
		description = "The maximum speed at which keen can move left or right when he is pogoing\n" +
						"A higher value means that keen will be able to move more quickly when on pogo\n" +
						"Default = 3";
		movement[15] = new movementOption(3.0,"Pogo max side speed", description, descriptionText);
		pogoMovement.add(movement[15]);

		movementPanel2.add(new JScrollPane(descriptionText));
		optionPane.addTab("Movement options 2",movementPanel2);

		//Construct other options
		JPanel OtherPanel = new JPanel();
		OtherPanel.setLayout(new BoxLayout(OtherPanel, BoxLayout.Y_AXIS));
		OtherPanel.setBorder(new TitledBorder("Other options"));

		debug = new JCheckBox("Show debug info");
		debug.setSelected(true);
		OtherPanel.add(debug);
		
		loadCustom = new JButton("Load custom level...");
		loadCustom.setMnemonic(KeyEvent.VK_L);
		loadCustom.addActionListener(this);
		OtherPanel.add(loadCustom);

		optionPane.addTab("Other options",OtherPanel);

		
		//Construct buttons
		Close = new JButton("Back");
		Close.setMnemonic(KeyEvent.VK_B);
		Close.addActionListener(this);
		add(Close,"South");
	}
	
	public KeenActorMovement getMovement()
	{
		KeenActorMovement newMovement = new KeenActorMovement();
		
		newMovement.WALK_SPEED = movement[0].getIntValue();
		newMovement.SCROLL_SPEED = movement[1].getIntValue();
		newMovement.GRIP_PAUSE_TIME = movement[2].getIntValue();

		newMovement.POLE_JUMP_SPEED = movement[3].getDoubleValue();
		newMovement.POLE_UP_SPEED = movement[4].getDoubleValue();
		newMovement.POLE_DOWN_SPEED = movement[5].getDoubleValue();

		newMovement.JUMP_SPEED = movement[6].getDoubleValue();
		newMovement.JUMP_DECCELERATION = movement[7].getDoubleValue();
		newMovement.JUMP_BOOST_DECCELERATION = movement[8].getDoubleValue();
		newMovement.SIDE_ACCELERATION = movement[9].getDoubleValue();
		
		newMovement.POGO_SPEED = movement[10].getDoubleValue();
		newMovement.POGO_BOOST_SPEED = movement[11].getDoubleValue();
		newMovement.POGO_DECCELERATION = movement[12].getDoubleValue();
		newMovement.POGO_BOOST_DECCELERATION = movement[13].getDoubleValue();
		newMovement.POGO_SIDE_ACCELERATION = movement[14].getDoubleValue();
		newMovement.POGO_MAX_SIDE_SPEED = movement[15].getDoubleValue();
		
		return newMovement;
	}
	
	public boolean getDebug()
	{
		return debug.isSelected();
	}
	
	private Object [] filterDisplayModes(DisplayMode [] modes)
	{
		Hashtable modeMap = new Hashtable();
		for(int i = 0; i < modes.length; i++)
		{
			DisplayModeWrapper wrapper = new DisplayModeWrapper(modes[i]);
			if(modeMap.contains(wrapper.toString()))
			{
				DisplayMode oldMode = ((DisplayModeWrapper)modeMap.get(wrapper.toString())).getMode();
				if(!((oldMode.getRefreshRate() == DisplayMode.REFRESH_RATE_UNKNOWN) //If its not unknown
					|| (modes[i].getRefreshRate() == DisplayMode.REFRESH_RATE_UNKNOWN)))
				{
					if(oldMode.getRefreshRate() < modes[i].getRefreshRate())
					{
						modeMap.put(wrapper.toString(), new DisplayModeWrapper(modes[i]));
					}
				}
			}
			else //It doenst contain this mode
			{
				modeMap.put(wrapper.toString(), new DisplayModeWrapper(modes[i]));
			}
		}
		
		//Impose some ordering
		Object [] sortArray = (Collections.list(modeMap.elements())).toArray();
		Arrays.sort(sortArray , new WrapperComparator());
		return sortArray;
	}
	
	public void actionPerformed(ActionEvent e)
	{
		if(e.getSource() == Close)
		{
			done(0);
		}
		else if(e.getSource() == SetFullScreen)
		{
			setFullScreen();
			requestFocus();
		}
		else if(e.getSource() == SetWindowed)
		{
			setWindowed();
			requestFocus();
		}
	    else if(e.getSource() == loadCustom)
        {
        	openLevel();
        }

	}
	
	public File getFile()
    {
    	return fileName;
    }
    
	private void openLevel()
	{
		if(fileSelector == null)
		{
			fileSelector = new JFileChooser();
			fileSelector.setFileFilter(new LevelFilter());
		}
		int retrunValue = fileSelector.showOpenDialog(this);
		if(retrunValue == JFileChooser.APPROVE_OPTION)
		{
			fileName = fileSelector.getSelectedFile();
		}
	}
	
	private void setFullScreen()
	{
		try
		{
			GraphicsDevice device = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice();
			DisplayMode mode = ((DisplayModeWrapper)ResolutionList.getSelectedValue()).getMode();

			if(!fullScreen)
			{
				hostApp.dispose();
				hostApp.setUndecorated(true);
				hostApp.setResizable(false);
				device.setFullScreenWindow(hostApp);
			}
			if(device.isDisplayChangeSupported())
			{
				device.setDisplayMode(mode);
				fullScreen = true;
			}
			SetWindowed.setEnabled(true);
			//mentPanel.setEnabled(false);
			//movementPanel2.setEnabled(false);
			hostApp.setVisible(true);
		}
		catch(Exception error)
		{
			error.printStackTrace();
			JOptionPane.showMessageDialog(this,error,"Error setting fullscreen mode", JOptionPane.WARNING_MESSAGE);
			System.exit(0);
		}

	}
	private void setWindowed()
	{
		GraphicsDevice device = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice();

		if(device.isDisplayChangeSupported())
		{
			device.setDisplayMode(originalMode);
		}
		device.setFullScreenWindow(null);

		hostApp.dispose();
		hostApp.setUndecorated(false);
		hostApp.setResizable(true);
		hostApp.setSize(Keen.DEFAULT_WIDTH, Keen.DEFAULT_HEIGHT);

		SetWindowed.setEnabled(false);
		//vementPanel.setEnabled(false);
		//vementPanel2.setEnabled(false);
		hostApp.setVisible(true);
		fullScreen = false;
	}
	
	public void valueChanged(ListSelectionEvent e)
	{
		SetFullScreen.setEnabled(true);
	}
	
	public boolean debug()
	{
		return debug.isSelected();
	}
	
	private class DisplayModeWrapper
	{
		private DisplayMode mode;
		
		public DisplayModeWrapper(DisplayMode mode)
		{
			this.mode = mode;
		}
			
		public String toString()
		{
			String stringRep = mode.getWidth() + " x " + mode.getHeight() + " x ";
			if(mode.getBitDepth() == DisplayMode.BIT_DEPTH_MULTI)
			{
				stringRep = stringRep + "Multi";
			}
			else
			{
				stringRep = stringRep + mode.getBitDepth() + "bit";
			}
			
			return  stringRep;
		}
		
		/**
		 * Returns the mode.
		 * @return DisplayMode
		 */
		public DisplayMode getMode()
		{
			return mode;
		}

		/**
		 * Sets the mode.
		 * @param mode The mode to set
		 */
		public void setMode(DisplayMode mode)
		{
			this.mode = mode;
		}

	}
	
	private class WrapperComparator implements Comparator
	{
		public int compare(Object obj1, Object obj2)
		{
			DisplayMode mode1 = ((DisplayModeWrapper)obj1).getMode();
			DisplayMode mode2 = ((DisplayModeWrapper)obj2).getMode();
			int result = 0;
			
			if(mode1.getWidth() != mode2.getWidth())
			{
				result = mode1.getWidth() - mode2.getWidth();
			}
			else //go to height
			{
				if(mode1.getHeight() != mode2.getHeight())
				{
					result = mode1.getHeight() - mode2.getHeight();
				}
				else //go to bits
				{
					if(mode1.getBitDepth() != mode2.getBitDepth())
					{
						result = mode1.getBitDepth() - mode2.getBitDepth();
					}
					else //go to refresh
					{
						result = mode1.getRefreshRate() - mode2.getRefreshRate();
					}
				}
			}
			
			return result;
		}
		
		public boolean equals(Object obj1)
		{
			return (this == obj1);
		}
	}
	
	private class movementOption extends JPanel implements ActionListener
	{
		JButton help = new JButton("?");
		JTextField val = new JTextField(7);
		JTextArea descriptionLabel;
		
		String description;
		public movementOption(double defaultVal, String title, String description, JTextArea label)
		{
			descriptionLabel = label;
			this.description = description;
			setBorder(new TitledBorder(title));
			val.setText(Double.toString(defaultVal));
			add(val);
			help.addActionListener(this);
			add(help);
		}
		
		public void actionPerformed(ActionEvent e)
		{
			if(e.getSource() == help)
			{
				descriptionLabel.setText(description);
				descriptionLabel.revalidate();
			}
		}
		
		public int getIntValue()
		{
			Double doubleVal = new Double(val.getText());
			return doubleVal.intValue();
		}
		
		public double getDoubleValue()
		{
			return Double.parseDouble(val.getText());
		}
	}
}
