package keenGame;

import java.awt.Rectangle;

/**
 * Manages regions of a level that need repainting, attempting
 * to minimise the ammount of redrawing that is needed.  Currenty
 * optimising of regions is not implemented.  eg If you tell the
 * repaint manager to redraw the same area twice, when you query
 * getDirtyRegions, the same area will be redrawn twice
 */
public class RepaintManager
{
	private final static int MAX_DIRTY_REGIONS = 500;

	private Rectangle[] dirtyArea;
	private Rectangle[] tempList;
	private Rectangle clipArea = new Rectangle();

	private int numRegions = 0;
	
	/**
	 * Constructs an empty RepaintManager
	 */
	public RepaintManager()
	{
		dirtyArea = new Rectangle[MAX_DIRTY_REGIONS];
		for (int i = 0; i < dirtyArea.length; i++)
		{
			dirtyArea[i] = new Rectangle();
		}	

		tempList = new Rectangle[MAX_DIRTY_REGIONS];
	}
	
	/**
	 * Adds a dirty region, to the list of regions that need redrawing
	 * Currently you can only have a maximum of 500 regions that can
	 * be redrawin, but ultimately the list should be implemented
	 * as a vector, so this limitation should be removed
	 * 
	 * @param x - The left corner of the dirty region
	 * @param y - The top corner of the dirty region
	 * @param width - The width of the dirty region
	 * @param height - The height of the dirty region
	 */
	public void addRegion(int x, int y, int width, int height)
	{
		dirtyArea[numRegions].x = x;
		dirtyArea[numRegions].y = y;
		dirtyArea[numRegions].width = width;
		dirtyArea[numRegions].height = height;
		numRegions++;
	}
	
	/**
	 * Clears the list of dirty regions
	 */
	public void clearRegions()
	{
		numRegions = 0;
	}
	
	/**
	 * Returns an array, containg Rectangles representing
	 * all the dirty regions, that fall within the specified area.
	 * 
	 * @param x - The left corner for the area to query for dirty regions
	 * @param y - The top corner for the area to query for dirty regions
	 * @param width - The width of the area to query for dirty regions
	 * @param height - The height of the area to query for dirty regions
	 */
	public Rectangle [] getDirtyRegions(int x, int y, int width, int height)
	{
		//Fill tempList with all matches
		int numMatches = 0;
		clipArea.x = x;
		clipArea.y = y;
		clipArea.height = height;
		clipArea.width = width;
		for (int i = 0; i < numRegions; i++)
		{
			if(dirtyArea[i].intersects(x, y, width, height))
			//upper left corner
				//lower right corner
				//
			
			
			/*
			((dirtyArea[i].x < (x + width)) &&
				(dirtyArea[i].y < ( y + height)) &&
				((dirtyArea[i].x + dirtyArea[i].width) > x) &&
				(dirtyArea[i].y + dirtyArea[i].height) > y) //If it is inside the area
							||
				(
				
				
				 //it is above or to the left)
							||
				( //it below or to the right))*/
			{
				tempList[numMatches] = dirtyArea[i];
				numMatches++;
			}
		}
		
		//Allocate a list 
		Rectangle [] returnList = new Rectangle[numMatches];
		
		//Copy across all in temp list
		for (int i = 0; i < returnList.length; i++)
		{
			returnList[i] = tempList[i];
		}
		return returnList;
	}
}
