package keenGame;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import keenGame.sprite.MovingActor;
import keenGame.sprite.Sprite;
import keenGame.sprite.hint.DirectionHint;
import keenGame.sprite.hint.LeftHint;
import keenGame.sprite.hint.RightHint;
import keenGame.sprite.keenActor.KeenActor;
import keenGame.sprite.monster.BirdMonster;
import keenGame.sprite.platform.BasicPlatform;
import keenGame.sprite.tile.AutoRectangleTile;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * XMLLevelLoader loads up all a level from a plain text file
 * Currently is supports lvl formation version 0.1
 * 
 * TODO: totally redo this so it doesnt implement iterator, and instead
 * make it return a list
 */
public class XMLLevelLoader extends LevelLoader
{
	//private int lineNumber = 0;
	private Level loadLevel;
	//private Document levelFile;

	public XMLLevelLoader(InputStream levelStream, Level loadLevel) throws IOException
	{
		super();
		this.loadLevel = loadLevel;
		Element levelRoot = getRootElement(levelStream);
		readLevel(levelRoot);
		readSprites(getElement(levelRoot, "sprites"));
	}
	
	private void readSprites(Element spriteRoot)
	{
		//Get a list of the elements
		List sprites = getChildElements(spriteRoot);
		
		for (Iterator iter = sprites.iterator(); iter.hasNext();)
		{
			Element currentElement = (Element)iter.next();
			spriteList.add(elementToSprite(currentElement));
		}
	}
	
	private Sprite elementToSprite(Element xml)
	{
		Sprite toReturn = null;
		if(xml.getTagName().equals("AutoRectangleTile"))
		{
			toReturn = getAutoRectangleTile(xml);
		}
		else if(xml.getTagName().equals("DirectionHint"))
		{
			toReturn = getDirectionHint(xml);
		}
		else if(xml.getTagName().equals("BasicPlatform"))
		{
			toReturn = getBasicPlatform(xml);
		}
		else if(xml.getTagName().equals("LeftHint"))
		{
			toReturn = getLeftHint(xml);
		}
		else if(xml.getTagName().equals("RightHint"))
		{
			toReturn = getRightHint(xml);
		}
		else if(xml.getTagName().equals("BirdMonster"))
		{
			toReturn = getBirdMonster(xml);
		}
		else
		{
			System.err.println("Unresolved sprite type:" + xml.getTagName());
		}

		return toReturn;
	}

	private KeenActor getKeenActor(Element xml)
	{
		int x = getNodeValue(getElement(xml, "x"));
		int y = getNodeValue(getElement(xml, "y"));
		String layer = getNodeText(getElement(xml,"spriteLayer"));
		String [] hitLayers =  getLayers(getElement(xml, "hitLayers"));

		String blastLayer = getNodeText(getElement(xml,"blastLayer"));
		String [] blastHitLayers =  getLayers(getElement(xml, "blastHitLayers"));

		
		KeenActor keen = new KeenActor(layer, hitLayers, blastLayer, blastHitLayers,
						loadLevel, x,y);
		return keen;
	}


	private Sprite getBirdMonster(Element xml)
	{
		int x = getNodeValue(getElement(xml, "x"));
		int y = getNodeValue(getElement(xml, "y"));
		String layer = getNodeText(getElement(xml,"spriteLayer"));
		String [] hitLayers =  getLayers(getElement(xml, "hitLayers"));
		
		return new BirdMonster(layer, hitLayers, loadLevel, x, y);
	}

	
	private Sprite getAutoRectangleTile(Element xml)
	{
		int x = getNodeValue(getElement(xml, "x"));
		int y = getNodeValue(getElement(xml, "y"));
		String layer = getNodeText(getElement(xml,"spriteLayer"));
		String graphic = getNodeText(getElement(xml,"graphic"));
		
		boolean canHit = false;
		
		if(xml.getAttribute("canHit").equals("true"))
		{
			canHit = true;
		}

		return new AutoRectangleTile(graphic, layer, x, y, canHit);
	}
	
	private Sprite getDirectionHint(Element xml)
	{
		int x = getNodeValue(getElement(xml, "x"));
		int y = getNodeValue(getElement(xml, "y"));
		String layer = getNodeText(getElement(xml,"spriteLayer"));
		int direction = parseDirection( xml.getAttribute("direction"));
		
		return new DirectionHint(layer, x, y, direction);
	}

	private Sprite getLeftHint(Element xml)
	{
		int x = getNodeValue(getElement(xml, "x"));
		int y = getNodeValue(getElement(xml, "y"));
		String layer = getNodeText(getElement(xml,"spriteLayer"));
		
		return new LeftHint(layer, x, y);
	}

	private Sprite getRightHint(Element xml)
	{
		int x = getNodeValue(getElement(xml, "x"));
		int y = getNodeValue(getElement(xml, "y"));
		String layer = getNodeText(getElement(xml,"spriteLayer"));
		
		return new RightHint(layer, x, y);
	}


	private Sprite getBasicPlatform(Element xml)
	{
		int x = getNodeValue(getElement(xml, "x"));
		int y = getNodeValue(getElement(xml, "y"));
		String layer = getNodeText(getElement(xml,"spriteLayer"));
		String graphic = getNodeText(getElement(xml,"graphic"));
		String []  hitLayers = getLayers(getElement(xml, "hitLayers"));
		int direction = parseDirection( xml.getAttribute("direction"));
		
		return new BasicPlatform(graphic,layer, hitLayers, loadLevel, x, y, direction);
	}

	
	private int parseDirection(String directionString)
	{
		int direction = MovingActor.NONE;
		if(directionString.equals("north"))
		{
			direction = MovingActor.NORTH;
		}
		else if(directionString.equals("north east"))
		{
			direction = MovingActor.NORTH_EAST;
		}
		else if(directionString.equals("east"))
		{
			direction = MovingActor.EAST;
		}
		else if(directionString.equals("south east"))
		{
			direction = MovingActor.SOUTH_EAST;
		}
		else if(directionString.equals("south"))
		{
			direction = MovingActor.SOUTH;
		}
		else if(directionString.equals("south west"))
		{
			direction = MovingActor.SOUTH_WEST;
		}
		else if(directionString.equals("west"))
		{
			direction = MovingActor.WEST;
		}
		else if(directionString.equals("north west"))
		{
			direction = MovingActor.NORTH_WEST;
		}
		return direction;
	}					
	
	private void readLevel(Element level)
	{
		width = Integer.parseInt(level.getAttribute("width"));
		height = Integer.parseInt(level.getAttribute("width"));
		
		this.layers = getLayers(getElement(level, "layers"));
		
		//Element layersElement = getElement(level, "layers");

		//parse layers
		//NodeList layers = layersElement.getElementsByTagName("layer");
			
		//this.layers = new String [layers.getLength()];
		//for(int i = 0; i < layers.getLength(); i++)
		//{
		//	this.layers[i] = getNodeText(layers.item(i));
		//}
		
		//Get keen
		keen = getKeenActor(getElement(getElement(level,"player"),"KeenActor"));
	}
	
	private String [] getLayers(Element layersElement)
	{
		//parse layers
		NodeList layers = layersElement.getElementsByTagName("layer");
			
		String [] layersArray = new String [layers.getLength()];
		for(int i = 0; i < layers.getLength(); i++)
		{
			layersArray[i] = getNodeText(layers.item(i));
		}
		return layersArray;
	}		
	
	private List getChildElements(Element parent)
	{
		LinkedList children = new LinkedList();
		//get list of children
		NodeList childrenNodes = parent.getChildNodes();
		
		for(int i = 0; i < childrenNodes.getLength(); i++)
		{
			Node child = childrenNodes.item(i);
			if(child instanceof Element)
			{
				children.add(child);
			}
		}
		return children;
	}
	
	/**
	 * Gets a child Element of the specified parent Element
	 */
	private Element getElement(Element parent, String name)
	{
		NodeList list = parent.getElementsByTagName(name);
		
		if(list.getLength() != 1)
		{
			System.err.println("Error, element " + parent + " has more than one child named " + name);
			System.exit(0);
		}
		
		return (Element)list.item(0);
	}
	
	private int getNodeValue(Node target)
	{
		return Integer.parseInt(getNodeText(target));
	}
	
	/**
	 * Gets the text of the specified target node
	 */
	private String getNodeText(Node target)
	{
		StringBuffer tempString = new StringBuffer();
		NodeList children = target.getChildNodes();
		for(int i = 0; i < children.getLength(); i++)
		{
			Node child = children.item(i);
			if (child instanceof Text)
			{
 				tempString.append( child.getNodeValue() );
			}
		}
		return tempString.toString();
	}

		
	
	private Element getRootElement(InputStream levelStream)
	{
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setValidating(false);
		try
		{
			DocumentBuilder builder = factory.newDocumentBuilder();
			
			builder.setErrorHandler(
				new org.xml.sax.ErrorHandler()
				{
					// ignore fatal errors (an exception is guaranteed)
					public void fatalError(SAXParseException exception)
					throws SAXException {}
		
					// treat validation errors as fatal
					public void error(SAXParseException e)
					throws SAXParseException
					{
						throw e;
					}
		
					// dump warnings too
					public void warning(SAXParseException err)
					throws SAXParseException
					{
						System.out.println("** Warning"
							+ ", line " + err.getLineNumber()
							+ ", uri " + err.getSystemId());
						System.out.println("   " + err.getMessage());
					}
				}
			); 
			Document doc = builder.parse(levelStream);
			return doc.getDocumentElement();
	    }
	    catch (SAXException sxe)
	    {
			// Error generated during parsing
			Exception  x = sxe;
			if (sxe.getException() != null)
			{
				x = sxe.getException();
				x.printStackTrace();
				System.exit(0);
			}
			else
			{
				sxe.printStackTrace();
				System.exit(0);
			}				
	    }
	    catch (ParserConfigurationException pce)
	    {
			// Parser with specified options can't be built
			pce.printStackTrace();
			System.exit(0);

		}
		catch (IOException ioe)
		{
			// I/O error
			ioe.printStackTrace();
			System.exit(0);
	    }
	    
	    return null;
	}
}
