package keenGame;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.io.IOException;
import java.io.InputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import keenGame.sprite.Sprite;
import keenGame.sprite.candy.PointsCandy;
import keenGame.sprite.hint.DirectionHint;
import keenGame.sprite.hint.LeftHint;
import keenGame.sprite.hint.RightHint;
import keenGame.sprite.keenActor.KeenActor;
import keenGame.sprite.monster.BirdMonster;
import keenGame.sprite.platform.BasicPlatform;
import keenGame.sprite.tile.AutoRectangleTile;
import keenGame.sprite.tile.Exit;
import keenGame.sprite.tile.Pole;
import keenGame.sprite.tile.PoleBottom;
import keenGame.sprite.tile.PoleTop;
import keenGame.sprite.tile.PolygonTile;
import keenGame.sprite.tile.RectangleTile;
import keenGame.sprite.tile.ThinFloor;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * @author Luke Sleeman
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class XMLLevelLoader2 extends LevelLoader
{
	private Level loadLevel;

	public XMLLevelLoader2(InputStream levelStream, Level loadLevel) throws IOException
	{
		super();
		this.loadLevel = loadLevel;
		XmlElement levelRoot = getRootElement(levelStream);
		readLevel(levelRoot);
		readSprites(levelRoot.getChild("sprites"));
	}
	
	private void readLevel(XmlElement levelRoot)
	{
		//get width and height
		width = Integer.parseInt(levelRoot.getAttribute("width"));
		height = Integer.parseInt(levelRoot.getAttribute("height"));
		
		//get layers
		XmlElement [] layerElements = levelRoot.getChild("layers").getChildren();
		layers = new String [layerElements.length];
		for(int i = 0; i < layers.length; i++)
		{
			layers[i] = layerElements[i].getText();
		}
		
		//get keenActor
		keen = new KeenActor(levelRoot.getChild("player").getChild("KeenActor"), loadLevel);
	}
	
	private void readSprites(XmlElement spriteRoot)
	{
		XmlElement [] children = spriteRoot.getChildren();
		
		for(int i = 0; i < children.length; i++)
		{
			spriteList.add(xmlToSprite(children[i]));
		}
	}

	private Sprite xmlToSprite(XmlElement spriteElement)
	{
		Sprite toReturn = null;
		if(spriteElement.getName().equals("AutoRectangleTile"))
		{
			toReturn = new AutoRectangleTile(spriteElement);
		}
		else if(spriteElement.getName().equals("DirectionHint"))
		{
			toReturn = new DirectionHint(spriteElement);
		}
		else if(spriteElement.getName().equals("BasicPlatform"))
		{
			toReturn = new BasicPlatform(spriteElement, loadLevel);
		}
		else if(spriteElement.getName().equals("LeftHint"))
		{
			toReturn = new LeftHint(spriteElement);
		}
		else if(spriteElement.getName().equals("RightHint"))
		{
			toReturn = new RightHint(spriteElement);
		}
		else if(spriteElement.getName().equals("BirdMonster"))
		{
			toReturn = new BirdMonster(spriteElement, loadLevel);
		}
		else if(spriteElement.getName().equals("Pole"))
		{
			toReturn = new Pole(spriteElement);
		}
		else if(spriteElement.getName().equals("PoleBottom"))
		{
			toReturn = new PoleBottom(spriteElement);
		}
		else if(spriteElement.getName().equals("PoleTop"))
		{
			toReturn = new PoleTop(spriteElement);
		}
		else if(spriteElement.getName().equals("PolygonTile"))
		{
			toReturn = new PolygonTile(spriteElement);
		}
		else if(spriteElement.getName().equals("RectangleTile"))
		{
			toReturn = new RectangleTile(spriteElement);
		}
		else if(spriteElement.getName().equals("ThinFloor"))
		{
			toReturn = new ThinFloor(spriteElement);
		}
		else if(spriteElement.getName().equals("PointsCandy"))
		{
			toReturn = new PointsCandy(spriteElement, loadLevel);
		}
		else if(spriteElement.getName().equals("Exit"))
		{
			toReturn = new Exit(spriteElement);
		}
		else
		{
			System.err.println("Unresolved sprite type:" + spriteElement.getName());
		}

		return toReturn;
	}

	private XmlElement getRootElement(InputStream levelStream)
	{
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setValidating(false);
		try
		{
			DocumentBuilder builder = factory.newDocumentBuilder();
			
			builder.setErrorHandler(
				new org.xml.sax.ErrorHandler()
				{
					// ignore fatal errors (an exception is guaranteed)
					public void fatalError(SAXParseException exception)
					throws SAXException {}
		
					// treat validation errors as fatal
					public void error(SAXParseException e)
					throws SAXParseException
					{
						throw e;
					}
		
					// dump warnings too
					public void warning(SAXParseException err)
					throws SAXParseException
					{
						System.out.println("** Warning"
							+ ", line " + err.getLineNumber()
							+ ", uri " + err.getSystemId());
						System.out.println("   " + err.getMessage());
					}
				}
			); 
			return new XmlElement(builder.parse(levelStream).getDocumentElement());
	    }
	    catch (SAXException sxe)
	    {
			// Error generated during parsing
			Exception  x = sxe;
			if (sxe.getException() != null)
			{
				x = sxe.getException();
				x.printStackTrace();
				System.exit(0);
			}
			else
			{
				sxe.printStackTrace();
				System.exit(0);
			}				
	    }
	    catch (ParserConfigurationException pce)
	    {
			// Parser with specified options can't be built
			pce.printStackTrace();
			System.exit(0);

		}
		catch (IOException ioe)
		{
			// I/O error
			ioe.printStackTrace();
			System.exit(0);
	    }
		return null;
	}
}
