package keenGame.editor;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Frame;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsEnvironment;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JToolBar;
import javax.swing.SpinnerNumberModel;

import keenGame.Cache;
import keenGame.LevelFilter;
import keenGame.XMLLevelSaver;

/**
 * @author luke
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class IQ extends JFrame implements ActionListener
{
	private LevelPanel editLevel;
	private LayerEdit layerEditor;
	private ToolPanel toolBox;
	private IQMenuBar menu;
	private IQToolBar toolBar;
	
	private JFileChooser fileSelector;
	private File fileName;
	private NewOptions options;

	public IQ()
	{
		super("IQ editor");
 		setDefaultCloseOperation(EXIT_ON_CLOSE);
 		setSize(900,600);
 		
		GraphicsConfiguration gConf = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration();
		Cache.setGraphicsConfig(gConf);
 		Cache.setClassLoader(getClass().getClassLoader());
 		
 		getContentPane().setLayout(new BorderLayout());
 		
 		//Add in levelPanel
 		editLevel = new LevelPanel();
 		getContentPane().add(editLevel, BorderLayout.CENTER);
 		
 		//ToolBox to the west
 		toolBox = new ToolPanel((LevelPanel)editLevel);
 		getContentPane().add(toolBox, BorderLayout.WEST);
 		
 		//ToolBar to the north
 		toolBar = new IQToolBar();
 		getContentPane().add(toolBar, BorderLayout.NORTH); 	
 		toolBar.zoom.addActionListener(this);
 		toolBar.gridSize.addActionListener(this);
 		
 		//The layer editor
 		layerEditor = new LayerEdit(this, (LevelPanel)editLevel);
 		layerEditor.addWindowListener(new crappyWindowAdaptor());
 		layerEditor.addLayerChangeListener(toolBox);
 		layerEditor.setVisible(false);

 		menu = new IQMenuBar();
 		setJMenuBar(menu);
 		
 		menu.exit.addActionListener(this);
 		menu.newLevel.addActionListener(this);
 		menu.openLevel.addActionListener(this);
 		menu.saveLevel.addActionListener(this);
 		menu.saveLevelAs.addActionListener(this);
 		menu.showLayers.addActionListener(this);
	 	menu.showGrid.addActionListener(this);
	 	menu.snapToGrid.addActionListener(this);
	}
	
	public void actionPerformed(ActionEvent e)
	{
		if(e.getSource() == menu.exit)
		{
			System.exit(0);
		}
		else if(e.getSource() == menu.saveLevel)
		{
			saveLevel();
		}
		else if(e.getSource() == menu.saveLevelAs)
		{
			saveLevelAs();
		}
		else if(e.getSource() == menu.showGrid)
		{
			editLevel.setGridVisible(menu.showGrid.isSelected());
		}
		else if(e.getSource() == menu.snapToGrid)
		{
			editLevel.setSnapToGrid(menu.snapToGrid.isSelected());
		}
		else if(e.getSource() == menu.showLayers)
		{
			layerEditor.setVisible(menu.showLayers.isSelected());
		}
		else if(e.getSource() == menu.openLevel)
		{
			openLevel();
		}
		else if(e.getSource() == menu.newLevel)
		{
			newLevel();
		}
		else if(e.getSource() == toolBar.zoom)
		{
			editLevel.setScale(Double.parseDouble((String)toolBar.zoom.getSelectedItem()));
		}
		else if(e.getSource() == toolBar.gridSize)
		{
			editLevel.setGridSize(Integer.parseInt((String)toolBar.gridSize.getSelectedItem()));
		}
	}
	
	private void saveLevel()
	{
		if(fileName == null)
		{
			saveLevelAs();
		}
		else
		{
			try
			{
				FileOutputStream file = new FileOutputStream(fileName);
				XMLLevelSaver.saveLevel(file, editLevel.getLevel(), editLevel.getKeen());
			}
			catch(IOException e)
			{
				System.err.println(e);
			}
		}
	}
	
	private void saveLevelAs()
	{
		if(fileSelector == null)
		{
			fileSelector = new JFileChooser();
			fileSelector.setFileFilter(new LevelFilter());
		}
		int retrunValue = fileSelector.showSaveDialog(this);
		if(retrunValue == JFileChooser.APPROVE_OPTION)
		{
			fileName = fileSelector.getSelectedFile();
			if(!((fileName.getName().endsWith(".xml")) ||
					(fileName.getName().endsWith(".XML"))))
			{
				fileName = new File(fileName + ".xml");
			}
			
			saveLevel();
			menu.saveLevel.setEnabled(true);
		}
	}
	
	private void openLevel()
	{
		if(fileSelector == null)
		{
			fileSelector = new JFileChooser();
			fileSelector.setFileFilter(new LevelFilter());
		}
		int retrunValue = fileSelector.showOpenDialog(this);
		if(retrunValue == JFileChooser.APPROVE_OPTION)
		{
			fileName = fileSelector.getSelectedFile();
			editLevel.loadLevel(fileName);
			menu.saveLevel.setEnabled(true);
		}
	}
	
	private void newLevel()
	{
		if(options == null)
		{
			options = new NewOptions(this);
		}
		options.setVisible(true);
		
		if(options.createLevel())
		{
			editLevel.newLevel(options.getWidth(),options.getHeight());
			fileName = null;
			menu.saveLevel.setEnabled(false);
		}
	}
	
	private class crappyWindowAdaptor extends WindowAdapter
	{
		public void windowClosed(WindowEvent e)
		{
			System.err.println("blah");
		}
	}		

	public static void main(String[] args)
	{
		IQ editor = new IQ();
		editor.setVisible(true);
	}
	
	private class IQToolBar extends JToolBar
	{
		public JComboBox zoom;
		public JComboBox gridSize;
		
		public IQToolBar()
		{
			super();
			setFloatable(false);
			
			String [] zoomLevels = {"1.0","2.0","3.0","4.0"};
			String [] gridSizes = {"6","12","5","10","15","20"};
			
			add(new JLabel("Zoom factor"));
			zoom = new JComboBox(zoomLevels);
			zoom.setSelectedIndex(0);
			add(zoom);
			
			add(new JLabel("Grid size"));
			gridSize = new JComboBox(gridSizes);
			gridSize.setSelectedIndex(5);
			add(gridSize);

		}
	}
		
	
	private class IQMenuBar extends JMenuBar
	{
		public JMenuItem exit;
		public JMenuItem newLevel;
		public JMenuItem saveLevel;
		public JMenuItem saveLevelAs;
		public JMenuItem openLevel;
		public JMenuItem about;
		public JCheckBoxMenuItem showLayers;
		public JCheckBoxMenuItem showGrid;
		public JCheckBoxMenuItem snapToGrid;
		
		public IQMenuBar()
		{
			super();
			
	 		JMenu fileMenu = new JMenu("File");		
	 		newLevel = new JMenuItem("New Level");
	 		fileMenu.add(newLevel);
	 		openLevel = new JMenuItem("Open ...");
	 		fileMenu.add(openLevel);
	  		saveLevel = new JMenuItem("Save");
			saveLevel.setEnabled(false);
	 		fileMenu.add(saveLevel);
	  		saveLevelAs = new JMenuItem("Save As...");
	 		fileMenu.add(saveLevelAs);
	  		fileMenu.addSeparator();
	  		exit = new JMenuItem("Exit");
	 		fileMenu.add(exit);
	 		
	 		add(fileMenu);
	 		
	 		JMenu viewMenu = new JMenu("View");
	 		showLayers = new JCheckBoxMenuItem("show layer dialog",false);

	 		viewMenu.add(showLayers);
	 		showGrid = new JCheckBoxMenuItem("show grid",false);

	 		viewMenu.add(showGrid);
	 		snapToGrid = new JCheckBoxMenuItem("snap to grid",false);

	 		viewMenu.add(snapToGrid);
	 		add(viewMenu);
	 		
	 		JMenu helpMenu = new JMenu("Help");
	 		about = new JMenuItem("about");
	 		helpMenu.add(about);
	 		add(helpMenu);
		}
			
	}
	
	private class NewOptions extends JDialog implements ActionListener
	{
		public JSpinner width, height;
		public JButton ok, cancel;
		
		private boolean createLevel = false;
		
		public NewOptions(Frame owner)
		{
			super(owner, "Create new level", true);
			
			Container cp = getContentPane();
			cp.setLayout(new BoxLayout(cp , BoxLayout.Y_AXIS));
			cp.add(new JLabel("Select width and height"));
			
			JPanel numPanel = new JPanel(new GridLayout(2,2));
			numPanel.add(new JLabel("Width:"));
			width = new JSpinner();
			width.setValue(new Integer(LevelPanel.DEFAULT_LEVEL_WIDTH));
			((SpinnerNumberModel)width.getModel()).setMinimum(new Integer(100));
			numPanel.add(width);			
			numPanel.add(new JLabel("Height:"));
			height = new JSpinner();
			height.setValue(new Integer(LevelPanel.DEFAULT_LEVEL_HEIGHT));
			((SpinnerNumberModel)height.getModel()).setMinimum(new Integer(100));
			numPanel.add(height);
			cp.add(numPanel);
			
			JPanel buttonPanel = new JPanel();
			ok = new JButton("Ok");
			ok.addActionListener(this);
			buttonPanel.add(ok);

			cancel = new JButton("Cancel");
			cancel.addActionListener(this);
			buttonPanel.add(cancel);
			cp.add(buttonPanel);
			
			setSize(250,180);
		}
		
		public int getWidth()
		{
			return ((Integer)width.getValue()).intValue();
		}

		public int getHeight()
		{
			return ((Integer)height.getValue()).intValue();
		}
		
		public boolean createLevel()
		{
			return createLevel;
		}
		
		public void actionPerformed(ActionEvent e)
		{
			if(e.getSource() == ok)
			{
				createLevel = true;
				setVisible(false);
			}
			else if(e.getSource() == cancel)
			{
				createLevel = false;
				setVisible(false);
			}
		}
	}
}
