package keenGame.editor;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.AbstractTableModel;


public class LayerEdit extends JDialog implements ActionListener, TableModelListener
{
	private LevelPanel levelWrapper;
	private JCheckBox autoLayers;
	private JTable layerTable;
	private LayerDataModel layerData;
	private JButton newLayer, layerUp, layerDown, deleteLayer, renameLayer;
	
	private Vector layerListeners = new Vector();
	
	private int numNewLayers = 0;
	private boolean hasWarned = false;
	
	public LayerEdit(Frame owner, LevelPanel levelWrapper)
	{
		super(owner, "layers");
		this.levelWrapper = levelWrapper;
		
		getContentPane().setLayout(new BoxLayout(getContentPane(),BoxLayout.Y_AXIS));
		
		//AutoLayers checkbox
		autoLayers = new JCheckBox("Automatically Manage Layers", false);
		autoLayers.addActionListener(this);
		autoLayers.setSelected(true);
		getContentPane().add(autoLayers);
		
		//Control buttons
		JPanel buttons = new JPanel();
		newLayer = new JButton("new");
		newLayer.addActionListener(this);
		newLayer.setEnabled(false);
		buttons.add(newLayer);
		
		renameLayer = new JButton("rename");
		renameLayer.addActionListener(this);
		renameLayer.setEnabled(false);
		buttons.add(renameLayer);

		deleteLayer = new JButton("delete");
		deleteLayer.addActionListener(this);
		deleteLayer.setEnabled(false);
		buttons.add(deleteLayer);
		
		layerUp = new JButton("<");
		layerUp.addActionListener(this);
		layerUp.setEnabled(false);
		buttons.add(layerUp);
		
		layerDown = new JButton(">");
		layerDown.addActionListener(this);
		layerDown.setEnabled(false);
		buttons.add(layerDown);	
		
		getContentPane().add(buttons);					
		
		String [] layers = levelWrapper.getLevel().getLayers();
		
		layerData = new LayerDataModel(layers);
		layerData.addTableModelListener(this);
		layerTable = new JTable(layerData);
		layerTable.setColumnSelectionAllowed(false);
		layerTable.setRowSelectionAllowed(true);
		layerTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		layerTable.setShowVerticalLines(false);
		layerTable.setDragEnabled(true);
		
		getContentPane().add(new JScrollPane(layerTable));
		
		
		setSize(500,250);
		setLocation(500,200);
	}
	
	public void tableChanged(TableModelEvent e)
	{
		Vector visibleLayers = new Vector();
		for(int i = 0; i < layerData.visible.length; i++)
		{
			if(layerData.visible[i])
			{
				visibleLayers.add(layerData.layers[i]);
			}
		}
		
		levelWrapper.setVisibleLayers((String[]) visibleLayers.toArray(new String[visibleLayers.size()]));
		
		fireLayersChanged();
	}
	
	private void fireLayersChanged()
	{
		boolean auto = autoLayers.isSelected();
		
		//notify our listeners
		Iterator it = layerListeners.iterator();
		while (it.hasNext())
		{
			((LayerChangeListener)it.next()).layerChanged(layerData.layers, auto);
		}
	}		
	
	public void addLayerChangeListener(LayerChangeListener newListener)
	{
		layerListeners.add(newListener);
	}
	
	public void actionPerformed(ActionEvent e)
	{
		if(e.getSource() == autoLayers)
		{
			if(!autoLayers.isSelected())
			{
				if(!hasWarned)
				{
					JOptionPane.showMessageDialog(this, 
						"Warning:  By manualy mananging layers you can easily cause\n" +
						"crashes, and generally mess things up.  Furthermore, there is\n" +
						"almost no reason to manage layers manualy.  The editor can handle\n" +
						"almost all situations\n", "Warning" , JOptionPane.WARNING_MESSAGE);
					hasWarned = true;
				}
				newLayer.setEnabled(true);
				newLayer.setEnabled(true);
				layerUp.setEnabled(true);
				layerDown.setEnabled(true);
				deleteLayer.setEnabled(true);
				renameLayer.setEnabled(true);
			}
			else
			{
				newLayer.setEnabled(false);
				layerUp.setEnabled(false);
				layerDown.setEnabled(false);
				deleteLayer.setEnabled(false);
				renameLayer.setEnabled(false);
			}
			fireLayersChanged();
		}
		else if(e.getSource() == newLayer)
		{
			layerData.newLayer();
		}
		else if(e.getSource() == renameLayer)
		{
			layerData.renameLayer();
		}
		else if(e.getSource() == deleteLayer)
		{
			layerData.deleteLayer();
		}
		else if(e.getSource() == layerUp)
		{
			layerData.layerUp();
		}
		else if(e.getSource() == layerDown)
		{
			layerData.layerDown();
		}
	}
	
	private int findLayer(String name)
	{
		boolean found = false;
		int index = -1;
		for(int i = 0; (i < layerData.layers.length) && !found; i++)
		{
			if(layerData.layers[i].equals(name))
			{
				found = true;
				index = i;
			}
		}
		return index;
	}
	
	private class LayerDataModel extends AbstractTableModel
	{
		String [] layers;
		boolean [] visible;
		
		LayerDataModel(String [] layers)
		{
			this.layers = layers;
			visible = new boolean[layers.length];
			
			for (int i = 0; i < visible.length; i++)
			{
				visible[i] = true;
			}
		}
		
		private void newLayer()
		{
			String newName = "Layer " + numNewLayers ;
			numNewLayers++;
			
			levelWrapper.getLevel().addLayer(newName);
			
			//Add it into the layerData
			
			//String [] newLayers = new String[layers.length + 1];
			boolean [] newVisible = new boolean[visible.length + 1];
		
			//copy old layers, and visibility
			for(int i = 0; i < layers.length; i++)
			{
				//newLayers[i] = layers[i];
				newVisible[i] = visible[i];
			}
		
			//newLayers[newLayers.length - 1] = newName;
			newVisible[newVisible.length -1] = true;
			visible = newVisible;
			layers = levelWrapper.getLevel().getLayers();
			
			
			fireTableRowsInserted(layers.length - 1, layers.length - 1);
			layerTable.getSelectionModel().setSelectionInterval(layers.length - 1, layers.length - 1);

		}
		
		public void renameLayer()
		{
			//System.err.println("Renaming layer " + layerTable.getSelectedRow());
			int index = layerTable.getSelectedRow();
			if (index == -1)
			{
				index = 0;
			}
			
			String oldName = layers[index];
			String newName = JOptionPane.showInputDialog("Enter new name", oldName);
			
			if(newName != null)
			{
				//Search for duplicates
				boolean duplicate = false;
				for(int i = 0; (i < layers.length) && !duplicate; i++)
				{
					if(layers[i].equals(newName))
					{
						duplicate = true;
					}
				}
				
				if(duplicate)
				{
					JOptionPane.showMessageDialog(null,"That name is already in use!");
				}
				else
				{
					levelWrapper.getLevel().renameLayer(oldName,newName);

					fireTableDataChanged();
					layerTable.getSelectionModel().setSelectionInterval(index,index);
				}
			}
		}
		
		public void deleteLayer()
		{
			int index = layerTable.getSelectedRow();
			if(index != -1)
			{
				levelWrapper.getLevel().removeLayer(layers[index]);
				
				
			
				//Add it into the layerData
				//String [] newLayers = new String[layers.length - 1];
				boolean [] newVisible = new boolean[visible.length - 1];
		
				//copy old layers, and visibility
				int numLayers = 0;
				for(int i = 0; i < layers.length; i++)
				{
					if(i != index)
					{
						//newLayers[numLayers] = layers[i];
						newVisible[numLayers] = visible[i];
						numLayers++;
					}
				}
			
				visible = newVisible;
				layers = levelWrapper.getLevel().getLayers();
				
				
				fireTableRowsDeleted(index,index);
				layerTable.getSelectionModel().setSelectionInterval(index,index);

			}
		}
		
		public void layerUp()
		{
											
			int index = layerTable.getSelectedRow();

			if((index != -1) && (index != 0))
			{
				levelWrapper.getLevel().swapLayers(layers[index], layers[index - 1]);
				
				
				//String tempLayer = layers[index];
				boolean tempVisible = visible[index];
				
				//layers[index] = layers[index - 1];
				visible[index] = visible[index - 1];
				
				//layers[index - 1] = tempLayer;
				visible[index - 1] = tempVisible;
				
				
				fireTableDataChanged();
				
				layerTable.getSelectionModel().setSelectionInterval(index-1,index-1);
			}
		}
		
		public void layerDown()
		{
	
			int index = layerTable.getSelectedRow();

			if((index != -1) && (index != layers.length - 1))
			{
				levelWrapper.getLevel().swapLayers(layers[index], layers[index + 1]);
				
				
				//String tempLayer = layers[index];
				boolean tempVisible = visible[index];
				
				//layers[index] = layers[index + 1];
				visible[index] = visible[index + 1];
				
				//layers[index + 1] = tempLayer;
				visible[index + 1] = tempVisible;
				
				
				fireTableDataChanged();
				layerTable.getSelectionModel().setSelectionInterval(index+1,index+1);

			}
		}
		
		public int getColumnCount() { return 2; }
		
		public int getRowCount() { return layers.length;}
		
		public Object getValueAt(int row, int col)
		{
			if(col == 0)
			{
				return new Boolean(visible[row]);
			}
			else
			{
				return layers[row];
			}
		}
		
		public Class getColumnClass(int c)
		{
			return getValueAt(0, c).getClass();
		}
		
		public boolean isCellEditable(int row, int col)
		{
			return (col == 0);
		}
		
		public void setValueAt(Object value, int row, int col)
		{
			if(col == 0)
			{
				visible[row] = ((Boolean)value).booleanValue();
				fireTableDataChanged();
			}
		}
		
		public String getColumnName(int col)
		{
			if(col == 0)
			{
				return "visible";
			}
			else
			{
				return "name";
			}
		}

	}
}
