package keenGame.editor.tools;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.io.File;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListCellRenderer;
import javax.swing.OverlayLayout;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import keenGame.ArrayMap;
import keenGame.Cache;
import keenGame.editor.LayerChangeListener;
import keenGame.editor.LevelPanel;
import keenGame.editor.tools.spriteCreators.AutoRectangleCreator;
import keenGame.editor.tools.spriteCreators.BasicPlatformCreator;
import keenGame.editor.tools.spriteCreators.BirdMonsterCreator;
import keenGame.editor.tools.spriteCreators.CandyCreator;
import keenGame.editor.tools.spriteCreators.DirectionHintCreator;
import keenGame.editor.tools.spriteCreators.ExitCreator;
import keenGame.editor.tools.spriteCreators.LeftHintCreator;
import keenGame.editor.tools.spriteCreators.PoleCreator;
import keenGame.editor.tools.spriteCreators.PolygonCreator;
import keenGame.editor.tools.spriteCreators.RectangleCreator;
import keenGame.editor.tools.spriteCreators.RightHintCreator;
import keenGame.editor.tools.spriteCreators.SpriteCreator;
import keenGame.editor.tools.spriteCreators.ThinFloorCreator;
import keenGame.sprite.MovingActor;
import keenGame.sprite.Sprite;

/**
 * @author Luke Sleeman
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class InsertTool extends Tool
{
	private int width = 0;
	private int height = 0;
	private int oldX, oldY;
	
	public InsertTool(LevelPanel level)
	{
		super("Insert", level);
		control = new InsertControl(levelWrapper.getLevel().getLayers());
	}
	
	public void setActive(boolean active)
	{
		super.setActive(active);
		if(active)
		{
			levelWrapper.setSpriteBoxVisible(true);
		}
	}
		
	
	public void mouseMoved(MouseEvent e)
	{
		if(active)
		{
			//LevelPanel source = (LevelPanel)(e.getSource());
			levelWrapper.setSpriteBoxVisible(true);
			Rectangle box = levelWrapper.getSpriteBox();
			
			levelWrapper.repaintLevel(levelWrapper.snapToGrid(oldX ) - 4 ,
								levelWrapper.snapToGrid(oldY ) -4,
								levelWrapper.transformToPanel(width + 5),
			 					levelWrapper.transformToPanel(height + 5) );
			
			box.x = e.getX();
			box.y = e.getY();
			box.width = width;
			box.height = height;
			
			//level.snapToGrid(box);
			box.x = levelWrapper.transformToMap(box.x);
			box.y = levelWrapper.transformToMap(box.y);			


			levelWrapper.repaintLevel(levelWrapper.snapToGrid(e.getX() ) -4 ,
								levelWrapper.snapToGrid(e.getY()  ) -4 ,
								levelWrapper.transformToPanel(width + 5),
			 					levelWrapper.transformToPanel(height + 5));
			oldX = e.getX();
			oldY = e.getY();
		}
	}
	
	public void layerChanged(String [] newLayers, boolean autoLayers)
	{
		((InsertControl)control).layerChanged(newLayers, autoLayers);
	}
	
	public void mouseClicked(MouseEvent e)
	{
		if(active)
		{
			ArrayMap level = levelWrapper.getLevel();
			int x = levelWrapper.transformToMap(e.getX());
			int y = levelWrapper.transformToMap(e.getY());
			
			if(((x + width) <= level.getWidth()) &&
				((y + height) <= level.getHeight()))
			{
				level.add(((InsertControl)control).getSpriteAt(x ,y));
				levelWrapper.repaint();
			}
		}
	}
	
	private class InsertControl extends Box implements LayerChangeListener, ListSelectionListener, ActionListener
	{
		private final String [] spriteTypes = {"Green Walls","Background Decorations","Wall Decorations", "Backgrounds",
												"Poles","Thin floors","Platforms","Monsters",
												 "Candy", "Secret walls"};
		
		private SpriteCreator currentCreator;
		private Vector [] creators;
		//private JList [] spriteList;
		private JScrollPane [] scrollList;
		private JScrollPane currentList;
		private JComboBox showSpriteType;
		
		public InsertControl(String [] layers)
		{
			super(BoxLayout.Y_AXIS);
			
			//Select what type
			showSpriteType = new JComboBox(spriteTypes);
			showSpriteType.addActionListener(this);
			add(showSpriteType);
			
			//setup our vectors
			creators = new Vector [spriteTypes.length];
			for(int i = 0; i < spriteTypes.length; i++)
			{
				creators[i] = new Vector();
			}			
			
			//add in creators
			loadCreators(layers);
	
			//add in our JLists
			JPanel spriteLists = new JPanel();
			spriteLists.setLayout(new OverlayLayout(spriteLists));
			
			//spriteList = new JList[spriteTypes.length];
			scrollList = new JScrollPane[spriteTypes.length];
			for(int i = 0; i < spriteTypes.length; i++)
			{
				JList newList = new JList(creators[i]);
				newList.addListSelectionListener(this);
				newList.setCellRenderer(new SpriteCellRenderer());
				
				scrollList[i] = new JScrollPane(newList);
				scrollList[i].setVisible(false);

				spriteLists.add(scrollList[i]);
			}
			currentList = scrollList[0];
			currentList.setVisible(true);
			add(spriteLists);

			//Add in controll panel	
			JPanel creatorControl = new JPanel();
			creatorControl.setLayout(new OverlayLayout(creatorControl));

			for(int i = 0; i < spriteTypes.length; i++)
			{						
				Iterator it = creators[i].iterator();
				while (it.hasNext())
				{
					JComponent newControl = ((SpriteCreator)it.next()).getControlPanel();
					newControl.setVisible(false);
					creatorControl.add(newControl);
				}
			}		
			add(creatorControl);
			
			//Set current creator
			currentCreator = (SpriteCreator)creators[0].elementAt(0);
			currentCreator.getControlPanel().setVisible(true);
		}
		
		private void loadCreators(String [] layers)
		{
			//Walls
			//System.out.println("////Loading walls");
			//loadDirectory(creators[0], "graphics/tiles/greenSet/walls/", layers, "walls", true);
			//loadDirectory(creators[0], "graphics/tiles/greenSet/floors/", layers, "floors", true);
			
			////Loading walls
			creators[0].add(new PolygonCreator(layers,"graphics/tiles/greenSet/walls/top22Med.dat", "walls"));
			creators[0].add(new PolygonCreator(layers,"graphics/tiles/greenSet/walls/top45up.dat", "walls"));
			creators[0].add(new RectangleCreator(layers,"graphics/tiles/greenSet/walls/wallRightLarge.dat", "walls"));
			creators[0].add(new RectangleCreator(layers,"graphics/tiles/greenSet/walls/wallRightMed.dat", "walls"));
			creators[0].add(new RectangleCreator(layers,"graphics/tiles/greenSet/walls/wallRightSmall.dat", "walls"));
			creators[0].add(new RectangleCreator(layers,"graphics/tiles/greenSet/walls/wallRightToFloor45up.dat", "walls"));
			creators[0].add(new RectangleCreator(layers,"graphics/tiles/greenSet/walls/wallTopCornerRight.dat", "walls"));
			creators[0].add(new AutoRectangleCreator(layers,"graphics/tiles/greenSet/walls/blockVLarge.png", "walls", true));
			creators[0].add(new AutoRectangleCreator(layers,"graphics/tiles/greenSet/walls/topLarge.png", "walls", true));
			creators[0].add(new AutoRectangleCreator(layers,"graphics/tiles/greenSet/walls/topMed.png", "walls", true));
			creators[0].add(new AutoRectangleCreator(layers,"graphics/tiles/greenSet/walls/blockMed.png", "walls", true));
			creators[0].add(new AutoRectangleCreator(layers,"graphics/tiles/greenSet/walls/blockVExtraLarge.png", "walls", true));
			creators[0].add(new AutoRectangleCreator(layers,"graphics/tiles/greenSet/walls/blockExtraLarge.png", "walls", true));
			creators[0].add(new AutoRectangleCreator(layers,"graphics/tiles/greenSet/walls/blockSmall.png", "walls", true));
			creators[0].add(new AutoRectangleCreator(layers,"graphics/tiles/greenSet/walls/blockHExtraLarge.png", "walls", true));
			creators[0].add(new AutoRectangleCreator(layers,"graphics/tiles/greenSet/walls/topSmall.png", "walls", true));
			creators[0].add(new AutoRectangleCreator(layers,"graphics/tiles/greenSet/walls/blockHLarge.png", "walls", true));
			creators[0].add(new AutoRectangleCreator(layers,"graphics/tiles/greenSet/walls/blockLarge.png", "walls", true));
			creators[0].add(new PolygonCreator(layers,"graphics/tiles/greenSet/floors/cornerLowerLeft.dat", "floors"));
			creators[0].add(new RectangleCreator(layers,"graphics/tiles/greenSet/floors/cornerUpperLeft.dat", "floors"));
			creators[0].add(new RectangleCreator(layers,"graphics/tiles/greenSet/floors/cornerUpperRight.dat", "floors"));
			creators[0].add(new PolygonCreator(layers,"graphics/tiles/greenSet/floors/floor22RampLeft.dat", "floors"));
			creators[0].add(new PolygonCreator(layers,"graphics/tiles/greenSet/floors/floor45downToFloor.dat", "floors"));
			creators[0].add(new PolygonCreator(layers,"graphics/tiles/greenSet/floors/floor45upMed.dat", "floors"));
			creators[0].add(new RectangleCreator(layers,"graphics/tiles/greenSet/floors/floor45upToFloor.dat", "floors"));
			creators[0].add(new RectangleCreator(layers,"graphics/tiles/greenSet/floors/floorLarge.dat", "floors"));
			creators[0].add(new RectangleCreator(layers,"graphics/tiles/greenSet/floors/floorMed.dat", "floors"));
			creators[0].add(new RectangleCreator(layers,"graphics/tiles/greenSet/floors/floorSmall.dat", "floors"));
			creators[0].add(new PolygonCreator(layers,"graphics/tiles/greenSet/floors/speedBump.dat", "floors"));
			creators[0].add(new RectangleCreator(layers,"graphics/tiles/greenSet/floors/wallEndLeft.dat", "floors"));
			creators[0].add(new RectangleCreator(layers,"graphics/tiles/greenSet/floors/wallLeftLarge.dat", "floors"));
			creators[0].add(new RectangleCreator(layers,"graphics/tiles/greenSet/floors/wallLeftMed.dat", "floors"));


			creators[0].add(new RectangleCreator(layers,"graphics/tiles/greenSet/misc/doorBackground.dat", "floors"));		
			creators[0].add(new AutoRectangleCreator(layers,"graphics/tiles/greenSet/misc/doorForground.png", "walls", false));

			creators[0].add(new ExitCreator(layers, "walls"));

			//decorations for background
			//System.out.println("////Loading decorations");

			//loadDirectory(creators[1], "graphics/tiles/decoration/", layers, "decorations", false);

			////Loading decorations
			creators[1].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/cardReader.png", "decorations", false));
			creators[1].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/pipes3.png", "decorations", false));
			creators[1].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/grill.png", "decorations", false));
			creators[1].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/cornerPipeLowerLeft.png", "decorations", false));
			creators[1].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/pipes2.png", "decorations", false));
			creators[1].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/maze.png", "decorations", false));
			creators[1].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/pipes1.png", "decorations", false));
			creators[1].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/vPipeMed.png", "decorations", false));
			creators[1].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/pipes0.png", "decorations", false));
			creators[1].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/thing.png", "decorations", false));
			creators[1].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/hPipeLarge.png", "decorations", false));
			creators[1].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/starWindow.png", "decorations", false));
			creators[1].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/vPipeLarge.png", "decorations", false));
			creators[1].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/light.png", "decorations", false));
			creators[1].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/cornerPipeUpperRight.png", "decorations", false));
			creators[1].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/hPipeMed.png", "decorations", false));

			//decorations for background
			//System.out.println("////Loading wall decorations");

			//loadDirectory(creators[2], "graphics/tiles/decoration/", layers, "wall decorations", false);
			////Loading wall decorations
			creators[2].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/cardReader.png", "wall decorations", false));
			creators[2].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/pipes3.png", "wall decorations", false));
			creators[2].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/grill.png", "wall decorations", false));
			creators[2].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/cornerPipeLowerLeft.png", "wall decorations", false));
			creators[2].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/pipes2.png", "wall decorations", false));
			creators[2].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/maze.png", "wall decorations", false));
			creators[2].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/pipes1.png", "wall decorations", false));
			creators[2].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/vPipeMed.png", "wall decorations", false));
			creators[2].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/pipes0.png", "wall decorations", false));
			creators[2].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/thing.png", "wall decorations", false));
			creators[2].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/hPipeLarge.png", "wall decorations", false));
			creators[2].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/starWindow.png", "wall decorations", false));
			creators[2].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/vPipeLarge.png", "wall decorations", false));
			creators[2].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/light.png", "wall decorations", false));
			creators[2].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/cornerPipeUpperRight.png", "wall decorations", false));
			creators[2].add(new AutoRectangleCreator(layers,"graphics/tiles/decoration/hPipeMed.png", "wall decorations", false));

			//backgrounds
			//System.out.println("////Loading backgrouns");

			//loadDirectory(creators[3], "graphics/tiles/backgrounds/", layers, "background", false);
			////Loading backgrouns
			creators[3].add(new AutoRectangleCreator(layers,"graphics/tiles/backgrounds/Ship Interior.gif", "background", false));
			creators[3].add(new AutoRectangleCreator(layers,"graphics/tiles/backgrounds/Hazard.gif", "background", false));
			creators[3].add(new AutoRectangleCreator(layers,"graphics/tiles/backgrounds/Bloog Interior.gif", "background", false));
			creators[3].add(new AutoRectangleCreator(layers,"graphics/tiles/backgrounds/Ship Interior4.gif", "background", false));
			creators[3].add(new AutoRectangleCreator(layers,"graphics/tiles/backgrounds/Ship Interior3.gif", "background", false));
			creators[3].add(new AutoRectangleCreator(layers,"graphics/tiles/backgrounds/Bloog Pipes.gif", "background", false));
			creators[3].add(new AutoRectangleCreator(layers,"graphics/tiles/backgrounds/Ship Interior2.gif", "background", false));
			creators[3].add(new AutoRectangleCreator(layers,"graphics/tiles/backgrounds/background.png", "background", false));
			creators[3].add(new AutoRectangleCreator(layers,"graphics/tiles/backgrounds/Bloog Interior2.gif", "background", false));
			
			//Poles
			creators[4].add(new PoleCreator(layers, "graphics/tiles/poles/purplePole.png", "poles"));
			creators[4].add(new PoleCreator(layers, "graphics/tiles/poles/purplePoleBottom.png", "poles"));
			creators[4].add(new PoleCreator(layers, "graphics/tiles/poles/purplePoleTop.png", "poles"));
			creators[4].add(new PoleCreator(layers, "graphics/tiles/poles/yellowPoleBottom.png", "poles"));
			creators[4].add(new PoleCreator(layers, "graphics/tiles/poles/yellowPoleDark.png", "poles"));
			creators[4].add(new PoleCreator(layers, "graphics/tiles/poles/yellowPoleLarge.png", "poles"));
			creators[4].add(new PoleCreator(layers, "graphics/tiles/poles/yellowPoleMed.png", "poles"));
			creators[4].add(new PoleCreator(layers, "graphics/tiles/poles/yellowPoleSmall.png", "poles"));
			creators[4].add(new PoleCreator(layers, "graphics/tiles/poles/yellowPoleTop.png", "poles"));

			creators[4].add(new ThinFloorCreator(layers, "graphics/tiles/poles/extra/poleFloorTop.dat", "decorations"));
			creators[4].add(new AutoRectangleCreator(layers, "graphics/tiles/poles/extra/poleFloorBottom.png", "walls", false ));
			
			//Thin floors
			creators[5].add(new ThinFloorCreator(layers, "graphics/tiles/thinFloor/greenThinFloorLeftEnd.dat", "decorations"));
			creators[5].add(new ThinFloorCreator(layers, "graphics/tiles/thinFloor/greenThinFloorMed.dat", "decorations"));
			creators[5].add(new ThinFloorCreator(layers, "graphics/tiles/thinFloor/standPipe.dat", "decorations"));

			//Platforms
			creators[6].add(new BasicPlatformCreator(layers, "graphics/platforms/roundPlatform.png", "platforms"));
			creators[6].add(new BasicPlatformCreator(layers, "graphics/platforms/squarePlatform.png", "platforms"));

			
			creators[6].add(new DirectionHintCreator(layers, "platforms", MovingActor.NORTH));
			creators[6].add(new DirectionHintCreator(layers, "platforms", MovingActor.NORTH_WEST));
			creators[6].add(new DirectionHintCreator(layers, "platforms", MovingActor.WEST));
			creators[6].add(new DirectionHintCreator(layers, "platforms", MovingActor.SOUTH_WEST));
			creators[6].add(new DirectionHintCreator(layers, "platforms", MovingActor.SOUTH));
			creators[6].add(new DirectionHintCreator(layers, "platforms", MovingActor.SOUTH_EAST));
			creators[6].add(new DirectionHintCreator(layers, "platforms", MovingActor.EAST));
			creators[6].add(new DirectionHintCreator(layers, "platforms", MovingActor.NORTH_EAST));

			//Actors
			creators[7].add(new LeftHintCreator(layers, "monsters"));
			creators[7].add(new RightHintCreator(layers, "monsters"));
			creators[7].add(new BirdMonsterCreator(layers,"monsters"));

			//Candy
			creators[8].add(new CandyCreator("graphics/candy/triDent0.png", "graphics/candy/triDent1.png", "graphics/candy/100.png",
								100, layers, "candy"));
			creators[8].add(new CandyCreator("graphics/candy/icyPole0.png", "graphics/candy/icyPole1.png", "graphics/candy/200.png",
								200, layers, "candy"));
			creators[8].add(new CandyCreator("graphics/candy/marshmallow0.png", "graphics/candy/marshmallow1.png", "graphics/candy/500.png",
								500, layers, "candy"));
			creators[8].add(new CandyCreator("graphics/candy/candyBlue0.png", "graphics/candy/candyBlue1.png", "graphics/candy/1000.png",
								1000, layers, "candy"));
			creators[8].add(new CandyCreator("graphics/candy/candyRed0.png", "graphics/candy/candyRed1.png", "graphics/candy/1000.png",
								1000, layers, "candy"));
			creators[8].add(new CandyCreator("graphics/candy/sugar0.png", "graphics/candy/sugar1.png", "graphics/candy/5000.png",
								5000, layers, "candy"));

			//Secrets
			creators[9].add(new AutoRectangleCreator(layers,"graphics/tiles/greenSet/walls/blockLarge.png", "walls", false));		
			creators[9].add(new AutoRectangleCreator(layers,"graphics/tiles/greenSet/walls/blockMed.png", "walls", false));		
			creators[9].add(new AutoRectangleCreator(layers,"graphics/tiles/greenSet/walls/blockSmall.png", "walls", false));		
			creators[9].add(new AutoRectangleCreator(layers,"graphics/tiles/greenSet/walls/topLarge.png", "walls", false));		
			creators[9].add(new AutoRectangleCreator(layers,"graphics/tiles/greenSet/walls/topMed.png", "walls", false));		
			creators[9].add(new AutoRectangleCreator(layers,"graphics/tiles/greenSet/walls/topSmall.png", "walls", false));		
			creators[9].add(new AutoRectangleCreator(layers,"graphics/tiles/greenSet/walls/wallRightLarge.png", "walls", false));		
			creators[9].add(new AutoRectangleCreator(layers,"graphics/tiles/greenSet/walls/wallRightMed.png", "walls", false));		
			creators[9].add(new AutoRectangleCreator(layers,"graphics/tiles/greenSet/walls/wallRightSmall.png", "walls", false));		
			creators[9].add(new AutoRectangleCreator(layers,"graphics/tiles/greenSet/walls/wallTopCornerRight.png", "walls", false));
		}
		
		private void loadDirectory(Vector addVector, String directory, String [] layers, String defaultLayer, boolean canHit)
		{
			String [] files = (new File(directory)).list();
			//Append directory name
			for (int i = 0; i < files.length; i++)
			{
				files[i] = directory + files[i];
			}
			
			//Get a listing of all .pngs
			Hashtable png = new Hashtable();
			for (int i = 0; i < files.length; i++)
			{
				if((files[i].endsWith(".png")) ||(files[i].endsWith(".gif")))
				{
					png.put(files[i],files[i]);
				}
			}
			
			//get a listing of all .dat
			Vector dat = new Vector();
			for (int i = 0; i < files.length; i++)
			{
				if(files[i].endsWith(".dat"))
				{
					dat.add(files[i]);
				}
			}
			
			//for each dat
			Iterator it = dat.iterator();
			while (it.hasNext())
			{
				String file = (String)it.next();
				SpriteCreator newCreator = null;
				
				//try with rectangle
				try
				{
					newCreator = new RectangleCreator(layers, file, defaultLayer);
					System.err.println("creators[0].add(new RectangleCreator(layers,\"" + file + "\", \"" + defaultLayer +"\"));");		

				}
				catch(Exception e)
				{
					//try with polygon
					try
					{
						newCreator = new PolygonCreator(layers, file, defaultLayer);
						System.err.println("creators[0].add(new PolygonCreator(layers,\"" + file + "\", \"" + defaultLayer +"\"));");		

					}
					catch(Exception e2)
					{
						System.err.println(file + " Cannot be loaded as a Rectangle, or polygon");
						System.exit(0);
					}
				}
				
				png.remove(newCreator.getGraphic());
				addVector.add(newCreator); //remove filename from list of .png
			}
			
			//for each png left
			//load with autorectangle
			
			it = png.values().iterator();
			while(it.hasNext())
			{
				String file = (String)it.next();
				addVector.add(new AutoRectangleCreator(layers, file, defaultLayer, canHit));
				
				System.err.println("creators[0].add(new AutoRectangleCreator(layers,\"" + file + "\", \"" + defaultLayer + "\", " + canHit + "));");		

				//System.err.println(file + " is a AutoRectangle");
			}
		}
		
		public void valueChanged(ListSelectionEvent e)
		{
			currentCreator.getControlPanel().setVisible(false);
			currentCreator = (SpriteCreator)((JList)(e.getSource())).getSelectedValue();
			currentCreator.getControlPanel().setVisible(true);
			width = currentCreator.getWidth();
			height = currentCreator.getHeight();
		}
		
		public void layerChanged(String [] newLayers, boolean autoLayers)
		{
			for(int i = 0; i < spriteTypes.length; i++)
			{						
				Iterator it = creators[i].iterator();
				while (it.hasNext())
				{
					((SpriteCreator)it.next()).layerChanged(newLayers, autoLayers);
				}
			}
		}
		
		public void actionPerformed(ActionEvent e)
		{
			if(e.getSource() == showSpriteType)
			{
				currentList.setVisible(false);
				scrollList[showSpriteType.getSelectedIndex()].setVisible(true);
				currentList = scrollList[showSpriteType.getSelectedIndex()];

				currentList.setVisible(true);
			}
		}
		
		public Sprite getSpriteAt(int x, int y)
		{
			return currentCreator.getSpriteAt(x,y);
		}
		
		private class SpriteCellRenderer extends JLabel implements ListCellRenderer
		{
			private static final int CELL_HEIGHT = 50;
			private static final int CELL_WIDTH = 150;
			private final Color CELL_STRIPE_COLOR = new Color(245,245,245);
			
		     // This is the only method defined by ListCellRenderer.
		     // We just reconfigure the JLabel each time we're called.
			public Component getListCellRendererComponent(
				JList list,
				Object value,            // value to display
				int index,               // cell index
				boolean isSelected,      // is the cell selected
				boolean cellHasFocus)    // the list and the cell have the focus
			{

         		SpriteCreator creator = (SpriteCreator)value;
         		setIcon(getScaledIcon(creator.getGraphic()));
         		setText(creator.getWidth() + " x " + creator.getHeight());
         		setFont(new Font("Areal" , Font.PLAIN, 10));
         		
				if (isSelected)
				{
					setBackground(list.getSelectionBackground());
					setForeground(list.getSelectionForeground());
					setBorder(BorderFactory.createLineBorder(list.getSelectionForeground()));
				}
				else
				{
					setForeground(list.getForeground());
					if((index % 2) == 0)
					{
						setBackground(list.getBackground());
					}
					else
					{
						setBackground(CELL_STRIPE_COLOR);
					}
					setBorder(null);
				}
				setEnabled(list.isEnabled());
				setOpaque(true);
				return this;
			}
			
			private ImageIcon getScaledIcon(String name)
			{
				//Get our image and scale it
				Image spriteImage = Cache.getImage(name);
				
				int width = spriteImage.getWidth(null);
				int height = spriteImage.getHeight(null);
				
				//Calculate scale for width
				double widthScale = (double)CELL_WIDTH / (double)width;
				
				//Calculate scale for height
				double heightScale = (double)CELL_HEIGHT / (double)height;
				
				if(widthScale > heightScale)
				{
					//scale to heightScale
					width = (int)((double)width * heightScale);
					height = (int)((double)height * heightScale);
				}
				else
				{
					width = (int)((double)width * widthScale);
					height = (int)((double)height * widthScale);	
				}
				
         		spriteImage = spriteImage.getScaledInstance(width,height, Image.SCALE_SMOOTH);
				return new ImageIcon(spriteImage);
			}
		}
	}
}
