package keenGame.editor.tools;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.border.TitledBorder;

import keenGame.editor.LevelPanel;
import keenGame.sprite.Sprite;

/**
 * @author Luke Sleeman
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class SelectTool extends Tool implements ActionListener
{
	private boolean selectBox = false;
	private boolean dragSprites = false;
	private int selectBoxX, selectBoxY;
	private int dragX, dragY;
	
	private int startX, startY;
	private int oldX, oldY, oldWidth, oldHeight;
	private Vector selection = new Vector();
	
	private JRadioButton allLayers, oneLayer;
	private JComboBox selectLayer;
	private JButton clear, delete, move;
	private JComboBox moveToLayer;
	
	
	public SelectTool(LevelPanel level)
	{
		super("Select", level);
		String [] layers = levelWrapper.getLevel().getLayers();
		
		control = new JPanel();
		control.setLayout(new BoxLayout(control, BoxLayout.Y_AXIS));

		//Layer selection mode
		ButtonGroup selectInGroup = new ButtonGroup();
		JPanel selectInPanel = new JPanel();
		selectInPanel.setLayout(new BoxLayout(selectInPanel, BoxLayout.Y_AXIS));
		selectInPanel.setBorder(new TitledBorder("Select sprites from:"));

		allLayers = new JRadioButton("All visible layers");
		allLayers.addActionListener(this);
		selectInPanel.add(allLayers);
		selectInGroup.add(allLayers);

		oneLayer = new JRadioButton("One layer");
		oneLayer.addActionListener(this);
		selectInPanel.add(oneLayer);
		selectInGroup.add(oneLayer);
		
		selectLayer = new JComboBox(layers);
		selectLayer.setEditable(false);
		selectLayer.setEnabled(false);
		selectInPanel.add(selectLayer);

		allLayers.setSelected(true);
		control.add(selectInPanel);
		
		JPanel toolPanel = new JPanel();
		toolPanel.setLayout(new BoxLayout(toolPanel, BoxLayout.Y_AXIS));
		toolPanel.setBorder(new TitledBorder("Tools:"));
		
		//Buttons
		clear = new JButton("Clear selction");
		clear.addActionListener(this);
		toolPanel.add(clear);
		
		delete = new JButton("Delete");
		delete.addActionListener(this);
		toolPanel.add(delete);
		
		//control.add(new JLabel("Move sprites to:"));
		JPanel movePanel = new JPanel();
		movePanel.setBorder(new TitledBorder("Move to layer:"));
		movePanel.setLayout(new BoxLayout(movePanel, BoxLayout.Y_AXIS));
		
		moveToLayer = new JComboBox(layers);
		moveToLayer.setEnabled(false);
		moveToLayer.addActionListener(this);
		movePanel.add(moveToLayer);
		
		move = new JButton("Move");
		move.setEnabled(false);
		move.addActionListener(this);
		movePanel.add(move);
		
		toolPanel.add(movePanel);
		control.add(toolPanel);
		
		//control.add(Box.createVerticalGlue());
	}
	
	public void setActive(boolean active)
	{
		super.setActive(active);
		selection.clear();
	}
	
	public void actionPerformed(ActionEvent e)
	{
		if((e.getSource() == oneLayer) || (e.getSource() == allLayers))
		{
			selectLayer.setEnabled(oneLayer.isSelected());
		}
		else if(e.getSource() == clear)
		{
			selection.clear();
			levelWrapper.repaint();
		}
		else if(e.getSource() == delete)
		{
			levelWrapper.getLevel().removeAll(selection);
			selection.clear();
			levelWrapper.repaint();
		}
		else if((e.getSource() == move) && (selection.size() != 0))
		{
			String newLayer = (String)moveToLayer.getSelectedItem();
			//remove sprites
			levelWrapper.getLevel().removeAll(selection);
			
			//iterate through and change layer
			Iterator it = selection.iterator();
			while (it.hasNext())
			{
				Sprite sprite = (Sprite)it.next();
				sprite.setLayer(newLayer);
				levelWrapper.getLevel().add(sprite);
			}
		}
	}
	
	public void layerChanged(String [] layers, boolean autoLayers)
	{
		String selectedLayer = (String)selectLayer.getSelectedItem();
		
		selectLayer.removeAllItems();
		moveToLayer.removeAllItems();
		
		for(int i = 0; i < layers.length; i++)
		{
			selectLayer.addItem(layers[i]);
			moveToLayer.addItem(layers[i]);
		}
		
		//Try and get back to that layer
		selectLayer.setSelectedItem(selectedLayer);
		moveToLayer.setSelectedItem(selectedLayer);
		
		moveToLayer.setEnabled(!autoLayers);
		move.setEnabled(!autoLayers);
	}
	
	/**
	 * @see java.awt.event.MouseMotionListener#mouseDragged(MouseEvent)
	 */
	public void mouseDragged(MouseEvent e)
	{
		if(active)
		{
			int x = levelWrapper.transformToMap(e.getX());
			int y = levelWrapper.transformToMap(e.getY());

			Rectangle box = levelWrapper.getSpriteBox();
				
			if((!selectBox) && (!dragSprites)) //We are about to start a selction box
			{
				//Are we inside a selected sprite
				boolean insideSelection = false;
				Iterator it = selection.iterator();
				
				int xNoSnap = levelWrapper.transformNoSnap(e.getX());
				int yNoSnap = levelWrapper.transformNoSnap(e.getY());
				
				while(it.hasNext() && !insideSelection)
				{
					Sprite sprite = (Sprite)it.next();
					
					insideSelection = (xNoSnap >= sprite.getX()) &&
									(xNoSnap <= (sprite.getX() + sprite.getWidth())) &&
									(yNoSnap >= sprite.getY()) &&
									(yNoSnap <= (sprite.getY() + sprite.getHeight()));
				}
				
				if(insideSelection)
				{
					dragSprites = true;
					levelWrapper.setSpriteBoxVisible(false);
					dragX = x;
					dragY = y;
				}
				else
				{
					//else start a selection box
					selectBoxX = x;
					selectBoxY = y;				
					
					selectBox = true;
	
					
					box.x = x;
					box.y = y;
					box.width = 0;
					box.height = 0;
					
					startX = e.getX();
					startY = e.getY();
					
					levelWrapper.setSpriteBoxVisible(true);
				}

			}
			else if(dragSprites)
			{
				if((x != dragX) || (y != dragY)) //There has been real movement
				{
					int deltaX = dragX - x;
					int deltaY = dragY - y;
					
					Iterator it = selection.iterator();
					boolean insideArea = true;
					int levelWidth = levelWrapper.getLevel().getWidth();
					int levelHeight = levelWrapper.getLevel().getHeight();
					
					while(it.hasNext() && insideArea)
					{
						Sprite sprite = (Sprite)it.next();
						int newX = sprite.getX() - deltaX;
						int newY = sprite.getY() - deltaY; 
						
						insideArea = (newX >= 0) && (newY >= 0) &&
									((sprite.getWidth() + newX) <= levelWidth) &&
									((sprite.getHeight() + newY) <= levelHeight);
					}
					if(insideArea)
					{
						it = selection.iterator();
						while(it.hasNext())
						{
							Sprite sprite = (Sprite)it.next();
							
							levelWrapper.getLevel().remove(sprite);
							
							sprite.setX(sprite.getX() - deltaX);
							sprite.setY(sprite.getY() - deltaY);
							
							levelWrapper.getLevel().add(sprite);
						}
						
						dragX = x;
						dragY = y;
						
						levelWrapper.repaint();
					}
				}
			}
			else if(selectBox)
			{
				if(!levelWrapper.isSnapToGrid())
				{
					levelWrapper.repaintLevel(oldX, oldY, oldWidth + 1, oldHeight + 1);
				}
				
				int repaintX = startX;
				int repaintY = startY;
				int repaintWidth = e.getX() - repaintX;
				int repaintHeight = e.getY() - repaintY;

				 					
				//We need all this to ensure that box.width, and box.height stay +ve	
				if((x > selectBoxX) && (y > selectBoxY)) // if we are below, and to the right of the box corner
				{
					box.x = selectBoxX;
					box.y = selectBoxY;
					
					box.width =  x - selectBoxX;
					box.height =  y - selectBoxY;
				}
				else if((x > selectBoxX) && (y < selectBoxY)) //If we are above and to the right
				{
					box.x = selectBoxX;
					box.y = y;
					repaintY = e.getY();
					
					box.width = x - selectBoxX;
					box.height = selectBoxY - y;
					repaintHeight = startY - e.getY();
				}
				else if((x < selectBoxX) && (y > selectBoxY)) //If we are below and to the left
				{
					box.x = x;
					repaintX = e.getX();
					box.y = selectBoxY;
					
					box.width = selectBoxX - x;
					repaintWidth = startX - e.getX();
					box.height = y - selectBoxY;
				}
				else if((x < selectBoxX) && (y < selectBoxY)) //If we are above and to the left
				{
					box.x = x;
					repaintX = e.getX();
					box.y = y;
					repaintY = e.getY();
					
					box.width = selectBoxX - x;
					repaintWidth = startX - e.getX();
					box.height = selectBoxY - y;
					repaintHeight = startY - e.getY();
				}

				if(!levelWrapper.isSnapToGrid())
				{
					repaintX = repaintX - 5;
					repaintY = repaintY - 5;
					repaintWidth = repaintWidth + 5;
					repaintHeight = repaintHeight + 5;
					levelWrapper.repaintLevel(repaintX, repaintY, repaintWidth + 1, repaintHeight + 1);
					oldX = repaintX;
					oldY = repaintY;
					oldWidth = repaintWidth;
					oldHeight = repaintHeight;
				}
				else
				{
					levelWrapper.repaint();
				}
			}
		}
	}
	
	/**
	 * @see java.awt.event.MouseListener#mouseReleased(MouseEvent)
	 */
	public void mouseReleased(MouseEvent e)
	{
		if(active)
		{
			if(selectBox)
			{
				selectBox(e);
			}
			else if(dragSprites)
			{
				dragSprites = false;
			}
			else //Its just a single click 
			{
				selectSingle(e);
			}
		}
	}
	
	private void selectBox(MouseEvent e)
	{
		levelWrapper.setSpriteBoxVisible(false);
		selectBox = false;
		levelWrapper.repaint();
		
		//get all the sprites in the box
		if((e.getModifiersEx() & InputEvent.CTRL_DOWN_MASK) != InputEvent.CTRL_DOWN_MASK)
		{
			selection.clear();
		}
		
		Rectangle box = levelWrapper.getSpriteBox();
		validateBox(box);

		
		Iterator it;
		if(allLayers.isSelected())
		{
			it = levelWrapper.getLevel().iterator(box.x, box.y, box.width, box.height,
												levelWrapper.getVisibleLayers());
		}
		else
		{
			it = levelWrapper.getLevel().iterator(box.x, box.y, box.width, box.height,
												(String)selectLayer.getSelectedItem());
		}
							
		//set them to be selected
		while(it.hasNext())
		{
			Object next = it.next();
			if(!selection.contains(next))
			{
				selection.add(next);
			}
		}
		
		levelWrapper.setSelectedSprites(selection);
		levelWrapper.setSelectedSpriteVisible(true);

	}
	
	private void selectSingle(MouseEvent e)
	{
		int x = levelWrapper.transformToMap(e.getX());
		int y = levelWrapper.transformToMap(e.getY());
		
		if((e.getModifiersEx() & InputEvent.CTRL_DOWN_MASK) != InputEvent.CTRL_DOWN_MASK)
		{
			selection.clear();
		}
		
		Iterator it;
		if(allLayers.isSelected())
		{
			it = levelWrapper.getLevel().iterator(x, y, 1, 1,
											levelWrapper.getVisibleLayers());
		}
		else
		{
			it = levelWrapper.getLevel().iterator(x, y, 1, 1,
											(String)selectLayer.getSelectedItem());
		}
		
		//set them to be selected
		Sprite lastSprite = null;
		while(it.hasNext())
		{
			Sprite current = (Sprite)it.next();
			
			if((current.getX() <= x) && (current.getY() <= y) &&
				(current.getX() + current.getWidth() >= x) &&
				(current.getY() + current.getHeight() >= y))
			{
				lastSprite = current;
			}
		}

		//If it is already selected remove
		if(lastSprite != null)
		{
			if(selection.contains(lastSprite))
			{
				selection.remove(lastSprite);
			}
			else
			{
				selection.add(lastSprite);
			}
		}
						
		levelWrapper.setSelectedSprites(selection);
		levelWrapper.setSelectedSpriteVisible(true);
	}
	
	private void validateBox(Rectangle box)
	{
		int width = levelWrapper.getLevel().getWidth();
		int height = levelWrapper.getLevel().getHeight();
			
		if(box.width == 0)
		{
			box.width = 1;
		}
		if(box.height == 0)
		{
			box.height = 1;
		}
		if(box.x < 0)
		{
			box.x = 0;
		}
		if(box.y < 0)
		{
			box.y = 0;
		}
		if(box.x > width)
		{
			box.x = width - 1;
		}
		if(box.y > height)
		{
			box.y = height - 1;
		}
		if((box.x + box.width) > width)
		{
			box.width = (width - box.x);
		}
		if((box.y + box.height) > height)
		{
			box.height = (height - box.y);
		}
	}
		
}
