package keenGame.editor.tools.spriteCreators;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.awt.Image;

import javax.swing.JComboBox;
import javax.swing.border.TitledBorder;

import keenGame.Cache;
import keenGame.sprite.MovingActor;
import keenGame.sprite.Sprite;
import keenGame.sprite.platform.BasicPlatform;

/**
 * @author Luke Sleeman
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class BasicPlatformCreator extends MovingActorCreator
{
	private static final String [] DEFAULT_HIT_LAYERS = {"platforms"};


	private JComboBox direction;
	
	public BasicPlatformCreator(String [] layers, String graphic, String defaultLayer)
	{
		super(layers, DEFAULT_HIT_LAYERS, defaultLayer);

		this.graphic = graphic;
		this.defaultLayer = defaultLayer;
			
		Image myImage = Cache.getImage(graphic);
		width = myImage.getWidth(null);
		height = myImage.getHeight(null);
		
		String [] directions = {"North","North West","West","South West","South",
								"South East", "East","North East"};
		direction = new JComboBox(directions);
		direction.setBorder(new TitledBorder("Initial Direction:"));
		controlPanel.add(direction);	
	}
	
	/**
	 * @see keenGame.editor.SpriteCreator#getSpriteAt(int, int, Level)
	 */
	public Sprite getSpriteAt(int x, int y)
	{
		String layer;
		if(insertLayer.isEnabled())
		{
			layer = insertLayer.getSelectedItem().toString();
		}
		else
		{
			layer = defaultLayer;
		}
		
		String [] colLayers = hitLayers.getLayers();
		if(colLayers == null)
		{
			colLayers = new String [1];
			colLayers[0] = defaultLayer;
		}
		
		return new BasicPlatform(graphic, layer , colLayers,
									null, x ,y ,
									directionToInt((String)direction.getSelectedItem()));
	}
	
	private int directionToInt(String direction)
	{
		int intDir = MovingActor.NONE;
		
		if(direction.equals("North"))
		{
			intDir = MovingActor.NORTH;
		}
		else if(direction.equals("North West"))
		{
			intDir = MovingActor.NORTH_WEST;
		}
		else if(direction.equals("West"))
		{
			intDir = MovingActor.WEST;
		}
		else if(direction.equals("South West"))
		{
			intDir = MovingActor.SOUTH_WEST;
		}		
		else if(direction.equals("South"))
		{
			intDir = MovingActor.SOUTH;
		}		
		else if(direction.equals("South East"))
		{
			intDir = MovingActor.SOUTH_EAST;
		}		
		else if(direction.equals("East"))
		{
			intDir = MovingActor.EAST;
		}
		else if(direction.equals("North East"))
		{
			intDir = MovingActor.NORTH_EAST;
		}
		else
		{
			System.err.println("couldnt find direction " + direction);
		}
		return intDir;
	}
}
