package keenGame.editor.tools.spriteCreators;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import javax.swing.AbstractListModel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.OverlayLayout;
import keenGame.editor.LayerChangeListener;

/**
 * @author Luke Sleeman
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
class LayerSelector extends JPanel implements LayerChangeListener
{
	JList list;
	LayerListModel data;
	String [] defaultLayers;
	
	public LayerSelector(String [] layers, String [] defaultLayers)
	{
		this.defaultLayers = defaultLayers;
		setLayout(new OverlayLayout(this));
		
		data = new LayerListModel(layers);
		list = new JList(data);
		list.setEnabled(false);
		
		add(new JScrollPane(list));
	}
	
	public String [] getLayers()
	{
		String [] toReturn;
		
		if(list.isEnabled())
		{
			if(list.getSelectedValues().length == 0)
			{
				toReturn = null;
			}
			else
			{
				Object [] values = list.getSelectedValues();
				
				toReturn = new String [values.length];
				for (int i = 0; i < values.length; i++)
				{
					toReturn[i] = (String)values[i];
				}
			}
		}
		else
		{
			toReturn = defaultLayers;
		}
		
		return toReturn;
	}
	
	public void layerChanged(String [] layers, boolean autoLayers)
	{
		data.layerChanged(layers, autoLayers);
		list.setEnabled(!autoLayers);
	}
	
	private class LayerListModel extends AbstractListModel implements LayerChangeListener
	{
		private String [] layers;
		
		public LayerListModel(String [] layers)
		{
			this.layers = layers;
		}
		
		public void layerChanged(String [] layers, boolean autoLayers)
		{
			int oldSize = this.layers.length;
			
			this.layers = layers;
			
			fireContentsChanged(this, 0, layers.length);
			if(oldSize > layers.length)
			{
				fireIntervalRemoved(this,layers.length + 1,oldSize);
			}
			else if(oldSize < layers.length)
			{
				fireIntervalAdded(this, oldSize + 1, layers.length);
			}
		}
		
		public Object getElementAt(int index)
		{
			return layers[index];
		}
		
		public int getSize()
		{
			return layers.length;
		}
	}
	
}
