package keenGame.sprite;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.awt.Graphics;

import keenGame.XmlElement;

/**
 * Provides an abstract base classes for all Hints, should
 * any class wish to provide a default implementation for
 * hasHit(Hint).
 * 
 * A hint is an invisible Sprite that is used by monsters
 * and various MovingActors, to control their movement.  
 * Generally onces a Monster, or moving platform hit a Hint
 * They will change their direction, to the direction specified
 * by the hint
 */
public abstract class Hint extends Sprite
{
	/**
     * Constructs an Hint.  When writing your own subclasses of Hint you
     * should always put in a call to this constructor with Super.  After 
     * this constructor has been called, you should define a hitArea, as well
     * as any other instance variables your Hint should need.
     * 
     * @param imageLocation - The location of an image (normally .png format) that
     * 		is used for this Hint
     * @param layer - The layer that this Hint should exist in
     * @param xValue - The x location of the Hint
     * @param yValue - The y location of the Hint
     * 
     * @see keenGame.sprite.Sprite#hitArea
     * @see keenGame.sprite.Actor#Actor(String, String, int, int)
     */
    public Hint(String layer, int xValue, int yValue)    
    {
        super(layer, xValue,yValue);
    }
    
    public Hint(XmlElement spriteElement)
    {
    	super(spriteElement);
    }
    
    /**
     * Does nothing, so the hint will not appear
     */
    public void draw(Graphics drawTo, int xOffset, int yOffset)
    {}


    public void draw(Graphics drawTo, int xOffset, int yOffset, boolean debug, double Scale)
    {
    	if(debug)
    	{
    		super.draw(drawTo, xOffset, yOffset, true, Scale);
    	}
    }
}