package keenGame.sprite;

/*
 * LinePointIterator.java
 *
 * Created on February 27, 2002, 11:42 AM
 *
 * This class utiles the Bresenham line drawing algorithm to calculte the interger points between
 * x0,y0 and x1,y1 that generates the 'straightest' line.  This class is designed to allow the client
 * to iterate through the points between x0,y0 and x1,y1.  To use:  call initIterator() with starting and
 * ending points.  Then, while hasMorePoints() returns true, you can iterate through the points on the line
 * by calling next() and using getX() and getY() to return the current point on the line.
 */

public class LinePointIterator
{
  int x0, y0, x1, y1;  // start and end points
  int stepx, stepy; // the direction to move, will be either -1 or 1
  int dx, dy; // the delta between the start and end points
  int fraction;  // used in calculating line path
  
  /** Creates a new instance of LinePointIterator */
  public LinePointIterator()
  {
  }
  
  /**
   *  Initializes the LinePointIterator to start iterating from startX, startY to 
   *  endX, endY. 
   */
  public void initIterator(int startX, int startY, int endX, int endY)
  {
    x0 = startX;
    y0 = startY;
    x1 = endX;
    y1 = endY;
    
    dy = y1 - y0;
    dx = x1 - x0;
 
    if (dy < 0)
    {
      dy = -dy;  
      stepy = -1;
    }
    else
      stepy = 1; 

    if (dx < 0)
    {
      dx = -dx;  
      stepx = -1; 
    } 
    else
      stepx = 1; 

    dy <<= 1;   // dy is now 2*dy
    dx <<= 1;   // dx is now 2*dx
    
    if (dx > dy)
      fraction = dy - (dx >> 1); // same as 2*dy - dx
    else
      fraction = dx - (dy >> 1); // same as 2dx - dy

  }
  
  /**
   *  Returns the current point's X
   */
  public int getX()
  {
    return x0;
  }
  
  /**
   *  Returns the current point's Y
   */
  public int getY()
  {
    return y0;
  }
  
  /**
   *  Checks to see if end of line is reached.
   */
  public boolean hasMorePoints()
  {
    if (dx > dy)
      return (x0 != x1);
    else
      return (y0 != y1);
  }
  
  /*
   *  Updates the state of the class to contain the next point's X,Y in the line
   */
  public void next()
  {
    if (hasMorePoints())
    {
      if (dx > dy)
      {
        if (fraction >= 0)
        {
          y0 += stepy;
          fraction -= dx; // same as fraction -= 2*dx
        }
        x0 += stepx;
        fraction += dy; // same as fraction -= 2*dy
      }
      else
      {
        if (fraction >= 0)
        {
          x0 += stepx;
          fraction -= dy;
        }
        y0 += stepy;
        fraction += dx;
      }
    }
  }
  
/* The following function included for reference, as it was converted from doing the entire
 * drawing in a single loop to doing it iteratively calling next() until hasMorePoints() is false
 *

  public void lineBresenham(int x0, int y0, int x1, int y1)
  {
    int dy = y1 - y0;
    int dx = x1 - x0;
    int stepx, stepy;
    
    if (dy < 0)
    { dy = -dy;  stepy = -1; } else
    { stepy = 1; }
    if (dx < 0)
    { dx = -dx;  stepx = -1; } else
    { stepx = 1; }
    dy <<= 1;                                                  // dy is now 2*dy
    dx <<= 1;                                                  // dx is now 2*dx
    
    raster.setPixel(pix, x0, y0);
    if (dx > dy)
    {
      int fraction = dy - (dx >> 1);                         // same as 2*dy - dx
      while (x0 != x1)
      {
        if (fraction >= 0)
        {
          y0 += stepy;
          fraction -= dx;                                // same as fraction -= 2*dx
        }
        x0 += stepx;
        fraction += dy;                                    // same as fraction -= 2*dy
        raster.setPixel(pix, x0, y0);
      }
    } 
    else
    {
      int fraction = dx - (dy >> 1);
      while (y0 != y1)
      {
        if (fraction >= 0)
        {
          x0 += stepx;
          fraction -= dy;
        }
        y0 += stepy;
        fraction += dx;
        raster.setPixel(pix, x0, y0);
      }
    }
  }
  
*/
  
}
