package keenGame.sprite;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.util.EventObject;

/**
 * And event that indicates than an object in the game world
 * has moved.  These are ususally fired off by MovingActors
 */
public class MovementEvent extends EventObject
{
	private int oldX, oldY, newX, newY;
	/**
	 * Constructor for MovementEvent.
	 * 
	 * @param source - The MovingActor that generated teh MovementEvent
	 * @param oldX - The X position of the MovingActor before the event occured
	 * @param oldY - The Y position of the MovingActor before the event occured
	 * @param newX - The X position of the MovingActor after the event occured
	 * @param newY - The Y position of the MovingActor after the event occured
	 */
	public MovementEvent(Object source, int oldX, int oldY, int newX, int newY)
	{
		super(source);
		this.oldX = oldX;
		this.oldY = oldY;
		this.newX = newX;
		this.newY = newY;
	}
	/**
	 * Returns the newX.
	 * @return int
	 */
	public int getNewX()
	{
		return newX;
	}

	/**
	 * Returns the newY.
	 * @return int
	 */
	public int getNewY()
	{
		return newY;
	}

	/**
	 * Returns the oldX.
	 * @return int
	 */
	public int getOldX()
	{
		return oldX;
	}

	/**
	 * Returns the oldY.
	 * @return int
	 */
	public int getOldY()
	{
		return oldY;
	}

}
