package keenGame.sprite;

import keenGame.Level;
import keenGame.XmlElement;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * Provides an abstract base classes for all Players, should
 * any class wish to provide a default implementation for
 * hasHit(Player).
 * 
 * A player is a MovingActor controlled by a human.  Currently
 * only KeenActor implements Player, but hopefully in the future
 * there will be other implemtations that will function over a network
 * allowing for multiplayer
 */
public abstract class Player extends MovingActor
{
	/**
     * Constructs an Player.  When writing your own subclasses of Player you
     * should always put in a call to this constructor with Super.  After 
     * this constructor has been called, you should define a hitArea, as well
     * as any other instance variables yourBullet should need.
     * 
     * @param imageLocation - The location of an image (normally .png format) that
     * 		is used for this Player
     * @param layer - The layer that this Player should exist in
     * @param hitLayers - An array of strings containing the layers which contain
     * 	Sprites with which this Player can collide
     * @param xValue - The x location of the Player
     * @param yValue - The y location of the Player
     * 
     * @see keenGame.sprite.Sprite#hitArea
     * @see keenGame.sprite.Actor#Actor(String, String, int, int)
     */
    public Player(String layer, 
    					String [] hitLayers, Level myWorld,
    					int xValue, int yValue)    
    {
        super(layer, hitLayers, myWorld, xValue,yValue);
    }
    
    public Player(XmlElement spriteElement, Level myLevel)
    {
    	super(spriteElement, myLevel);
    }
    
    public boolean hasHit(Bullet hitSprite)
    {
    	return false;
    }
    
    public boolean hasHit(Monster hitSprite)
    {
    	return false;
    }
}