package keenGame.sprite.candy;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import keenGame.Level;
import keenGame.XmlElement;
import keenGame.sprite.Candy;
import keenGame.sprite.keenActor.KeenActor;

/**
 * PointsCandy is a type of Candy that gives Keen points when he collects it
 */
public class PointsCandy extends Candy
{
	private final static int BIG_POINTS_OFFSET = 15;
	private final static int POINTS_OFFSET = 5;
	
	private int points;

	/**
     * Constructs an PointsCandy.
     * 
     * @param image0 - The first frame of the candy animation
     * @param image1 - The second frame of the candy animation
     * @param endImage - The images of points, that goes floating upwards when
     * 					keen collects the candy
     * @param layer - The layer that this sprite should exist in
	 * @param myWorld - the Level in which the PointsCandy exists
     * @param xValue - The x location of the PointsCandy
     * @param yValue - The y location of the PointsCandy
     * @param points - The number of points Keen will collect from the PointsCandy
     * 
     * @see keenGame.sprite.Actor#Actor(String, String, int, int)
     */
	public PointsCandy(String image0, String image1 ,String endImage,
				String layer, Level myWorld,
				 int xValue, int yValue, int points)    
    {
        super(image0,image1,endImage, layer,  myWorld, xValue,yValue);
        init(points);
    }
    
    public PointsCandy(XmlElement spriteElement, Level myWorld)
    {
    	super(spriteElement, myWorld);
    	int newPoints = spriteElement.getChild("points").getValue();
    	init(newPoints);
    }
    
    private void init(int points)
    {
        this.points = points;
    }
    
    public Element toXml(Document xmlDoc)
	{
		return addToElement(xmlDoc, xmlDoc.createElement("PointsCandy"));
	}
	
	protected Element addToElement(Document xmlDoc, Element element)
    {
    	super.addToElement(xmlDoc, element);
    	
    	//Graphic
    	Element pointsElement = xmlDoc.createElement("points");
		pointsElement.appendChild(xmlDoc.createTextNode(Integer.toString(points)));
		element.appendChild(pointsElement);
		
		return element;
    }

	/**
	 * At the moment this just adjusts the position of the
	 * number that moves upwards.
	 * TODO:  Make this give points to keen
	 */
    protected void collectedBy(KeenActor keen)
    {
    	if(points >= 1000)
    	{
    		attemptMove(x - BIG_POINTS_OFFSET ,y);
    	}
    	else
    	{
    		attemptMove(x - POINTS_OFFSET, y);
    	}
    }
}
