package keenGame.sprite.hint;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.awt.Rectangle;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import keenGame.Cache;
import keenGame.XmlElement;
import keenGame.sprite.Hint;

/**
 * A LeftHint sugests that a MovingActor that hits it, walk to the left.
 * DirectionHints are usually used to control the paths that monsters
 * take, stoping them from running out over space, etc
 */
public class LeftHint extends Hint
{
	/**
     * Constructs a LeftHint.  
     * 
     * @param layer - The layer that this LeftHint should exist in
     * @param xValue - The x location of the LeftHint
     * @param yValue - The y location of the LeftHint
     * @see keenGame.sprite.Actor#Actor(String, String, int, int)
     */
    public LeftHint(String layer, int xValue, int yValue)
    {
    	super(layer, xValue, yValue);
    	init();
    }
    
    public LeftHint(XmlElement spriteElement)
    {
    	super(spriteElement);
    	init();
    }
    
    private void init()
    {
    	image = Cache.getImage("graphics/hint/leftHint.png");
    	width = image.getWidth(null);
    	height = image.getHeight(null);
    	
    	hitArea = new Rectangle(x, y, width, height);
    }
    
    public Element toXml(Document xmlDoc)
	{
		return addToElement(xmlDoc, xmlDoc.createElement("LeftHint"));
	}
}
