package keenGame.sprite.keenActor;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import keenGame.Cache;
import keenGame.Level;
import keenGame.XmlElement;
import keenGame.sprite.Platform;
import keenGame.sprite.Player;
import keenGame.sprite.bullet.Blast;
import keenGame.sprite.tile.Exit;
import keenGame.sprite.tile.Pole;
import keenGame.sprite.tile.PoleBottom;
import keenGame.sprite.tile.PoleTop;
import keenGame.sprite.tile.ShapeTile;
import keenGame.sprite.tile.Switch;
import keenGame.sprite.tile.ThinFloor;


/**
 * KeenActor is Commander Keen.  It represnts the keen sprite that
 * is controlled by the player.  Currently this class is a huge mess
 * and needs refactoring.  In future I hope to break up KeenActor
 * into many "state objects" which encompas a single state that keen
 * can be in.  This will allow me to dramatically cut down on code duplacation
 * and be much easer to modify.  I will also be able to implement substates, by 
 * inheretance.  EG, have a abstract base class, and extend it with concreate 
 * implementations for falling left, and falling right.
 */
public class KeenActor extends Player implements KeyListener
{
    //Key Constants
    private static final int KEY_UP = KeyEvent.VK_UP;
    private static final int KEY_DOWN = KeyEvent.VK_DOWN;
    private static final int KEY_LEFT = KeyEvent.VK_LEFT;
    private static final int KEY_RIGHT = KeyEvent.VK_RIGHT;
    private static final int KEY_JUMP = KeyEvent.VK_CONTROL;
    private static final int KEY_POGO = KeyEvent.VK_Z;
    private static final int KEY_FIRE = KeyEvent.VK_SPACE;
    
    //We shouldnt need this as keenGame.Keen should do it
    private static final int KEY_ESC = KeyEvent.VK_ESCAPE;

	//Used true if they key is pressed
	private boolean key_up = false;
	private boolean key_down = false;
	private boolean key_left = false;
	private boolean key_right = false;
	private boolean key_jump = false;
	private boolean key_pogo = false;
	private boolean key_fire = false;
	private boolean key_esc = false;

	//States.  Adds onto what is in movingActor
    public static final int STANDING_LEFT = 5;
    public static final int STANDING_RIGHT = 6;
    public static final int LOOKING_UP = 7;
    public static final int LOOKING_DOWN = 8;
    public static final int JUMPING_LEFT = 9;
    public static final int JUMPING_RIGHT = 10;
    public static final int FALLING_LEFT = 11;
    public static final int FALLING_RIGHT = 12;
    public static final int POGO_UP_LEFT = 13;
    public static final int POGO_UP_RIGHT = 14;
    public static final int POGO_DOWN_LEFT = 15;
    public static final int POGO_DOWN_RIGHT = 16;
    public static final int POLE_LEFT = 17;
    public static final int POLE_RIGHT = 18;
    public static final int GRIP_LEFT = 19;
    public static final int GRIP_RIGHT = 20;
    public static final int GRIP_LEFT_PULL_UP = 21;
    public static final int GRIP_RIGHT_PULL_UP = 22;
    public static final int DIEING = 23;
    public static final int WALKING_INTO_DOOR = 24;
    public static final int READING = 25;
    public static final int STARTING_READING = 26;
    public static final int STOPING_READING = 27;

	//Sub-states
    public static final int NONE = 0;
    public static final int SHOOTING = 1;
    public static final int SHOOTING_UP = 2;
    public static final int SHOOTING_DOWN = 3;
    public static final int UP = 4;
    public static final int DOWN = 5;
    public static final int PRESSING_SWITCH = 6;

	//Speed constants
	private KeenActorMovement movement = new KeenActorMovement();
	
	//Fram rate constants
	private static final int FRAME_INTERVAL = 2;
   	private static final int FALLING_FRAME_INTERVAL = 20;
   	private static final int DOUBLE_FRAME_INTERVAL = 6;
   	
   	private static final int SHOOT_STAND_LEFT_X_OFFSET = 20;
   	private static final int SHOOT_STAND_RIGHT_X_OFFSET = 22;
   	private static final int SHOOT_STAND_Y_OFFSET = 5;

   	private static final int SHOOT_STAND_UP_Y_OFFSET = 3;
   	private static final int SHOOT_STAND_UP_X_OFFSET = 18;

   	private static final int SHOOT_JUMP_LEFT_X_OFFSET = 25;
   	private static final int SHOOT_JUMP_RIGHT_X_OFFSET = 33;

   	private static final int SHOOT_JUMP_UP_DOWN_X_OFFSET = 3;
   	private static final int SHOOT_JUMP_UP_Y_OFFSET = 10;
   	private static final int SHOOT_JUMP_DOWN_Y_OFFSET = 35;
   	
   	private static final int POLE_OFFSET = 13;
   	private static final int SHOOT_POLE_LEFT_UP_X_OFFSET = 14;
   	private static final int SHOOT_POLE_RIGHT_UP_X_OFFSET = 4;
   	private static final int SHOOT_POLE_LEFT_DOWN_X_OFFSET = 11;
   	private static final int SHOOT_POLE_RIGHT_DOWN_X_OFFSET = 0;
   	private static final int SHOOT_POLE_UP_Y_OFFSET = 25;
   	private static final int SHOOT_POLE_DOWN_Y_OFFSET = 35;
   	private static final int SHOOT_POLE_RIGHT_HOROZONTAL_X_OFFSET = 27;
   	private static final int SHOOT_POLE_LEFT_HOROZONTAL_X_OFFSET = 17;
   	private static final int SHOOT_POLE_HOROZONTAL_Y_OFFSET = 3;
   	
   	private static final boolean DRAW_KEYS = false;
   	
    //Animations
    private Image [] stand = new Image[2];
    private Image [] runRight = new Image[4];
    private Image [] runLeft = new Image[4];
    private Image [] flightLeft = new Image[3];
    private Image [] flightRight = new Image[3];
    private Image [] pogoLeft = new Image[2];
    private Image [] pogoRight = new Image[2];
    private Image [] gripRight = new Image[5];
    private Image [] gripLeft = new Image[5];
    private Image [] lookDirection = new Image[3];
    private Image [] shoot = new Image[7];
    private Image [] shootPole = new Image[6];
    private Image [] poleLeft = new Image[3];
    private Image [] poleRight = new Image[3];
    private Image [] poleDown = new Image[4];
    private Image pressSwitch;
    private Image platformSurf;

        
    private int frame = 0;
    private int changeFrameCount = FRAME_INTERVAL;

	//Instance variables;
	private double fallSpeed = 0;
	private double sideSpeed = 0;
	private double jumpSpeed = 0;
	private int scrollWidth, scrollHeight;
	
	private Pole gripPole;
	private Switch pushSwitch;
	private Platform standPlatform;
	
	private boolean canJump = true;
	private boolean canReleasePogo = true;
	private boolean canPogo = true;
	private boolean canGrip = false;
	private boolean canGripPole = false;
	private boolean canPushSwitch = false;
	private boolean canKeyPushSwitch = true;
	private boolean waitStateChange = false;
	private boolean canFire = true;
	private boolean hitBottom = false;
	private boolean hitTop = false;
	private boolean isPlatformSurf = false;
	private boolean canFall = false;

	private int SubState = NONE;
	
	private String blastLayer;
	private String [] blastCollisionLayers;
	
	//Cache
	private int screenX, screenY;
    
    
	public KeenActor(String layer, 
    					String [] hitLayers, 
    					String blastLayer, String [] blastCollisionLayers,
    					Level myWorld,
    					int xValue, int yValue)    
    {
        super(layer, hitLayers, myWorld, xValue,yValue);
        init(blastLayer, blastCollisionLayers);
    }
    
    public KeenActor(XmlElement spriteElement, Level myLevel)
    {
    	super(spriteElement, myLevel);
    	
    	String newBlastLayer = spriteElement.getChild("blastLayer").getText();
    	
    	//get layers
		XmlElement [] newBlastLayersElement = spriteElement.getChild("blastHitLayers").getChildren();
    	String [] newBlastLayers = new String[newBlastLayersElement.length];
    	
    	for(int i = 0; i < newBlastLayers.length; i++)
    	{
    		newBlastLayers[i] = newBlastLayersElement[i].getText();
    	}
    	
    	init(newBlastLayer, newBlastLayers);
    }
    
    private void init(String blastLayer, String [] blastCollisionLayers)
    {
        this.blastLayer = blastLayer;
        this.blastCollisionLayers = blastCollisionLayers;
        
        //Load up the images
		//Stand
        stand[0] = Cache.getImage("graphics/keen/keenStandRight.png");
        stand[1] = Cache.getImage("graphics/keen/keenStandLeft.png");

        //runRight
        runRight[0] = Cache.getImage("graphics/keen/keenRunRight0.png");
        runRight[1] = Cache.getImage("graphics/keen/keenRunRight1.png");
        runRight[2] = Cache.getImage("graphics/keen/keenRunRight2.png");
        runRight[3] = Cache.getImage("graphics/keen/keenRunRight3.png");

        //runLeft
        runLeft[0] = Cache.getImage("graphics/keen/keenRunLeft0.png");
        runLeft[1] = Cache.getImage("graphics/keen/keenRunLeft1.png");
        runLeft[2] = Cache.getImage("graphics/keen/keenRunLeft2.png");
        runLeft[3] = Cache.getImage("graphics/keen/keenRunLeft3.png");
        
        //flightRight
        flightRight[0] = Cache.getImage("graphics/keen/keenJumpRight.png");
        flightRight[1] = Cache.getImage("graphics/keen/keenFallRight0.png");
        flightRight[2] = Cache.getImage("graphics/keen/keenFallRight1.png");

        //flightLeft
        flightLeft[0] = Cache.getImage("graphics/keen/keenJumpLeft.png");
        flightLeft[1] = Cache.getImage("graphics/keen/keenFallLeft0.png");
        flightLeft[2] = Cache.getImage("graphics/keen/keenFallLeft1.png");
        
		//gripLeft
        gripLeft[0] = Cache.getImage("graphics/keen/keenGripLeft0.png");
        gripLeft[1] = Cache.getImage("graphics/keen/keenGripLeft1.png");
        gripLeft[2] = Cache.getImage("graphics/keen/keenGripLeft2.png");
        gripLeft[3] = Cache.getImage("graphics/keen/keenGripLeft3.png");
        gripLeft[4] = Cache.getImage("graphics/keen/keenGripLeft4.png");

		//gripRight
        gripRight[0] = Cache.getImage("graphics/keen/keenGripRight0.png");
        gripRight[1] = Cache.getImage("graphics/keen/keenGripRight1.png");
        gripRight[2] = Cache.getImage("graphics/keen/keenGripRight2.png");
        gripRight[3] = Cache.getImage("graphics/keen/keenGripRight3.png");
        gripRight[4] = Cache.getImage("graphics/keen/keenGripRight4.png");
        
        //PogoLeft
        pogoLeft[0] = Cache.getImage("graphics/keen/keenPogoLeftCompress.png");
        pogoLeft[1] = Cache.getImage("graphics/keen/keenPogoLeft.png");

        //PogoRight
        pogoRight[0] = Cache.getImage("graphics/keen/keenPogoRightCompress.png");
        pogoRight[1] = Cache.getImage("graphics/keen/keenPogoRight.png");
        
        //Look direction
        lookDirection[0] = Cache.getImage("graphics/keen/keenLookDown0.png");
        lookDirection[1] = Cache.getImage("graphics/keen/keenLookDown1.png");
        lookDirection[2] = Cache.getImage("graphics/keen/keenLookUp.png");
        
        //Shooting
        shoot[0] = Cache.getImage("graphics/keen/keenShootLeft.png");
        shoot[1] = Cache.getImage("graphics/keen/keenShootRight.png");
        shoot[2] = Cache.getImage("graphics/keen/keenShootUp.png");
        shoot[3] = Cache.getImage("graphics/keen/keenJumpShootLeft.png");
        shoot[4] = Cache.getImage("graphics/keen/keenJumpShootUp.png");
        shoot[5] = Cache.getImage("graphics/keen/keenJumpShootRight.png");
        shoot[6] = Cache.getImage("graphics/keen/keenJumpShootDown.png");
        
        //Pole Left
        poleLeft[0] = Cache.getImage("graphics/keen/keenPoleLeft0.png");
        poleLeft[1] = Cache.getImage("graphics/keen/keenPoleLeft1.png");
        poleLeft[2] = Cache.getImage("graphics/keen/keenPoleLeft2.png");

		//Pole right
        poleRight[0] = Cache.getImage("graphics/keen/keenPoleRight0.png");
        poleRight[1] = Cache.getImage("graphics/keen/keenPoleRight1.png");
        poleRight[2] = Cache.getImage("graphics/keen/keenPoleRight2.png");
        
        //Pole down
        poleDown[0] = Cache.getImage("graphics/keen/keenPoleDown0.png");
        poleDown[1] = Cache.getImage("graphics/keen/keenPoleDown1.png");
        poleDown[2] = Cache.getImage("graphics/keen/keenPoleDown2.png");
        poleDown[3] = Cache.getImage("graphics/keen/keenPoleDown3.png");
        
        //Pole Shoot
        shootPole[0] = Cache.getImage("graphics/keen/keenPoleShootUpLeft.png");
        shootPole[1] = Cache.getImage("graphics/keen/keenPoleShootLeft.png");
        shootPole[2] = Cache.getImage("graphics/keen/keenPoleShootDownLeft.png");
        shootPole[3] = Cache.getImage("graphics/keen/keenPoleShootUpRight.png");
        shootPole[4] = Cache.getImage("graphics/keen/keenPoleShootRight.png");
        shootPole[5] = Cache.getImage("graphics/keen/keenPoleShootDownRight.png");

		//PressSwitch
		pressSwitch = Cache.getImage("graphics/keen/keenPressSwitch.png");

		//PressSwitch
		platformSurf = Cache.getImage("graphics/keen/keenPlatformSurf.png");
      
        changeStateTo(FALLING_RIGHT);
		fallSpeed = 1;
		sideSpeed = 0;
		
		width = stand[0].getWidth(null);
        height = stand[0].getHeight(null);
        image = stand[0];
        hitArea = new Rectangle(x, y, width, height);
    }
    
    public void setMovement(KeenActorMovement newMovement)
    {
    	movement = newMovement;
    }
    
	public Element toXml(Document xmlDoc)
	{
		return addToElement(xmlDoc, xmlDoc.createElement("KeenActor"));
	}
	
	protected Element addToElement(Document xmlDoc, Element element)
    {
    	super.addToElement(xmlDoc, element);
    	
		//blast layer
    	Element blastElement = xmlDoc.createElement("blastLayer");
		blastElement.appendChild(xmlDoc.createTextNode(blastLayer));
		element.appendChild(blastElement);
		
		//add parent node
		Element hit = xmlDoc.createElement("blastHitLayers");
		element.appendChild(hit);
		
		//loop around, addin in
		for (int i = 0; i < blastCollisionLayers.length; i++)
		{
			Element layerElement = xmlDoc.createElement("layer");
			layerElement.appendChild(xmlDoc.createTextNode(blastCollisionLayers[i]));
			hit.appendChild(layerElement);
		}
		
    	return element;
    }
	
    public void update()
    {
		updateState();
		updateKeen();
    }
	
	public boolean attemptMove(int moveX, int moveY)
	{
		if(moveY != y)
		{
			hitBottom = false;
			canGripPole = false;
			hitTop = false;
			canPushSwitch = false;
			canFall = false;
				
			if((standPlatform != null)) //dont disconnect if we are not going to reconect
			{
				standPlatform.disconnect();
				isPlatformSurf = false;
				standPlatform = null;
			}
		}
		return super.attemptMove(moveX, moveY);
	}
	
	public void movePlatform(int speed, int direction)
	{
		hitBottom = false;
		canGripPole = false;
		hitTop = false;
		canPushSwitch = false;
		
		boolean stayOn = false;
		
    	switch(direction) //This case statement can be optmised more
        {
            case NORTH:
                stayOn = super.attemptMove(x,y - speed);
                break;
            case NORTH_EAST:
                stayOn =  super.attemptMove(x + speed,y - speed);
                break;
            case EAST:
                stayOn =  super.attemptMove(x + speed,y);
                break;
            case SOUTH_EAST:
                stayOn =  super.attemptMove(x + speed,y + speed);
                break;
            case SOUTH:
                stayOn =  super.attemptMove(x,y + speed);
                break;
            case SOUTH_WEST:
                stayOn =  super.attemptMove(x - speed,y + speed);
                break;
            case WEST:
                stayOn =  super.attemptMove(x - speed,y);
                break;
            case NORTH_WEST:
                stayOn =  super.attemptMove(x -  speed,y - speed);
                break;
        }
        
        if(!stayOn)
        {
        	super.attemptMove(x,y+1);
			standPlatform.disconnect();
			isPlatformSurf = false;
			standPlatform = null;
			if(State == STANDING_LEFT)
			{
				changeStateTo(FALLING_LEFT);
			}
			else
			{
				changeStateTo(FALLING_LEFT);
			}
			fallSpeed = 1;
			sideSpeed = 0;
        }
	}
	
   
    private void changeStateTo(int newState)
    {
    	if(State != newState)
    	{
	    	State = newState;
	    	SubState = NONE;
	    	frame = 0;
	    	changeFrameCount = 0;
	    	waitStateChange = false;
    	}
    }
    
    private void changeSubStateTo(int newState)
    {
    	if(SubState != newState)
    	{
	    	SubState = newState;
	    	frame = 0;
	    	changeFrameCount = 0;
	    }
    }    
   	
    /**
     * Updates keen based on the current state
     */
    private void updateKeen()
    {
    	//Update of keen based on current state ******************
		if((State == WALKING_LEFT) && (SubState != SHOOTING))
		{
			if(!walk(WALKING_LEFT,movement.WALK_SPEED)) //We are falling
			{
				changeStateTo(FALLING_LEFT);
				fallSpeed = 1;
				sideSpeed = 0;
			}
		}
		else if((State == WALKING_RIGHT) && (SubState != SHOOTING))
		{
			if(!walk(WALKING_RIGHT,movement.WALK_SPEED)) //We are falling
			{
				changeStateTo(FALLING_RIGHT);
				fallSpeed = 1;
				sideSpeed = 0;
			}
		}
		else if(((State == STANDING_RIGHT) || (State == STANDING_LEFT)) && (SubState == PRESSING_SWITCH))
		{
			if(x > pushSwitch.getX())
			{
				attemptMove(x - 1, y);
			}
			else if(x < pushSwitch.getX())
			{
				attemptMove(x + 1, y);
			}
		}
		else if(State == FALLING_RIGHT) //*********************************** Jump/Fall
		{
			fallSpeed = fall(fallSpeed);
			canGrip = true;
			attemptMove(x + (int)sideSpeed,y);
			canGrip = false;
			
			//Have we hit something?
			if(fallSpeed == 0)
			{
				changeStateTo(STANDING_RIGHT);
			}
		}
		else if(State == FALLING_LEFT)
		{
			fallSpeed = fall(fallSpeed);
						
			canGrip = true;
			attemptMove(x + (int)sideSpeed,y);
			canGrip = false;
			
			//Have we hit something?
			if(fallSpeed == 0)
			{
				changeStateTo(STANDING_LEFT);
			}
		}
		else if(State == JUMPING_LEFT)
		{
			if(!attemptMove(x,y - (int)jumpSpeed))
			{
				changeStateTo(FALLING_LEFT);
				fallSpeed = 1;
			}
			if(!attemptMove(x + (int)sideSpeed,y))
			{
				sideSpeed = 0;
			}
			if(jumpSpeed < 0)
			{
				changeStateTo(FALLING_LEFT);
				fallSpeed = 1;
			}
		}
		else if(State == JUMPING_RIGHT)
		{
			if(!attemptMove(x,y - (int)jumpSpeed))
			{
				changeStateTo(FALLING_RIGHT);
				fallSpeed = 1;
			}
			if(!attemptMove(x + (int)sideSpeed,y))
			{
				sideSpeed = 0;
			}
			if(jumpSpeed < 0)			
			{
				changeStateTo(FALLING_RIGHT);
				fallSpeed = 1;
			}
		}
		else if(State == POGO_DOWN_RIGHT) //********************************* Pogo
		{
			fallSpeed = fall(fallSpeed);
			attemptMove(x + (int)sideSpeed,y);
			//Have we hit something?
			if(fallSpeed == 0)
			{
				changeStateTo(POGO_UP_RIGHT);
				if(key_jump)
				{
					jumpSpeed = movement.POGO_BOOST_SPEED;
				}
				else
				{
					jumpSpeed = movement.POGO_SPEED;
				}
			}
		}
		else if(State == POGO_DOWN_LEFT)
		{
			fallSpeed = fall(fallSpeed);
			attemptMove(x + (int)sideSpeed,y);
			//Have we hit something?
			if(fallSpeed == 0)
			{
				changeStateTo(POGO_UP_LEFT);
				if(key_jump)
				{
					jumpSpeed = movement.POGO_BOOST_SPEED;
				}
				else
				{
					jumpSpeed = movement.POGO_SPEED;
				}
			}
		}
		else if(State == POGO_UP_LEFT)
		{
			if(!attemptMove(x,y - (int)jumpSpeed))
			{
				changeStateTo(POGO_DOWN_LEFT);
				fallSpeed = 1;
			}
			if(!attemptMove(x + (int)sideSpeed,y))
			{
				sideSpeed = 0;
			}
			if(jumpSpeed < 0)
			{
				changeStateTo(POGO_DOWN_LEFT);
				fallSpeed = 1;
			}
		}
		else if(State == POGO_UP_RIGHT)
		{
			if(!attemptMove(x,y - (int)jumpSpeed))
			{
				changeStateTo(POGO_DOWN_RIGHT);
				fallSpeed = 1;
			}
			if(!attemptMove(x + (int)sideSpeed,y))
			{
				sideSpeed = 0;
			}
			if(jumpSpeed < 0)			
			{
				changeStateTo(POGO_DOWN_RIGHT);
				fallSpeed = 1;
			}
		}
		else if(((State == POLE_LEFT) || (State == POLE_RIGHT)) && SubState == UP)
		{
			if(!attemptMove(x,y - (int)movement.POLE_UP_SPEED))
			{
				changeSubStateTo(NONE);
			}
			if(hitTop)
			{
				changeSubStateTo(NONE);
			}
		}
		else if((State == POLE_LEFT) && SubState == DOWN)
		{
			if(!attemptMove(x,y + (int)movement.POLE_DOWN_SPEED))
			{
				changeStateTo(STANDING_LEFT);
			}
			else if(hitBottom)
			{
				changeStateTo(FALLING_LEFT);
				fallSpeed = 1;
				sideSpeed = 0;
			}
		}
		else if((State == POLE_RIGHT) && SubState == DOWN)
		{
			if(!attemptMove(x,y + (int)movement.POLE_DOWN_SPEED))
			{
				changeStateTo(STANDING_RIGHT);
			}
			else if(hitBottom)
			{
				changeStateTo(FALLING_RIGHT);
				fallSpeed = 1;
				sideSpeed = 0;
			}
		}

		else if((State == STANDING_LEFT || (State == STANDING_RIGHT)) && SubState == DOWN)
		{
			//if(((myWorld.getViewY()+movement.SCROLL_SPEED + scrollHeight) < y) &&
			//  ((myWorld.getViewY()+myWorld.getViewportHeight() + movement.SCROLL_SPEED) < myWorld.HEIGHT))
			//{
			//myWorld.setViewY(myWorld.getViewY() + movement.SCROLL_SPEED);
			//}
			myWorld.scrollDown();
		}
		else if((State == STANDING_LEFT || (State == STANDING_RIGHT)) && (SubState == UP || SubState == SHOOTING_UP))
		{
			//if((((myWorld.getViewY()+myWorld.getViewportHeight()) - scrollHeight - movement.SCROLL_SPEED) > y) &&
			//  ((myWorld.getViewY()- movement.SCROLL_SPEED) > 0))
			//{
			//myWorld.setViewY(myWorld.getViewY() - movement.SCROLL_SPEED);
			//}
			myWorld.scrollUp();
		}
    }
    
    /**
     * Updates the state of keen based on which keys are currently pressed
     */
    private void updateState()
    {
    	//System.err.println("start updateState() " + canGripPole);
    	//Change of states based on keypresses *******************
    	//if(key_esc)
		//{
			//System.exit(0);
		//}
		if((State == STANDING_LEFT) && canFall && key_down && key_jump)
		{
				if(standPlatform != null)
				{
					standPlatform.disconnect();
					isPlatformSurf = false;
					standPlatform = null;
				}
				changeStateTo(FALLING_LEFT);
				fallSpeed = 1;
				sideSpeed = 0;
				
				y = y + 1 ;
				((Rectangle)hitArea).y = y;
		}
		else if((State == STANDING_RIGHT) && canFall && key_down && key_jump)
		{
				if(standPlatform != null)
				{
					standPlatform.disconnect();
					isPlatformSurf = false;
					standPlatform = null;
				}
				changeStateTo(FALLING_RIGHT);
				fallSpeed = 1;
				sideSpeed = 0;
				
				y = y + 1 ;
				((Rectangle)hitArea).y = y;
		}
		else if(((State == STANDING_LEFT) || (State == JUMPING_LEFT) ||
				 (State == FALLING_LEFT)) && canGripPole && (key_up || (key_down && canFall)) &&  !waitStateChange && !hitTop)
		{
			changeStateTo(POLE_LEFT);
			x = gripPole.getX() - POLE_OFFSET;
			//System.err.println("grip pole left");
		}
		else if(((State == STANDING_RIGHT) || (State == JUMPING_RIGHT) ||
				 (State == FALLING_RIGHT)) && canGripPole && (key_up || (key_down && canFall)) &&  !waitStateChange && !hitTop) //waitStateChange
		{
			changeStateTo(POLE_RIGHT);
			x = gripPole.getX() - POLE_OFFSET;
			//System.err.println("grip pole left");

		}
		else if(((State == STANDING_LEFT) || (State == STANDING_RIGHT))
		  && key_up && SubState == NONE && canPushSwitch && canKeyPushSwitch)
		{
			pushSwitch.toggle();
			changeSubStateTo(PRESSING_SWITCH);
			canKeyPushSwitch = false;
		}
		else if(((State == STANDING_LEFT) || (State == STANDING_RIGHT))
		  && key_left && SubState == NONE)
		{
			changeStateTo(WALKING_LEFT);
		}
		else if(((State == STANDING_LEFT) || (State == STANDING_RIGHT))
				&& key_right && SubState == NONE)
		{
			changeStateTo(WALKING_RIGHT);

		} //************************************************************ Pogoing
		else if(((State == POGO_UP_LEFT) || (State == POGO_DOWN_LEFT)) && key_fire && canFire)
		{
			if(State == POGO_UP_LEFT)
			{
				changeStateTo(JUMPING_LEFT);
			}
			else
			{
				changeStateTo(FALLING_LEFT);
			}			
			canPogo = false;
						
			if(key_up)
			{
				myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_UP_DOWN_X_OFFSET,y - SHOOT_JUMP_UP_Y_OFFSET,NORTH));
				changeSubStateTo(SHOOTING_UP);
			}
			else if(key_down)
			{
				myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_UP_DOWN_X_OFFSET,y + SHOOT_JUMP_DOWN_Y_OFFSET,SOUTH));
				changeSubStateTo(SHOOTING_DOWN);
			}
			else
			{
				myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x - SHOOT_JUMP_LEFT_X_OFFSET,y + SHOOT_STAND_Y_OFFSET,WEST));
				changeSubStateTo(SHOOTING);
			}
			canFire = false;
		}
		else if(((State == POGO_UP_RIGHT) || (State == POGO_DOWN_RIGHT)) && key_fire && canFire)
		{
			if(State == POGO_UP_RIGHT)
			{
				changeStateTo(JUMPING_RIGHT);
			}
			else
			{
				changeStateTo(FALLING_RIGHT);
			}			
			canPogo = false;
						
			if(key_up)
			{
				myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_UP_DOWN_X_OFFSET,y - SHOOT_JUMP_UP_Y_OFFSET,NORTH));
				changeSubStateTo(SHOOTING_UP);
			}
			else if(key_down)
			{
				myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_UP_DOWN_X_OFFSET,y + SHOOT_JUMP_DOWN_Y_OFFSET,SOUTH));
				changeSubStateTo(SHOOTING_DOWN);
			}
			else
			{
				myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_RIGHT_X_OFFSET,y + SHOOT_STAND_Y_OFFSET,EAST));
				changeSubStateTo(SHOOTING);
			}
			canFire = false;
		}

		else if((State == POGO_UP_LEFT) && key_pogo && canReleasePogo)
		{
			changeStateTo(JUMPING_LEFT);
			canPogo = false;
		}
		else if((State == POGO_UP_RIGHT) && key_pogo && canReleasePogo)
		{
			changeStateTo(JUMPING_RIGHT);
			canPogo = false;
		}
		else if((State == POGO_DOWN_LEFT) && key_pogo && canReleasePogo)
		{
			changeStateTo(FALLING_LEFT);
			canPogo = false;
		}
		else if((State == POGO_DOWN_RIGHT) && key_pogo && canReleasePogo)
		{
			changeStateTo(FALLING_RIGHT);
			canPogo = false;
		} 
		else if((State == STANDING_LEFT) && key_pogo && canPogo && SubState == NONE)
		{
			changeStateTo(POGO_UP_LEFT);
			canReleasePogo = false;
			jumpSpeed = movement.POGO_SPEED;
			sideSpeed = 0;
		}
		else if((State == STANDING_RIGHT) && key_pogo && canPogo && SubState == NONE)
		{
			changeStateTo(POGO_UP_RIGHT);
			canReleasePogo = false;
			jumpSpeed = movement.POGO_SPEED;
			sideSpeed = 0;
		}
		else if((State == WALKING_LEFT) && key_pogo && canPogo)
		{
			changeStateTo(POGO_UP_LEFT);
			canReleasePogo = false;
			jumpSpeed = movement.POGO_SPEED;
			sideSpeed = -movement.WALK_SPEED;
		}
		else if((State == WALKING_RIGHT) && key_pogo && canPogo)
		{
			changeStateTo(POGO_UP_RIGHT);
			canReleasePogo = false;
			sideSpeed = movement.WALK_SPEED;
			jumpSpeed = movement.POGO_SPEED;
		}
		else if((State == JUMPING_RIGHT) && key_pogo && canPogo)
		{
			changeStateTo(POGO_UP_RIGHT);
			canReleasePogo = false;
		}
		else if((State == JUMPING_LEFT) && key_pogo && canPogo)
		{
			changeStateTo(POGO_UP_LEFT);
			canReleasePogo = false;
		}
		else if((State == FALLING_RIGHT) && key_pogo && canPogo)
		{
			changeStateTo(POGO_DOWN_RIGHT);
			canReleasePogo = false;
		}
		else if((State == FALLING_LEFT) && key_pogo && canPogo)
		{
			changeStateTo(POGO_DOWN_LEFT);
			canReleasePogo = false;
		}
		else if(((State == POGO_UP_LEFT) || (State == POGO_UP_RIGHT)) && key_left)
		{
			sideSpeed = sideSpeed - movement.POGO_SIDE_ACCELERATION;
			changeStateTo(POGO_UP_LEFT);
			if(sideSpeed < -movement.POGO_MAX_SIDE_SPEED)
			{
				sideSpeed = -movement.POGO_MAX_SIDE_SPEED;
			}
			
			if(key_jump)
			{
				jumpSpeed = jumpSpeed - movement.POGO_BOOST_DECCELERATION;
			}
			else
			{
				jumpSpeed = jumpSpeed - movement.POGO_DECCELERATION;
			}
		}
		else if(((State == POGO_UP_LEFT) || (State == POGO_UP_RIGHT)) && key_right)
		{
			sideSpeed = sideSpeed + movement.POGO_SIDE_ACCELERATION;
			changeStateTo(POGO_UP_RIGHT);
			if(sideSpeed > movement.POGO_MAX_SIDE_SPEED)
			{
				sideSpeed = movement.POGO_MAX_SIDE_SPEED;
			}

			if(key_jump)
			{
				jumpSpeed = jumpSpeed - movement.POGO_BOOST_DECCELERATION;
			}
			else
			{
				jumpSpeed = jumpSpeed - movement.POGO_DECCELERATION;
			}
		}
		else if(((State == POGO_DOWN_LEFT) || (State == POGO_DOWN_RIGHT)) && key_left)
		{
			sideSpeed = sideSpeed - movement.POGO_SIDE_ACCELERATION;
			changeStateTo(POGO_DOWN_LEFT);
			if(sideSpeed < -movement.POGO_MAX_SIDE_SPEED)
			{
				sideSpeed = -movement.POGO_MAX_SIDE_SPEED;
			}
		}
		else if(((State == POGO_DOWN_LEFT) || (State == POGO_DOWN_RIGHT)) && key_right)
		{
			sideSpeed = sideSpeed + movement.POGO_SIDE_ACCELERATION;
			changeStateTo(POGO_DOWN_RIGHT);
			if(sideSpeed > movement.POGO_MAX_SIDE_SPEED)
			{
				sideSpeed = movement.POGO_MAX_SIDE_SPEED;
			}
		}
		else if(((State == POGO_UP_RIGHT) || (State == POGO_UP_LEFT)) && key_jump)
		{
			jumpSpeed = jumpSpeed - movement.POGO_BOOST_DECCELERATION;
		}
		else if(((State == POGO_UP_RIGHT) || (State == POGO_UP_LEFT)) && !key_jump)
		{
			jumpSpeed = jumpSpeed - movement.POGO_DECCELERATION;
		}  //**************************************************** Jumping / Falling
		else if((State == STANDING_LEFT) && key_jump && canJump && SubState == NONE)
		{
			changeStateTo(JUMPING_LEFT);
			jumpSpeed = movement.JUMP_SPEED;
			sideSpeed = 0;
		}
		else if((State == STANDING_RIGHT) && key_jump && canJump && SubState == NONE)
		{
			changeStateTo(JUMPING_RIGHT);
			jumpSpeed = movement.JUMP_SPEED;
			sideSpeed = 0;
		}
		else if((State == WALKING_LEFT) && key_jump && canJump)
		{
			changeStateTo(JUMPING_LEFT);
			jumpSpeed = movement.JUMP_SPEED;
			sideSpeed = -movement.WALK_SPEED;
		}
		else if((State == WALKING_RIGHT) && key_jump && canJump)
		{
			changeStateTo(JUMPING_RIGHT);
			sideSpeed = movement.WALK_SPEED;
			jumpSpeed = movement.JUMP_SPEED;
		}
		else if(((State == JUMPING_LEFT) || (State == JUMPING_RIGHT)) && key_left)
		{
			sideSpeed = sideSpeed - movement.SIDE_ACCELERATION;
			changeStateTo(JUMPING_LEFT);
			if(sideSpeed < -movement.WALK_SPEED)
			{
				sideSpeed = -movement.WALK_SPEED;
			}
			
			if(key_jump)
			{
				jumpSpeed = jumpSpeed - movement.JUMP_BOOST_DECCELERATION;
				canJump = false;
			}
			else
			{
				jumpSpeed = jumpSpeed - movement.JUMP_DECCELERATION;
			}
			
			if(key_fire && canFire)
			{
				if(key_up)
				{
					myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_UP_DOWN_X_OFFSET,y - SHOOT_JUMP_UP_Y_OFFSET,NORTH));
					changeSubStateTo(SHOOTING_UP);
				}
				else if(key_down)
				{
					myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_UP_DOWN_X_OFFSET,y + SHOOT_JUMP_DOWN_Y_OFFSET,SOUTH));
					changeSubStateTo(SHOOTING_DOWN);
				}
				else
				{
					if(State == JUMPING_LEFT)
					{
						myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x - SHOOT_JUMP_LEFT_X_OFFSET,y + SHOOT_STAND_Y_OFFSET,WEST));
					}
					else if(State == JUMPING_RIGHT)
					{
						myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_RIGHT_X_OFFSET,y + SHOOT_STAND_Y_OFFSET,EAST));
					}				
					changeSubStateTo(SHOOTING);
				}
				canFire = false;
			}
		}
		else if(((State == JUMPING_LEFT) || (State == JUMPING_RIGHT)) && key_right)
		{
			sideSpeed = sideSpeed + movement.SIDE_ACCELERATION;
			changeStateTo(JUMPING_RIGHT);
			if(sideSpeed > movement.WALK_SPEED)
			{
				sideSpeed = movement.WALK_SPEED;
			}
			if(key_jump)
			{
				jumpSpeed = jumpSpeed - movement.JUMP_BOOST_DECCELERATION;
				canJump = false;
			}
			else
			{
				jumpSpeed = jumpSpeed - movement.JUMP_DECCELERATION;
			}
			if(key_fire && canFire)
			{
				if(key_up)
				{
					myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_UP_DOWN_X_OFFSET,y - SHOOT_JUMP_UP_Y_OFFSET,NORTH));
					changeSubStateTo(SHOOTING_UP);
				}
				else if(key_down)
				{
					myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_UP_DOWN_X_OFFSET,y + SHOOT_JUMP_DOWN_Y_OFFSET,SOUTH));
					changeSubStateTo(SHOOTING_DOWN);
				}
				else
				{
					if(State == JUMPING_LEFT)
					{
						myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x - SHOOT_JUMP_LEFT_X_OFFSET,y + SHOOT_STAND_Y_OFFSET,WEST));
					}
					else if(State == JUMPING_RIGHT)
					{
						myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_RIGHT_X_OFFSET,y + SHOOT_STAND_Y_OFFSET,EAST));
					}				
					changeSubStateTo(SHOOTING);
				}
				canFire = false;
			}
		}
		else if(((State == FALLING_LEFT) || (State == FALLING_RIGHT)) && key_left)
		{
			sideSpeed = sideSpeed - movement.SIDE_ACCELERATION;
			changeStateTo(FALLING_LEFT);
			if(sideSpeed < -movement.WALK_SPEED)
			{
				sideSpeed = -movement.WALK_SPEED;
			}
			if(key_fire && canFire)
			{
				if(key_up)
				{
					
					myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_UP_DOWN_X_OFFSET,y - SHOOT_JUMP_UP_Y_OFFSET,NORTH));
					changeSubStateTo(SHOOTING_UP);
				}
				else if(key_down)
				{
					myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_UP_DOWN_X_OFFSET,y + SHOOT_JUMP_DOWN_Y_OFFSET,SOUTH));
					changeSubStateTo(SHOOTING_DOWN);
				}
				else
				{
					if(State == FALLING_LEFT)
					{
						myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x - SHOOT_JUMP_LEFT_X_OFFSET,y + SHOOT_STAND_Y_OFFSET,WEST));
					}
					else if(State == FALLING_RIGHT)
					{
						myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_RIGHT_X_OFFSET,y + SHOOT_STAND_Y_OFFSET,EAST));
					}				
					changeSubStateTo(SHOOTING);
				}
				canFire = false;
			}
		}
		else if(((State == FALLING_LEFT) || (State == FALLING_RIGHT)) && key_right)
		{
			sideSpeed = sideSpeed + movement.SIDE_ACCELERATION;
			changeStateTo(FALLING_RIGHT);
			if(sideSpeed > movement.WALK_SPEED)
			{
				sideSpeed = movement.WALK_SPEED;
			}
			if(key_fire && canFire)
			{
				if(key_up)
				{
					myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_UP_DOWN_X_OFFSET,y - SHOOT_JUMP_UP_Y_OFFSET,NORTH));
					changeSubStateTo(SHOOTING_UP);
				}
				else if(key_down)
				{
					myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_UP_DOWN_X_OFFSET,y + SHOOT_JUMP_DOWN_Y_OFFSET,SOUTH));
					changeSubStateTo(SHOOTING_DOWN);
				}
				else
				{	
					if(State == FALLING_LEFT)
					{
						myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x - SHOOT_JUMP_LEFT_X_OFFSET,y + SHOOT_STAND_Y_OFFSET,WEST));
					}
					else if(State == FALLING_RIGHT)
					{
						myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_RIGHT_X_OFFSET,y + SHOOT_STAND_Y_OFFSET,EAST));
					}				
					changeSubStateTo(SHOOTING);
				}
				canFire = false;
			}
		}
		else if(((State == JUMPING_RIGHT) || (State == JUMPING_LEFT)) && key_jump)
		{
			jumpSpeed = jumpSpeed - movement.JUMP_BOOST_DECCELERATION;
			canJump = false;
			if(key_fire && canFire)
			{
				if(key_up)
				{
					myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_UP_DOWN_X_OFFSET,y - SHOOT_JUMP_UP_Y_OFFSET,NORTH));
					changeSubStateTo(SHOOTING_UP);
				}
				else if(key_down)
				{
					myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_UP_DOWN_X_OFFSET,y + SHOOT_JUMP_DOWN_Y_OFFSET,SOUTH));
					changeSubStateTo(SHOOTING_DOWN);
				}
				else
				{
					if(State == JUMPING_LEFT)
					{
						myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x - SHOOT_JUMP_LEFT_X_OFFSET,y + SHOOT_STAND_Y_OFFSET,WEST));
					}
					else if(State == JUMPING_RIGHT)
					{
						myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_RIGHT_X_OFFSET,y + SHOOT_STAND_Y_OFFSET,EAST));
					}				
					changeSubStateTo(SHOOTING);
				}
				canFire = false;
			}

		}
		else if(((State == FALLING_RIGHT) || (State == FALLING_LEFT)) && key_jump)
		{
			canJump = false;
			if(key_fire && canFire)
			{
				if(key_up)
				{
					myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_UP_DOWN_X_OFFSET,y - SHOOT_JUMP_UP_Y_OFFSET,NORTH));
					changeSubStateTo(SHOOTING_UP);
				}
				else if(key_down)
				{
					myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_UP_DOWN_X_OFFSET,y + SHOOT_JUMP_DOWN_Y_OFFSET,SOUTH));
					changeSubStateTo(SHOOTING_DOWN);
				}
				else
				{
					if(State == FALLING_LEFT)
					{
						myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x - SHOOT_JUMP_LEFT_X_OFFSET,y + SHOOT_STAND_Y_OFFSET,WEST));
					}
					else if(State == FALLING_RIGHT)
					{
						myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_RIGHT_X_OFFSET,y + SHOOT_STAND_Y_OFFSET,EAST));
					}				
					changeSubStateTo(SHOOTING);
				}
				canFire = false;
			}

		}
		else if(((State == JUMPING_RIGHT) || (State == JUMPING_LEFT)) && !key_jump)
		{
			jumpSpeed = jumpSpeed - movement.JUMP_DECCELERATION;
			if(key_fire && canFire)
			{
				if(key_up)
				{
					myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_UP_DOWN_X_OFFSET,y - SHOOT_JUMP_UP_Y_OFFSET,NORTH));
					changeSubStateTo(SHOOTING_UP);
				}
				else if(key_down)
				{
					myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_UP_DOWN_X_OFFSET,y + SHOOT_JUMP_DOWN_Y_OFFSET,SOUTH));
					changeSubStateTo(SHOOTING_DOWN);
				}
				else
				{
					if(State == JUMPING_LEFT)
					{
						myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x - SHOOT_JUMP_LEFT_X_OFFSET,y + SHOOT_STAND_Y_OFFSET,WEST));
					}
					else if(State == JUMPING_RIGHT)
					{
						myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_RIGHT_X_OFFSET,y + SHOOT_STAND_Y_OFFSET,EAST));
					}				
					changeSubStateTo(SHOOTING);
				}
				canFire = false;
			}
		}
		else if(((State == FALLING_LEFT) || (State == FALLING_RIGHT) ||
				  (State == JUMPING_LEFT) || (State == JUMPING_RIGHT)) &&
				  (!key_left || !key_right))
		{
			if(sideSpeed > 0)
			{
				sideSpeed = sideSpeed - movement.SIDE_ACCELERATION;
			}
			else if(sideSpeed < 0)
			{
				sideSpeed = sideSpeed + movement.SIDE_ACCELERATION;
			}

			if(key_fire && canFire)
			{
				if(key_up)
				{
					myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_UP_DOWN_X_OFFSET,y - SHOOT_JUMP_UP_Y_OFFSET,NORTH));
					changeSubStateTo(SHOOTING_UP);
				}
				else if(key_down)
				{
					myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_UP_DOWN_X_OFFSET,y + SHOOT_JUMP_DOWN_Y_OFFSET,SOUTH));
					changeSubStateTo(SHOOTING_DOWN);
				}
				else
				{
					if((State == JUMPING_LEFT) ||(State == FALLING_LEFT))
					{
						myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x - SHOOT_JUMP_LEFT_X_OFFSET,y + SHOOT_STAND_Y_OFFSET,WEST));
					}
					else if((State == JUMPING_RIGHT) ||(State == FALLING_RIGHT))
					{
						myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_JUMP_RIGHT_X_OFFSET,y + SHOOT_STAND_Y_OFFSET,EAST));
					}				
					changeSubStateTo(SHOOTING);
				}
				canFire = false;
			}
		}//********************************************************************************** POLE
		else if((State == POLE_LEFT) && key_jump && canJump)
		{
			changeStateTo(JUMPING_LEFT);
			jumpSpeed = movement.POLE_JUMP_SPEED;
			sideSpeed = 0;
			canJump = false;
			waitStateChange = true;
		}
		else if((State == POLE_RIGHT) && key_jump && canJump)
		{
			changeStateTo(JUMPING_RIGHT);
			jumpSpeed = movement.POLE_JUMP_SPEED;
			sideSpeed = 0;
			canJump = false;
			waitStateChange = true;
			
		}
		else if((State == POLE_LEFT) && key_right)
		{
			changeStateTo(POLE_RIGHT);
		}		
		else if((State == POLE_RIGHT) && key_left)
		{
			changeStateTo(POLE_LEFT);
		}
		else if((State == POLE_LEFT)  && key_fire && canFire && key_up)
		{
			changeSubStateTo(SHOOTING_UP);
			myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_POLE_LEFT_UP_X_OFFSET,y - SHOOT_POLE_UP_Y_OFFSET,NORTH));
			canFire = false;
		}
		else if((State == POLE_RIGHT)  && key_fire && canFire && key_up)
		{
			changeSubStateTo(SHOOTING_UP);
			myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x - SHOOT_POLE_RIGHT_UP_X_OFFSET,y - SHOOT_POLE_UP_Y_OFFSET,NORTH));
			canFire = false;
		}
		else if((State == POLE_LEFT)  && key_fire && canFire && key_down)
		{
			changeSubStateTo(SHOOTING_DOWN);
			myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_POLE_LEFT_DOWN_X_OFFSET,y + SHOOT_POLE_DOWN_Y_OFFSET,SOUTH));
			canFire = false;
		}
		else if((State == POLE_RIGHT)  && key_fire && canFire && key_down)
		{
			changeSubStateTo(SHOOTING_DOWN);
			myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_POLE_RIGHT_DOWN_X_OFFSET,y + SHOOT_POLE_DOWN_Y_OFFSET,SOUTH));
			canFire = false;
		}
		else if((State == POLE_LEFT) && key_fire && canFire)
		{
			changeSubStateTo(SHOOTING);
			myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x - SHOOT_POLE_LEFT_HOROZONTAL_X_OFFSET,y + SHOOT_POLE_HOROZONTAL_Y_OFFSET,WEST));
			canFire = false;
		}
		else if((State == POLE_RIGHT) && key_fire && canFire)
		{
			changeSubStateTo(SHOOTING);
			myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_POLE_RIGHT_HOROZONTAL_X_OFFSET,y + SHOOT_POLE_HOROZONTAL_Y_OFFSET,EAST));
			canFire = false;
		}
		else if(((State == POLE_LEFT) || (State == POLE_RIGHT)) && (SubState == NONE) && key_up)
		{
			changeSubStateTo(UP);
		}
		else if(((State == POLE_LEFT) || (State == POLE_RIGHT)) && (SubState == UP) && !key_up)
		{
			changeSubStateTo(NONE);
		}
		else if(((State == POLE_LEFT) || (State == POLE_RIGHT)) && (SubState == NONE) && key_down)
		{
			changeSubStateTo(DOWN);
		}
		else if(((State == POLE_LEFT) || (State == POLE_RIGHT)) && (SubState == DOWN) && !key_down)
		{
			changeSubStateTo(NONE);
		}	
		else if((State == GRIP_LEFT) && (key_left || key_up))
		{
			changeStateTo(GRIP_LEFT_PULL_UP);
		}
		else if((State == GRIP_LEFT) && (key_right || key_down))
		{
			changeStateTo(FALLING_LEFT);
			fallSpeed = 1;
		}
		else if((State == GRIP_RIGHT) && (key_right || key_up))
		{
			changeStateTo(GRIP_RIGHT_PULL_UP);
		}
		else if((State == GRIP_RIGHT) && (key_left || key_down))
		{
			changeStateTo(FALLING_RIGHT);
			fallSpeed = 1;
		}
		else if((State == WALKING_LEFT)
				&& !key_left)
		{
			changeStateTo(STANDING_LEFT);
		}
		else if((State == WALKING_LEFT) && key_fire && canFire)
		{
			myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x - SHOOT_STAND_LEFT_X_OFFSET,y + SHOOT_STAND_Y_OFFSET,WEST));
			changeSubStateTo(SHOOTING);
			canFire = false;
		}			
		else if((State == WALKING_RIGHT)
				&& !key_right)
		{
			changeStateTo(STANDING_RIGHT);
		}
		else if((State == WALKING_RIGHT) && key_fire && canFire)
		{
			myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_STAND_RIGHT_X_OFFSET,y + SHOOT_STAND_Y_OFFSET,EAST));
			changeSubStateTo(SHOOTING);
			canFire = false;
		}			
		else if(((State == STANDING_LEFT) || (State == STANDING_RIGHT)) && key_up && SubState == NONE)
		{
			changeSubStateTo(UP);
		}
		else if(((State == STANDING_LEFT) || (State == STANDING_RIGHT)) && key_down && SubState == NONE)
		{
			changeSubStateTo(DOWN);
		}
		else if((SubState == UP)  && !key_up)
		{
			changeSubStateTo(NONE);
		}
		else if((SubState == DOWN) && !key_down)
		{
			changeSubStateTo(NONE);
		}
		else if((SubState == UP) && ((State == STANDING_LEFT) || (State == STANDING_RIGHT)) && key_up && key_fire && canFire)
		{
			myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_STAND_UP_Y_OFFSET,y - SHOOT_STAND_UP_Y_OFFSET,NORTH));
			changeSubStateTo(SHOOTING_UP);
			canFire = false;
		}
		else if((State == STANDING_LEFT) && (SubState == NONE) && key_fire && canFire)
		{
			myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x - SHOOT_STAND_LEFT_X_OFFSET,y + SHOOT_STAND_Y_OFFSET,WEST));
			changeSubStateTo(SHOOTING);
			canFire = false;
		}
		else if((State == STANDING_RIGHT) && (SubState == NONE) && key_fire && canFire)
		{
			myWorld.addSprite(new Blast(blastLayer, blastCollisionLayers, myWorld,
				x + SHOOT_STAND_RIGHT_X_OFFSET,y + SHOOT_STAND_Y_OFFSET,EAST));
			changeSubStateTo(SHOOTING);
			canFire = false;
		}
    }
    
    /*
    private void updateViewPort()
    {
    	//Update the viewport, so keen is visible ****************
		if((x > (myWorld.getViewX() + ((myWorld.getViewportWidth() - scrollWidth)- width)))
		  && (x < ((myWorld.WIDTH - scrollWidth) - width)))
		{
			myWorld.setViewX(myWorld.getViewX() +(x - (myWorld.getViewX() + ((myWorld.getViewportWidth() - scrollWidth)- width))));
		}
		else if((x < (myWorld.getViewX() +  scrollWidth))
			   &&(x > scrollWidth))
		{
			myWorld.setViewX(myWorld.getViewX()-((myWorld.getViewX() +  scrollWidth)-x));
		}
	
		if((y > (myWorld.getViewY() + ((myWorld.getViewportHeight() - scrollHeight)- height)))
		  && (y < ((myWorld.HEIGHT - scrollHeight) - height)))
		{
			myWorld.setViewY(myWorld.getViewY() +(y - (myWorld.getViewY() + ((myWorld.getViewportHeight() - scrollHeight)- height))));
		}
		else if((y < (myWorld.getViewY() +  scrollHeight))
			   &&(y > scrollHeight))
		{
			myWorld.setViewY(myWorld.getViewY()-((myWorld.getViewY() +  scrollHeight)-y));
		}
    }
    */
    
    public void setFrame()
    {
    	//Set to the correct animation sequence
    	if(changeFrameCount == 0)
	    {
	    	//Reset our frame count
   	    	changeFrameCount = FRAME_INTERVAL;

			//change the frame
	    	if(State == WALKING_RIGHT)
	    	{
				if(SubState == NONE)
				{	    	
		    		if(frame == runRight.length-1)
		    		{
		    			frame = 0;
		    		}
		    		else
		    		{
		    			frame++;
		    		}
		    		image = runRight[frame];
		    	}
	    		else if(SubState == SHOOTING)
	    		{
	    			if(frame ==0)
	    			{
	    				image = shoot[1];
	    				frame++;
	    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    			}
	    			else
	    			{
	    				image = runRight[0];
	    				changeSubStateTo(NONE);
	    			}
	    		}

	    	}
	    	else if(State == WALKING_LEFT)
	    	{
	    		if(SubState == NONE)
	    		{
					if(frame == runLeft.length-1)
					{
						frame = 0;
					}
					else
					{
						frame++;
					}
					image = runLeft[frame];
		    	}
	    		else if(SubState == SHOOTING)
	    		{
	    			if(frame ==0)
	    			{
	    				image = shoot[0];
	    				frame++;
	    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    			}
	    			else
	    			{
	    				image = runLeft[0];
	    				changeSubStateTo(NONE);
	    			}
	    		}
	    	}
	    	else if(State == FALLING_LEFT)
	    	{
	    		if(SubState == NONE)
	    		{
		    		if(frame == 0)
		    		{
		    			image = flightLeft[1];
		    			frame++;
		    		}
		    		else
		    		{
		    			image = flightLeft[2];
		    			frame = 0;
		    		}
		    		changeFrameCount = FALLING_FRAME_INTERVAL;
		    	}
	    		else if(SubState == SHOOTING)
	    		{
	    			if(frame ==0)
	    			{
	    				image = shoot[3];
	    				frame++;
	    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    			}
	    			else
	    			{
	    				image = flightLeft[1];
	    				changeSubStateTo(NONE);
	    			}
	    		}
	    		else if(SubState == SHOOTING_UP)
	    		{
	    			if(frame == 0)
	    			{
	    				image = shoot[4];
	    				frame++;
	    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    			}
	    			else
	    			{
	    				image = flightLeft[1];
	    				changeSubStateTo(NONE);
	    			}
	    		}
	    		else if(SubState == SHOOTING_DOWN)
	    		{
	    			if(frame ==0)
	    			{
	    				image = shoot[6];
	    				frame++;
	    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    			}
	    			else
	    			{
	    				image = flightLeft[1];
	    				changeSubStateTo(NONE);
	    			}
	    		}
	    	}
	    	else if(State == FALLING_RIGHT)
	    	{
	    		if(SubState == NONE)
	    		{
		    		if(frame == 0)
		    		{
		    			image = flightRight[1];
		    			frame++;
		    		}
		    		else
		    		{
		    			image = flightRight[2];
		    			frame = 0;
		    		}
		    		changeFrameCount = FALLING_FRAME_INTERVAL;
	    		}
	    		else if(SubState == SHOOTING)
	    		{
	    			if(frame ==0)
	    			{
	    				image = shoot[5];
	    				frame++;
	    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    			}
	    			else
	    			{
	    				image = flightRight[1];
	    				changeSubStateTo(NONE);
	    			}
	    		}
	    		else if(SubState == SHOOTING_UP)
	    		{
	    			if(frame ==0)
	    			{
	    				image = shoot[4];
	    				frame++;
	    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    			}
	    			else
	    			{
	    				image = flightRight[1];
	    				changeSubStateTo(NONE);
	    			}
	    		}
	    		else if(SubState == SHOOTING_DOWN)
	    		{
	    			if(frame ==0)
	    			{
	    				image = shoot[6];
	    				frame++;
	    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    			}
	    			else
	    			{
	    				image = flightRight[1];
	    				changeSubStateTo(NONE);
	    			}
	    		}
	    	}
	    	else if(State == GRIP_LEFT_PULL_UP)
	    	{
	    		changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    		String [] backup = hitLayers;
	    		hitLayers = new String[0];
	    		if(frame == 0)
	    		{
	    			image = gripLeft[2];
	    			attemptMove(x - 6, y - 9);
	    			frame++;
	    		}
	    		else if(frame == 1)
	    		{
	    			image = gripLeft[3];
	    			attemptMove(x-8, y-6);
	    			frame++;
	    		}
	    		else if(frame == 2)
	    		{
	    			image = gripLeft[4];
	    			attemptMove(x-7, y-10);
	    			frame++;
	    		}
	    		else if(frame == 3)
	    		{
	    			image = stand[1];
	    			attemptMove(x-5, y-7);
	    			changeStateTo(STANDING_LEFT);
	    		}
	    		hitLayers = backup;
			}	    		
	    	else if(State == GRIP_RIGHT_PULL_UP)
	    	{
	    		changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    		String [] backup = hitLayers;
	    		hitLayers = new String[0];
	    		if(frame == 0)
	    		{
	    			image = gripRight[2];
	    			attemptMove(x + 6, y-9);
	    			frame++;
	    		}
	    		else if(frame == 1)
	    		{
	    			image = gripRight[3];
	    			attemptMove(x+8,y-6);
	    			frame++;
	    		}
	    		else if(frame == 2)
	    		{
	    			image = gripRight[4];
	    			attemptMove(x+5,y-10);
	    			frame++;
	    		}
	    		else if(frame == 3)
	    		{
	    			image = stand[0];
	    			attemptMove(x+6,y-7);
	    			changeStateTo(STANDING_RIGHT);
	    		}
	    		hitLayers = backup;
			}	    		

	    	else if(State == JUMPING_LEFT)
	    	{
	    		if(SubState == NONE)
	    		{
	    			image = flightLeft[0];
	    		}
	    		else if(SubState == SHOOTING)
	    		{
	    			if(frame ==0)
	    			{
	    				image = shoot[3];
	    				frame++;
	    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    			}
	    			else
	    			{
	    				image = flightLeft[0];
	    				changeSubStateTo(NONE);
	    			}
	    		}
	    		else if(SubState == SHOOTING_UP)
	    		{
	    			if(frame ==0)
	    			{
	    				image = shoot[4];
	    				frame++;
	    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    			}
	    			else
	    			{
	    				image = flightLeft[0];
	    				changeSubStateTo(NONE);
	    			}
	    		}
	    		else if(SubState == SHOOTING_DOWN)
	    		{
	    			if(frame ==0)
	    			{
	    				image = shoot[6];
	    				frame++;
	    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    			}
	    			else
	    			{
	    				image = flightLeft[0];
	    				changeSubStateTo(NONE);
	    			}
	    		}
	    	}
	    	else if(State == JUMPING_RIGHT)
	    	{
	    		if(SubState == NONE)
	    		{
	    			image = flightRight[0];
	    		}
	    		else if(SubState == SHOOTING)
	    		{
	    			if(frame ==0)
	    			{
	    				image = shoot[5];
	    				frame++;
	    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    			}
	    			else
	    			{
	    				image = flightRight[0];
	    				changeSubStateTo(NONE);
	    			}
	    		}
	    		else if(SubState == SHOOTING_UP)
	    		{
	    			if(frame ==0)
	    			{
	    				image = shoot[4];
	    				frame++;
	    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    			}
	    			else
	    			{
	    				image = flightRight[0];
	    				changeSubStateTo(NONE);
	    			}
	    		}
	    		else if(SubState == SHOOTING_DOWN)
	    		{
	    			if(frame ==0)
	    			{
	    				image = shoot[6];
	    				frame++;
	    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    			}
	    			else
	    			{
	    				image = flightRight[0];
	    				changeSubStateTo(NONE);
	    			}
	    		}
	    	}
	    	else if((State == POGO_UP_RIGHT) || (State == POGO_DOWN_RIGHT))
	    	{
	    		image = pogoRight[1];
	    	}
	    	else if((State == POGO_UP_LEFT) || (State == POGO_DOWN_LEFT))
	    	{
	    		image = pogoLeft[1];
	    	}
	    	else if(State == STANDING_LEFT)
	    	{
	    		if(SubState == NONE)
	    		{
	    			if(!isPlatformSurf)
	    			{
	    				image = stand[1];
	    			}
	    			else
	    			{
	    				image = platformSurf;
	    			}
	    		}
	    		else if(SubState == DOWN)
	    		{
	    			if(frame == 0)
	    			{
	    				image = lookDirection[0];
	    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    				frame++;
	    			}
	    			else
	    			{
	    				image = lookDirection[1];
	    			}
	    		}
	    		else if(SubState == UP)
	    		{
	    			image = lookDirection[2];
	    		}
	    		else if(SubState == SHOOTING)
	    		{
	    			if(frame ==0)
	    			{
	    				image = shoot[0];
	    				frame++;
	    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    			}
	    			else
	    			{
	    				image = stand[1];
	    				changeSubStateTo(NONE);
	    			}
	    		}
	    		else if(SubState == SHOOTING_UP)
	    		{
	    			if(frame ==0)
	    			{
	    				image = shoot[2];
	    				frame++;
	    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    			}
	    			else
	    			{
	    				image = stand[1];
	    				changeSubStateTo(NONE);
	    			}
	    		}
	    		else if(SubState == PRESSING_SWITCH)
	    		{
	    			if(frame == 0)
	    			{
	    				image = pressSwitch;
	    				frame++;
	    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    			}
	    			else
	    			{
	    				image = stand[1];
	    				changeSubStateTo(NONE);
	    			}
	    		}
	    	}
	    	else if(State == STANDING_RIGHT)
	    	{
	    		if(SubState == NONE)
	    		{
	    			if(!isPlatformSurf)
	    			{
	    				image = stand[0];
	    			}
	    			else
	    			{
	    				image = platformSurf;
	    			}
	    		}
	    		else if(SubState == DOWN)
	    		{
	    			if(frame == 0)
	    			{
	    				image = lookDirection[0];
	    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    				frame++;
	    			}
	    			else
	    			{
	    				image = lookDirection[1];
	    			}
	    		}
	    		else if(SubState == UP)
	    		{
	    			image = lookDirection[2];
	    		}
	    		else if(SubState == SHOOTING)
	    		{
	    			if(frame ==0)
	    			{
	    				image = shoot[1];
	    				frame++;
	    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    			}
	    			else
	    			{
	    				image = stand[0];
	    				changeSubStateTo(NONE);
	    			}
	    		}
	    		else if(SubState == SHOOTING_UP)
	    		{
	    			if(frame == 0)
	    			{
	    				image = shoot[2];
	    				frame++;
	    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    			}
	    			else
	    			{
	    				image = stand[0];
	    				changeSubStateTo(NONE);
	    			}
	    		}
	    		else if(SubState == PRESSING_SWITCH)
	    		{
	    			if(frame == 0)
	    			{
	    				image = pressSwitch;
	    				frame++;
	    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    			}
	    			else
	    			{
	    				image = stand[0];
	    				changeSubStateTo(NONE);
	    			}
	    		}
	    	}
	    	else if(State == GRIP_RIGHT)
	    	{
	    		image = gripRight[0];
	    	}
	    	else if(State == GRIP_LEFT)
	    	{
	    		image = gripLeft[0];
	    	}
	    	else if(State == POLE_LEFT)
	    	{
	    		if(SubState == NONE)
	    		{
	    			image = poleLeft[0];
	    		}
	    		else if(SubState == UP)
	    		{
	    			if(frame == 0)
	    			{
	    				image = poleLeft[1];
	    				frame++;
	    			}
	    			else
	    			{
	    				image = poleLeft[2];
	    				frame =0;
	    			}
	    		}
	    		else if(SubState == DOWN)
	    		{
	    			image = poleDown[frame];
    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
  			
	    			if(frame == poleDown.length-1)
	    			{
	    				frame = 0;
	    			}
	    			else
	    			{
	    				frame++;
	    			}
	    		}
	    		else if(SubState == SHOOTING)
	    		{
	    			if(frame ==0)
	    			{
	    				image = shootPole[1];
	    				frame++;
	    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    			}
	    			else
	    			{
	    				image = poleLeft[0];
	    				changeSubStateTo(NONE);
	    			}
	    		}
	    		else if(SubState == SHOOTING_UP)
	    		{
	    			if(frame == 0)
	    			{
	    				image = shootPole[0];
	    				frame++;
	    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    			}
	    			else
	    			{
	    				image = poleLeft[1];
	    				changeSubStateTo(UP);
	    			}
	    		}
	    		else if(SubState == SHOOTING_DOWN)
	    		{
	    			if(frame == 0)
	    			{
	    				image = shootPole[2];
	    				frame++;
	    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    			}
	    			else
	    			{
	    				image = poleDown[1];
	    				changeSubStateTo(DOWN);
	    			}
	    		}
	    	}
	    	else if(State == POLE_RIGHT)
	    	{
	    		if(SubState == NONE)
	    		{
	    			image = poleRight[0];
	    		}
	    		else if(SubState == UP)
	    		{
	    			if(frame == 0)
	    			{
	    				image = poleRight[1];
	    				frame++;
	    			}
	    			else
	    			{
	    				image = poleRight[2];
	    				frame =0;
	    			}
	    		}
	    		else if(SubState == DOWN)
	    		{
	    			image = poleDown[frame];
    				changeFrameCount = DOUBLE_FRAME_INTERVAL;

	    			if(frame == poleDown.length-1)
	    			{
	    				frame = 0;
	    			}
	    			else
	    			{
	    				frame++;
	    			}
	    		}
	    		else if(SubState == SHOOTING)
	    		{
	    			if(frame ==0)
	    			{
	    				image = shootPole[4];
	    				frame++;
	    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    			}
	    			else
	    			{
	    				image = poleRight[0];
	    				changeSubStateTo(NONE);
	    			}
	    		}
	    		else if(SubState == SHOOTING_UP)
	    		{
	    			if(frame == 0)
	    			{
	    				image = shootPole[3];
	    				frame++;
	    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    			}
	    			else
	    			{
	    				image = poleDown[0];
	    				changeSubStateTo(UP);
	    			}
	    		}
	    		else if(SubState == SHOOTING_DOWN)
	    		{
	    			if(frame == 0)
	    			{
	    				image = shootPole[5];
	    				frame++;
	    				changeFrameCount = DOUBLE_FRAME_INTERVAL;
	    			}
	    			else
	    			{
	    				image = poleRight[1];
	    				changeSubStateTo(DOWN);
	    			}
	    		}


	    	}
	    	else	    	
	    	{
				image = stand[0];
	    	}
    	}
    	else
    	{
    		changeFrameCount--;
    	}	
    }
    
    public void draw(Graphics drawTo, int xOffset, int yOffset)
    {
    	if(DRAW_KEYS)
    	{
	    	drawTo.setColor(Color.BLACK);
	        drawTo.setClip(20,300,200,1010);
	        
	        drawTo.drawString("key_up: " + key_up,20,310);
	        drawTo.drawString("key_down: " + key_down,20,320);
	
	        drawTo.drawString("key_left: " + key_left,20,340);
	        drawTo.drawString("key_right: " + key_right,20,350);
	
	        drawTo.drawString("key_jump: " + key_jump,20,370);
	        drawTo.drawString("key_fire: " + key_fire,20,380);
	        drawTo.drawString("key_pogo: " + key_pogo,20,390);
	        drawTo.drawString("Can_fall: " + canFall,20,400);
		}

    	if((SubState == SHOOTING) || (SubState == SHOOTING_UP) || (SubState == SHOOTING_DOWN))
    	{
        	screenX = x - xOffset - 9;
        	screenY = y - yOffset - 8;
        }
        else
        {
        	screenX = x - xOffset;
        	screenY = y - yOffset;
		}        
        
       	//drawTo.setClip(screenX,screenY,image.getWidth(null),image.getHeight(null));
       	drawTo.drawImage(image,screenX,screenY, null); //Is this the best way to put it to the screen???
       	
    }
    
    public boolean hasHit(ShapeTile hitSprite)
    {
    	Point gripPoint;
    	if(canGrip)
    	{
    		if(key_right)
    		{
    			gripPoint = hitSprite.getLeftGrip(x,y);
    			if(gripPoint != null)
    			{
    				String [] backup = hitLayers;
	    			hitLayers = new String[0];

					attemptMove(gripPoint.x - width, gripPoint.y);
					hitLayers = backup;

    				//x = gripPoint.x - width;
    				//y = gripPoint.y;
    				changeStateTo(GRIP_RIGHT);
    				nextUpdateTime = System.currentTimeMillis() + movement.GRIP_PAUSE_TIME;
    			}
    		}
    		else if(key_left)
    		{
    			gripPoint = hitSprite.getRightGrip(x,y);
    			if(gripPoint != null)
    			{
    				String [] backup = hitLayers;
	    			hitLayers = new String[0];

					attemptMove(gripPoint.x , gripPoint.y);
					hitLayers = backup;

    				//x = gripPoint.x;
    				//y = gripPoint.y;
    				changeStateTo(GRIP_LEFT);
    				nextUpdateTime = System.currentTimeMillis() + movement.GRIP_PAUSE_TIME;
    			}
    		}
    	}
    	return true;
    }
    
    public boolean hasHit(Pole hitSprite)
    {
    	canGripPole = true;
    	gripPole = hitSprite;
    	return false;
    }

    public boolean hasHit(Platform hitSprite)
    {
    	
    	if(((hitSprite.getY() + (hitSprite.getHeight() / 2))== (y + height)) 
    		&& (standPlatform == null) &&
    		((State == FALLING_LEFT) || (State == FALLING_RIGHT) 
    		|| (State == POGO_DOWN_RIGHT) || (State == POGO_DOWN_LEFT)
    		|| (State == WALKING_RIGHT) || (State == WALKING_LEFT)
    		|| (State == STANDING_RIGHT) || (State == STANDING_LEFT)))
    	{
	    	isPlatformSurf = hitSprite.connect(this);
	    	standPlatform = hitSprite;
    		canFall = true;
	    	return true;
    	}
    	else
    	{
    		return false;
    	}
    }

    public boolean hasHit(ThinFloor hitSprite)
    {
    	if((hitSprite.getFloorY() == (y + height)) 
    		&& ((State == FALLING_LEFT) || (State == FALLING_RIGHT) 
    		|| (State == POGO_DOWN_RIGHT) || (State == POGO_DOWN_LEFT)
    		|| (State == WALKING_RIGHT) || (State == WALKING_LEFT)))
    	{
    		canFall = true;
	    	return true;
    	}
    	else
    	{
    		return false;
    	}
    }


    public boolean hasHit(PoleTop hitSprite)
    {
    	if(hitSprite.getY() > (getY() + (getHeight() / 3))) 
    	{
	    	hitTop = true; //If the top is to far down our body we dont want to be able
	    					//to grip onto the pole
    	}

    	if(((State == POLE_LEFT) || (State == POLE_RIGHT)) && (SubState == UP))
    	{
    		return true;
    	}
    	return false;
    }

    public boolean hasHit(PoleBottom hitSprite)
    {
    	hitBottom = true;
    	return false;
    }

    public boolean hasHit(Switch hitSprite)
    {
    	canPushSwitch = true;
    	pushSwitch = hitSprite;
    	return false;
    }
    
    public boolean hasHit(Exit hitSprite)
    {
    	myWorld.stop();
    	return false;
    }

    //Keyboard stuff
    public void keyPressed(KeyEvent e)
    {
        if(e.getKeyCode() == KEY_UP)
        {
            key_up = true;
            canKeyPushSwitch = true;
        }
		else if(e.getKeyCode() == KEY_DOWN)
		{
			key_down = true;
		}
		else if(e.getKeyCode() == KEY_LEFT)
		{
			key_left = true;
		}
		else if(e.getKeyCode() == KEY_RIGHT)
		{
			key_right = true;
		}
		else if(e.getKeyCode() == KEY_JUMP)
		{
			key_jump = true;
		}
        else if(e.getKeyCode() == KEY_POGO)
        {
        	key_pogo = true;
        }
        else if(e.getKeyCode() == KEY_FIRE)
        {
        	key_fire = true;
        }
        //else if(e.getKeyCode() == KEY_ESC)
        //{
        //	key_esc = true;
        //}
    }
    
    public void keyReleased(KeyEvent e)
    {
    	if(e.getKeyCode() == KEY_UP)
        {
            key_up = false;
        }
		else if(e.getKeyCode() == KEY_DOWN)
		{
			key_down = false;
		}
		else if(e.getKeyCode() == KEY_LEFT)
		{
			key_left = false;
		}
		else if(e.getKeyCode() == KEY_RIGHT)
		{
			key_right = false;
		}
		else if(e.getKeyCode() == KEY_JUMP)
		{
			key_jump = false;
			canJump = true;
		}
        else if(e.getKeyCode() == KEY_POGO)
        {
        	key_pogo = false;
        	canReleasePogo = true;
        	canPogo = true;
        }
        else if(e.getKeyCode() == KEY_FIRE)
        {
        	key_fire = false;
        	canFire = true;
        }
        //else if(e.getKeyCode() == KEY_ESC)
        //{
        	//key_esc = false;
        //}
        else if(e.getKeyCode() == KeyEvent.VK_R)
        {
        	x = 50;
        	y = 50;
        }
    }

    public void keyTyped(KeyEvent e)
    {
    }
}