package keenGame.sprite.platform;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Rectangle;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import keenGame.Level;
import keenGame.XmlElement;
import keenGame.sprite.Platform;
import keenGame.sprite.hint.DirectionHint;
import keenGame.sprite.keenActor.KeenActor;

/**
 * A BasicPlatform can be stood on by a KeenActor.  When a KeenActor
 * is standing on a platform, he will move with the platform.  BasicPlatforms
 * will follow any DirectionHits they collide with
 */
public class BasicPlatform extends Platform
{
	protected int Direction;
	protected int Speed = 1;
	protected KeenActor standKeen, tempKeen;
	private String graphic;

	/**
     * Constructs an BasicPlatform.
     * 
     * @param imageLocation - The location of an image (normally .png format) that
     * 		is used for this BasicPlatform
     * @param layer - The layer that this BasicPlatform should exist in
     * @param hitLayers - An array of strings containing the layers which contain
     * 	Sprites with which this BasicPlatform can collide
     * @param xValue - The x location of the BasicPlatform
     * @param yValue - The y location of the BasicPlatform
	 * @param myWorld - the Level in which the BasicPlatform exists
     * 
     * @see keenGame.sprite.Sprite#hitArea
     * @see keenGame.sprite.Actor#Actor(String, String, int, int)
     */
    public BasicPlatform(String imageLocation, String layer, 
    					String [] hitLayers, Level myWorld,
    					int xValue, int yValue, int Direction) 	
    {
    	super(imageLocation, layer, hitLayers, myWorld, xValue,yValue);
    	init(Direction);
    }
    
   	public BasicPlatform(XmlElement spriteElement, Level myWorld)
   	{
   		super(spriteElement, myWorld);
   		

   		//get direction
   		int newDirection = stringToDirection(spriteElement.getAttribute("direction"));

		init(newDirection);   		
   	}
    
    private void init(int Direction)
    {
    	hitArea = new Rectangle(x, y + (height / 2), width, (height / 2));
    	
    	State = FLYING;
    	this.Direction = Direction;
    }
    
	public Element toXml(Document xmlDoc)
	{
		return addToElement(xmlDoc, xmlDoc.createElement("BasicPlatform"));
	}
	
	protected Element addToElement(Document xmlDoc, Element element)
    {
    	super.addToElement(xmlDoc, element);
    	
		element.setAttribute("direction",directionToString(Direction));	
		
    	return element;
    }

	/**
	 * @see keenGame.sprite.Actor#update()
	 */
	public void update()
	{
		if(standKeen != null) //We move keenGame first because it is possible for direction to change when we move
		{
			tempKeen = standKeen; //Just incase we have any unpleasentness				
			standKeen.movePlatform(Speed, Direction);
		}
		directionMove(Speed, Direction);
	}

	/**
	 * TODO:  Replace this with movement listeners when KeenActor is
	 * refactored
	 */
    public void disconnect()
    {
    	standKeen = null;
    }

	/**
	 * TODO:  Replace this with movement listeners when KeenActor is
	 * refactored
	 */
    public boolean connect(KeenActor standKeen)
    {
    	this.standKeen = standKeen;
    	return false;
    }
    
	/**
	 * Starts the platform moving, in the direction
	 * specified by the DirectionHint
	 */
    public boolean hasHit(DirectionHint hitSprite)
    {
		Direction = hitSprite.getDirection();    	
    	return false;
    }
}
