package keenGame.sprite.platform;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import keenGame.Level;
import keenGame.sprite.SwitchListener;
import keenGame.sprite.keenActor.KeenActor;
import keenGame.sprite.tile.Switch;


/**
 * A SwitchPlatform extends BasicPlatform, with the ability to be actived
 * and deactivated by a switch.  When constructed, SwitchPlatforms are 
 * deactivated, and turned off
 */
public class SwitchPlatform extends BasicPlatform implements SwitchListener
{
	private boolean moving = false;	
	
	/**
     * Constructs an SwitchPlatform.  The SwitchPlatform will not move
     * until the switch it is connected to is activated.
     * 
     * @param imageLocation - The location of an image (normally .png format) that
     * 		is used for this SwitchPlatform
     * @param layer - The layer that this SwitchPlatform should exist in
     * @param hitLayers - An array of strings containing the layers which contain
     * 	Sprites with which this SwitchPlatform can collide
     * @param xValue - The x location of the SwitchPlatform
     * @param yValue - The y location of the SwitchPlatform
	 * @param myWorld - the Level in which the SwitchPlatform exists
     * 
     * @see keenGame.sprite.Sprite#hitArea
     * @see keenGame.sprite.Actor#Actor(String, String, int, int)
     */
    public SwitchPlatform(String imageLocation, String layer, 
    					String [] hitLayers, Level myWorld,
    					int xValue, int yValue, int Direction) 
    {	
    	super(imageLocation, layer, hitLayers, myWorld,
    			xValue, yValue, Direction);
    }

	/**
	 * @see keenGame.sprite.SwitchListener#SwitchToggled(Switch)
	 */
	public void SwitchToggled(Switch s)
	{
		moving = s.isOn();
	}

	/**
	 * @see keenGame.sprite.Actor#update()
	 */	
	public void update()
	{
		if(moving)
		{
			super.update();
		}
	}
	
	/**
	 * TODO:  Replace this with movement listeners when KeenActor is
	 * refactored
	 */	
	public boolean connect(KeenActor standKeen)
    {
    	this.standKeen = standKeen;
    	return true;
    }
}
