package keenGame.sprite.tile;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.awt.Rectangle;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import keenGame.Cache;
import keenGame.XmlElement;
import keenGame.sprite.Sprite;

/**
 * A  pole, can be climbed up or down by a KeenActor.  Evey pole should
 * be capped by a PoleBottom, and a PoleTop.
 * TODO: Refactor,  so we dont need PoleButton, and  PoleTop
 */
public class Pole extends Sprite
{
	private String graphic;
	/**
     * Constructs a Pole.  The hitArea will automatically be set
     * to the shape of the image.
     * 
     * @param Pole - The location of an image (normally .png format) that
     * 		is used for this sprite
     * @param layer - The layer that this Pole should exist in
     * @param xValue - The x location of the Pole
     * @param yValue - The y location of the Pole
     */
    public Pole(String imageLocation, String layer, int xValue, int yValue)
    {
    	super(layer, xValue,yValue);
    	init(imageLocation);
    }

    public Pole(XmlElement spriteElement)
    {
    	super(spriteElement);
    	
    	String newGraphic = spriteElement.getChild("graphic").getText();
    	init(newGraphic);
    }
    
    private void init(String imageLocation)
    {
    	image = Cache.getImage(imageLocation);
    	width = image.getWidth(null);
    	height = image.getHeight(null);

    	graphic = imageLocation;
    	hitArea = new Rectangle(x, y, width, height);
    }
    
	public Element toXml(Document xmlDoc)
	{
		return addToElement(xmlDoc, xmlDoc.createElement("Pole"));
	}
    
    protected Element addToElement(Document xmlDoc, Element element)
    {
    	super.addToElement(xmlDoc, element);
    	
    	//graphic
    	Element graphicElement = xmlDoc.createElement("graphic");
		graphicElement.appendChild(xmlDoc.createTextNode(graphic));
		element.appendChild(graphicElement);
		
    	return element;
    }
}
