package keenGame.sprite.tile;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.awt.Point;
import java.awt.Rectangle;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import keenGame.Cache;
import keenGame.XmlElement;

/**
 * A RectangleTile, is a ShapeTile, with a Rectangular hitArea.  It should
 * be used  for odd shaped bits of scenery, walls, floors, etc, that keen
 * can grip hold of
 */
public class RectangleTile extends ShapeTile
{
	private boolean gripLeft, gripRight;
	private String tileFile;
	
    /**
     * Constructs a RectangleTile.  
     * 
     * @param fileLocation - This should be the location of a RectangleTile file
     * which defines the shape of thre rectangle, as well as the imaged used.
     * @param layer - The layer that this RectangleTile should exist in
     * @param xValue - The x location of the RectangleTile
     * @param yValue - The y location of the RectangleTile
     */
    public RectangleTile(String fileLocation, String layer, int xValue, int yValue)
	{
    	super(layer, xValue,yValue);
    	init(fileLocation);
	}

    public RectangleTile(XmlElement spriteElement)
    {
    	super(spriteElement);
    	
    	String newFile = spriteElement.getChild("file").getText();
    	init(newFile);
    }
	
	private void init(String fileLocation)
	{
		RectangleTileFile file = Cache.getRectangleTile(fileLocation);
		tileFile = fileLocation;
		
		image = file.tileImage;
        width = image.getWidth(null);
        height = image.getHeight(null);		
    	
    	//Our floor just has a boring old square hitArea
    	hitArea = new Rectangle(file.x+ x, file.y+y, file.width, file.height);
    	
    	gripLeft = file.gripLeft;
    	gripRight = file.gripRight;
    }
    
    public Element toXml(Document xmlDoc)
	{
		return addToElement(xmlDoc, xmlDoc.createElement("RectangleTile"));
	}
    
    protected Element addToElement(Document xmlDoc, Element element)
    {
    	super.addToElement(xmlDoc, element);
    	
    	//File
    	Element graphicElement = xmlDoc.createElement("file");
		graphicElement.appendChild(xmlDoc.createTextNode(tileFile));
		element.appendChild(graphicElement);
		
    	return element;
    }
    
    public Point getRightGrip(int keenActorX, int keenActorY)
    {
	    Rectangle hitArea = ((Rectangle)this.hitArea);
		if((keenActorX >= (hitArea.x + hitArea.width)) && gripRight &&
    	  ((keenActorY > hitArea.y) && (keenActorY < hitArea.y + KEEN_GRAB_DISTANCE)))
		{
			return new Point(hitArea.x + hitArea.width, hitArea.y);
    	}
		else
		{
    		return null;
		}
    }

	public Point getLeftGrip(int keenActorX, int keenActorY)
	{
	    Rectangle hitArea = ((Rectangle)this.hitArea);
    	if((keenActorX <= hitArea.x) && gripLeft &&
	   	  ((keenActorY > hitArea.y) && (keenActorY < hitArea.y + KEEN_GRAB_DISTANCE)))
    	{
    		return new Point(hitArea.x , hitArea.y);
    	}
    	else
    	{
    		return null;
    	}
	}
}
