package keenGame.sprite.tile;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.awt.Point;

import keenGame.XmlElement;
import keenGame.sprite.Sprite;

/**
 * Shape tiles are Sprites, that can be griped onto by keen.
 * The should be used for walls, and floors that keen can stand on
 */
public abstract class ShapeTile extends Sprite implements GripAble
{
    /**
     * Constructs a ShapeTile.  When writing your own subclasses of ShapeTile you
     * should always put in a call to this constructor with Super.  After 
     * this constructor has been called, you should define a hitArea, as well
     * as any other instance variables your ShapeTile should need.
     * 
     * @param imageLocation - The location of an image (normally .png format) that
     * 		is used for this ShapeTile
     * @param layer - The layer that this ShapeTile should exist in
     * @param xValue - The x location of the ShapeTile
     * @param yValue - The y location of the ShapeTile
     */
    public ShapeTile(String layer, int xValue, int yValue)
	{
    	super(layer, xValue,yValue);
	}
	
	public ShapeTile(XmlElement spriteElement)
	{
		super(spriteElement);
	}
	
	/**
	 * Returns the position of the point keen can grip onto, on the 
	 * right  hand side of the ShapeTile
	 */
	public abstract Point getRightGrip(int keenActorX, int keenActorY);

	/**
	 * Returns the position of the point keen can grip onto, on the 
	 * right  hand side of the ShapeTile
	 */
	public abstract Point getLeftGrip(int keenActorX, int keenActorY);	
}
