package keenGame.test;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.util.Iterator;

import junit.framework.TestCase;
import keenGame.ArrayMapLayer;
import keenGame.sprite.Sprite;

/**
 * Test case, for ArrayMapLayerTest
 * @see keenGame.ArrayMapLayer
 */
public class ArrayMapLayerTest extends TestCase
{
	private ArrayMapLayer testMap;
	private Sprite [] SpriteArray;
	
	/**
	 * Constructor for ArrayMapLayerTest.
	 * @param arg0
	 */
	public ArrayMapLayerTest(String arg0)
	{
		super(arg0);
	}
	
	/**
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception
	{
		super.setUp();
		testMap = new ArrayMapLayer(500, 500, 10, 3);
		
		SpriteArray = new TestSprite[10];
		for (int i = 0; i < SpriteArray.length; i++)
		{
			SpriteArray[i] = new TestSprite(i*10 ,i*10 ,20,20);
		}

	}

	public void testSize()
	{
		//Add 10 things into it
		for (int i = 0; i < SpriteArray.length; i++)
		{
			testMap.add(SpriteArray[i]);
		}
		assertTrue(testMap.size() == 10);

		//Remove 5
		for (int i = 0; i < 5; i++)
		{
			testMap.remove(SpriteArray[i]);
		}
		assertTrue(testMap.size() == 5);

		//Add the 5 back in
		for (int i = 0; i < 5; i++)
		{
			testMap.add(SpriteArray[i]);
		}
		assertTrue(testMap.size() == 10);
	}

	public void testClear()
	{
		//Add 10 things into the map
		for (int i = 0; i < SpriteArray.length; i++)
		{
			testMap.add(SpriteArray[i]);
		}

		//Clear it
		testMap.clear();
		
		//check size
		assertTrue(testMap.size() == 0);
		
		//Iterate over the map, and check that nothing comes out
		assertTrue(testMap.iterator().hasNext() == false);
	}

	/*
	 * Test for Iterator iterator()
	 */
	public void testIterator()
	{
		for (int i = 0; i < SpriteArray.length; i++)
		{
			testMap.add(SpriteArray[i]);
		}
		//And one to check it goes to the very end
		TestSprite endTest = new TestSprite(492,492,3,3);
		testMap.add(endTest);

		//check they come out of the iterator	
		Iterator it = testMap.iterator();
		
		for (int i = 0; i < SpriteArray.length; i++)
		{
			assertTrue(it.hasNext());
			assertTrue(it.next() == SpriteArray[i]);
		}
		assertTrue(it.hasNext());
		assertTrue(it.next() == endTest);
		assertTrue(!it.hasNext());
	}

	/*
	 * Test for Iterator iterator(int, int, int, int)
	 */
	public void testIteratorIIII()
	{
		for (int i = 0; i < SpriteArray.length; i++)
		{
			testMap.add(SpriteArray[i]);
		}

		//Check that not all come out
		Iterator it = testMap.iterator(0,30,49,49);
		
		for (int i = 2; i < 3; i++)
		{
			assertTrue(it.hasNext());
			assertTrue(it.next() == SpriteArray[i]);
		}
		assertTrue(it.hasNext());

		//Check for not finding it
		testMap = new ArrayMapLayer(500, 500, 10, 3);
		Sprite tester = new TestSprite(10,10,50,50);
		testMap.add(tester);
		it = testMap.iterator(60,60,50,50);
		assertTrue(!it.hasNext());
		
		//Check for finding it
		testMap = new ArrayMapLayer(500, 500, 10, 3);
		tester = new TestSprite(10,10,50,50);
		testMap.add(tester);
		it = testMap.iterator(0,0,50,50);
		assertTrue(it.hasNext());
		assertTrue(it.next() == tester);
		assertTrue(!it.hasNext());
		
		//Check for at the edge
		TestSprite edgeTest = new TestSprite(492,492,3,3);
		testMap.add(edgeTest);
		it = testMap.iterator(490,490,10,10);
		assertTrue(it.hasNext());
		assertTrue(it.next() == edgeTest);
		assertTrue(!it.hasNext());

		//Test for the upper left corner
		testMap.clear();
		edgeTest = new TestSprite(2,2,3,3);
		testMap.add(edgeTest);
		it = testMap.iterator(0,0,9,9);
		assertTrue(it.hasNext());
		assertTrue(it.next() == edgeTest);
		assertTrue(!it.hasNext());
		
		it = testMap.iterator(0,0,10,10);
		assertTrue(it.hasNext());
		assertTrue(it.next() == edgeTest);
		assertTrue(!it.hasNext());

		it = testMap.iterator(10,0,10,30);
		assertTrue(!it.hasNext());

		it = testMap.iterator(0,10,30,10);
		assertTrue(!it.hasNext());
	}

	/*
	 * Test for boolean add(Object)
	 */
	public void testAddObject()
	{
		//Create a sprite
		Sprite tester = new TestSprite(10,10,50,50);
		//Add it in
		testMap.add(tester);
		
		//Check it is in
		assertTrue(testMap.size() == 1);
		assertTrue(testMap.contains(tester));
	}
	
	public void testArrayMap()
	{
		assertTrue(testMap.getWidth() >= 500);
		assertTrue(testMap.getHeight() >= 500);
		
		//Do somethign odd
		testMap = new ArrayMapLayer(500, 500, 9, 10);
		assertTrue(testMap.getWidth() >= 500);
		assertTrue(testMap.getHeight() >= 500);

		testMap = new ArrayMapLayer(500, 500, 8, 10);
		assertTrue(testMap.getWidth() >= 500);
		assertTrue(testMap.getHeight() >= 500);

		testMap = new ArrayMapLayer(500, 500, 7, 10);
		assertTrue(testMap.getWidth() >= 500);
		assertTrue(testMap.getHeight() >= 500);

		testMap = new ArrayMapLayer(500, 500, 6, 10);
		assertTrue(testMap.getWidth() >= 500);
		assertTrue(testMap.getHeight() >= 500);
	}
		

	public void testFitsInside()
	{
		//test above
		Sprite tester = new TestSprite(1,-1,50,50);
		try
		{
			testMap.add(tester);
			fail("Allowed adding of invalid sprite");
		}
		catch(UnsupportedOperationException e)
		{}

		//Test below
		tester = new TestSprite(1,501,50,50);
		try
		{
			testMap.add(tester);
			fail("Allowed adding of invalid sprite");
		}
		catch(UnsupportedOperationException e)
		{}

		tester = new TestSprite(1,451,50,50);
		try
		{
			testMap.add(tester);
			fail("Allowed adding of invalid sprite");
		}
		catch(UnsupportedOperationException e)
		{}

		//Test left
		tester = new TestSprite(-1,1,50,50);
		try
		{
			testMap.add(tester);
			fail("Allowed adding of invalid sprite");
		}
		catch(UnsupportedOperationException e)
		{}

		//Test right
		tester = new TestSprite(501,1,50,50);
		try
		{
			testMap.add(tester);
			fail("Allowed adding of invalid sprite");
		}
		catch(UnsupportedOperationException e)
		{}

		tester = new TestSprite(451,1,50,50);
		try
		{
			testMap.add(tester);
			fail("Allowed adding of invalid sprite");
		}
		catch(UnsupportedOperationException e)
		{}

		//Test it actualy fits!
		tester = new TestSprite(50,50,50,50);
		try
		{
			testMap.add(tester);
		}
		catch(UnsupportedOperationException e)
		{
			fail("Disallowed adding a valid sprite");
		}
	}

	/*
	 * Test for boolean remove(Object)
	 */
	public void testRemoveObject()
	{
		Sprite tester = new TestSprite(10,10,50,50);
		//Add it in
		testMap.add(tester);

		//Remove
		assertTrue(testMap.remove(tester));
		
		//Check it is gone
		assertTrue(testMap.size() == 0);
		assertTrue(!testMap.contains(tester));

		//Test edge		
		TestSprite endTest = new TestSprite(492,492,3,3);
		testMap.add(endTest);
		assertTrue(testMap.remove(endTest));

		//Check it is gone
		assertTrue(testMap.size() == 0);
		assertTrue(!testMap.contains(tester));
		
		//Test removing when nothing is there
		assertTrue(!testMap.remove(endTest));
		testMap.add(tester);
		assertTrue(!testMap.removeFrom(100,100,50,50,endTest));
	}
	
	/*
	 * Tests a simulated move of an object
	 */
	public void testMoveObject()
	{
		TestSprite tester = new TestSprite(10,10,50,50);
		//Add it in
		testMap.add(tester);
		
		//Move it to the right by 100
		for(int distance = 0; distance < 100; distance++)
		{
			testMap.remove(tester);
			tester.right();
			testMap.add(tester);
		}

		//Check that it is there
		Iterator it = testMap.iterator(110,10,100,100);
		assertTrue(it.hasNext());
		assertTrue(it.next() == tester);
		assertTrue(!it.hasNext());

		//Move it to the down by 100
		for(int distance = 0; distance < 100; distance++)
		{
			testMap.remove(tester);
			tester.down();
			testMap.add(tester);
		}

		//Check that it is there
		it = testMap.iterator(110,110,100,100);
		assertTrue(it.hasNext());
		assertTrue(it.next() == tester);
		assertTrue(!it.hasNext());
	}
	
	public void testBig()
	{
		ArrayMapLayer testLayer = new ArrayMapLayer(500,500);
		Iterator it = testLayer.iterator(0,0,500,500);
		
		while(it.hasNext())
		{
			it.next();
		}
	}
}
