package keenGame.test;

import java.awt.Rectangle;

import junit.framework.TestCase;
import keenGame.RepaintManager;

public class RepaintManagerTest extends TestCase
{
	private RepaintManager testManager;

	public RepaintManagerTest(String arg0)
	{
		super(arg0);
	}

	protected void setUp() throws Exception
	{
		super.setUp();
		
		testManager = new RepaintManager();
	}

	public void testBasic()
	{
		//Add two areas
		testManager.addRegion(1,1,10,10);
		testManager.addRegion(50,50,15,15);

		//check they come out
		Rectangle[] testArray = testManager.getDirtyRegions(0,0,100,100);
		assertTrue(testArray.length == 2);

		Rectangle testRect = testArray[0];
		assertTrue(testRect.x == 1);
		assertTrue(testRect.y == 1);
		assertTrue(testRect.width == 10);
		assertTrue(testRect.height == 10);

		testRect = testArray[1];
		assertTrue(testRect.x == 50);
		assertTrue(testRect.y == 50);
		assertTrue(testRect.width == 15);
		assertTrue(testRect.height == 15);

		//clear
		testManager.clearRegions();
		
		//Add two areas
		testManager.addRegion(1,50,10,10);
		testManager.addRegion(50,1,15,15);

		//check they come out
		testArray = testManager.getDirtyRegions(0,0,100,100);
		assertTrue(testArray.length == 2);

		testRect = testArray[0];
		assertTrue(testRect.x == 1);
		assertTrue(testRect.y == 50);
		assertTrue(testRect.width == 10);
		assertTrue(testRect.height == 10);

		testRect = testArray[1];
		assertTrue(testRect.x == 50);
		assertTrue(testRect.y == 1);
		assertTrue(testRect.width == 15);
		assertTrue(testRect.height == 15);
	}

	public void testPartialMatch()
	{
		//Add two areas
		testManager.addRegion(50,50,10,10);
		testManager.addRegion(500,500,10,10);

		//query for one and check it comes out
		Rectangle[] testArray = testManager.getDirtyRegions(0,0,100,100);
		assertTrue(testArray.length == 1);

		Rectangle testRect = testArray[0];
		assertTrue(testRect.x == 50);
		assertTrue(testRect.y == 50);
		assertTrue(testRect.width == 10);
		assertTrue(testRect.height == 10);

		//query for none and check it comes out
		testArray = testManager.getDirtyRegions(0,0,10,10);
		assertTrue(testArray.length == 0);
		
		//Check to see that a rectangle partialy above the area comes out
		testArray = testManager.getDirtyRegions(0,55,100,100);
		assertTrue(testArray.length == 1);

		testRect = testArray[0];
		assertTrue(testRect.x == 50);
		assertTrue(testRect.y == 50);
		assertTrue(testRect.width == 10);
		assertTrue(testRect.height == 10);

		//Check to see that a rectangle partialy below the area comes out
		testArray = testManager.getDirtyRegions(0,0,100,55);
		assertTrue(testArray.length == 1);

		testRect = testArray[0];
		assertTrue(testRect.x == 50);
		assertTrue(testRect.y == 50);
		assertTrue(testRect.width == 10);
		assertTrue(testRect.height == 10);

		//Check to see that a rectangle partialy left of the area comes out
		testArray = testManager.getDirtyRegions(55,0,100,100);
		assertTrue(testArray.length == 1);

		testRect = testArray[0];
		assertTrue(testRect.x == 50);
		assertTrue(testRect.y == 50);
		assertTrue(testRect.width == 10);
		assertTrue(testRect.height == 10);

		//Check to see that a rectangle partialy right of the area comes out
		testArray = testManager.getDirtyRegions(0,0,55,100);
		assertTrue(testArray.length == 1);

		testRect = testArray[0];
		assertTrue(testRect.x == 50);
		assertTrue(testRect.y == 50);
		assertTrue(testRect.width == 10);
		assertTrue(testRect.height == 10);


	}
	
	public void testOptimiseOverlap()
	{}
	
	public void testOptimisePartialOverlap()
	{}

}
