package keenGame.test;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import junit.framework.TestCase;
import keenGame.ViewPort;
import keenGame.sprite.MovementEvent;

public class ViewPortTest extends TestCase
{
	/**
	 * Constructor for ViewPortTest.
	 * @param arg0
	 */
	public ViewPortTest(String arg0)
	{
		super(arg0);
	}

	public void testViewPort()
	{
		//test normal
		TestSprite actor = new TestSprite(500,200,50,50);
		ViewPort vp = new ViewPort(actor,1000,1000,640,400);
		
		assertTrue(vp.getX() == 205);
		assertTrue(vp.getY() == 25);
		
		//test upper left
		actor = new TestSprite(50,20,50,50);
		vp = new ViewPort(actor,1000,1000,640,400);
		
		assertTrue(vp.getX() == 0);
		assertTrue(vp.getY() == 0);
		
		//test lower right
		actor = new TestSprite(900,900,50,50);
		vp = new ViewPort(actor,1000,1000,640,400);
		
		assertTrue(vp.getX() == 360);
		assertTrue(vp.getY() == 600);
	}

	public void testMoveUp()
	{
		//test normal
		TestSprite actor = new TestSprite(500,200,50,50);
		ViewPort vp = new ViewPort(actor,1000,1000,640,400);

		vp.moveUp();
		assertTrue(vp.getY() == 21);
		vp.moveUp();
		assertTrue(vp.getY() == 17);

		//test Actor already at bottom
		actor = new TestSprite(500,200,50,50);
		vp = new ViewPort(actor,1000,1000,640,400);

		actor.setY(275);
		vp.moveUp();

		//check it failed
		assertTrue(vp.getY() == 25);
		
		//test At top of level
		actor = new TestSprite(500,20,50,50);
		vp = new ViewPort(actor,1000,1000,640,400);

		vp.moveUp();
		assertTrue(vp.getY() == 0);
	}

	public void testMoveDown()
	{
		//test normal
		TestSprite actor = new TestSprite(500,200,50,50);
		ViewPort vp = new ViewPort(actor,1000,1000,640,400);

		vp.moveDown();
		assertTrue(vp.getY() == 29);
		vp.moveDown();
		assertTrue(vp.getY() == 33);
		
		//test Actor already at top
		actor = new TestSprite(500,200,50,50);
		vp = new ViewPort(actor,1000,1000,640,400);

		actor.setY(125);
		vp.moveDown();

		//check it failed
		assertTrue(vp.getY() == 25);
		
		//test At bottom of level
		actor = new TestSprite(500,900,50,50);
		vp = new ViewPort(actor,1000,1000,640,400);

		vp.moveDown();
		assertTrue(vp.getY() == 600);
	}

	public void testMovementPerformed()
	{
		//Start at upper left corner
		TestSprite actor = new TestSprite(20,20,50,50);
		ViewPort vp = new ViewPort(actor,1000,1000,640,400);
		
		//move right until at upper right
		for(int distance =0; distance < 930; distance++)
		{
			actor.right();
			vp.movementPerformed(new MovementEvent(actor,
				actor.getX()-1,
				actor.getY(),
				actor.getX(),
				actor.getY()));
		}
		assertTrue(vp.getX() == 360);
		assertTrue(vp.getY() == 0);
		
		//move down until at lower right
		for(int distance =0; distance < 930; distance++)
		{
			actor.down();
			vp.movementPerformed(new MovementEvent(actor,
				actor.getX(),
				actor.getY()-1,
				actor.getX(),
				actor.getY()));
		}
		assertTrue(vp.getX() == 360);
		assertTrue(vp.getY() == 600);
		
		//move left until lower left
		for(int distance =0; distance < 930; distance++)
		{
			actor.left();
			vp.movementPerformed(new MovementEvent(actor,
				actor.getX()+1,
				actor.getY(),
				actor.getX(),
				actor.getY()));
		}
		assertTrue(vp.getX() == 0);
		assertTrue(vp.getY() == 600);
		
		//move up until back in upper left corner
		for(int distance =0; distance < 930; distance++)
		{
			actor.up();
			vp.movementPerformed(new MovementEvent(actor,
				actor.getX(),
				actor.getY()+1,
				actor.getX(),
				actor.getY()));
		}
		assertTrue(vp.getX() == 0);
		assertTrue(vp.getY() == 0);
	}
}
