package keenGame.test;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.io.FileInputStream;
import java.util.Iterator;
import java.util.List;

import junit.framework.TestCase;
import keenGame.XMLLevelLoader;
import keenGame.sprite.MovingActor;
import keenGame.sprite.Sprite;
import keenGame.sprite.hint.DirectionHint;
import keenGame.sprite.tile.AutoRectangleTile;

/**
 * @author luke
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class XMLLevelLoaderTest extends TestCase
{

	private XMLLevelLoader test;
	/**
	 * Constructor for XMLLevelLoaderTest.
	 * @param arg0
	 */
	public XMLLevelLoaderTest(String arg0)
	{
		super(arg0);
	}
	
	protected void setUp() throws Exception
	{
		super.setUp();
		test = new XMLLevelLoader(new FileInputStream("levels/level1.xml"), null);
	}

	public void testGetWidthAndHeight()
	{
		assertTrue(test.getWidth() == 1000);
		assertTrue(test.getHeight() == 1000);
	}

	public void testGetLayers()
	{
		String [] layers = test.getLayers();
		assertTrue(layers.length == 7);
		assertTrue(layers[0].equals("background"));
		assertTrue(layers[1].equals("walls"));
		assertTrue(layers[2].equals("monsters"));
		assertTrue(layers[3].equals("platforms"));
		assertTrue(layers[4].equals("keen"));
		assertTrue(layers[5].equals("shots"));
		assertTrue(layers[6].equals("foreground"));
	}
	
	public void testGetKeen()
	{
		Sprite testKeen = test.getKeen();
		
		assertTrue(testKeen.getLayer().equals("keen"));
		assertTrue(testKeen.getX() == 60);
		assertTrue(testKeen.getY() == 70);
	}
	
	public void testGetSprites()
	{
		//Check we have the right number of sprites
		List sprites = test.getSprites();
		//assertTrue(sprites.size() == 8);
		
		//test first sprite is ok
		Iterator it = sprites.iterator();
		testAutoRectangleTile(it);
		
		//Skip up  untill the hints
		it.next();
		it.next();

		testDirectionHint(it);
	}
	
	private void testAutoRectangleTile(Iterator it)
	{
		Sprite testSprite = (Sprite)it.next();

		assertTrue(testSprite instanceof AutoRectangleTile);		
		assertTrue(testSprite.getLayer().equals("walls"));
		assertTrue(testSprite.getX() == 20);
		assertTrue(testSprite.getY() == 220);
		
		//test second sprite is ok
		testSprite = (Sprite)it.next();

		assertTrue(testSprite instanceof AutoRectangleTile);		
		assertTrue(testSprite.getLayer().equals("walls"));
		assertTrue(testSprite.getX() == 200);
		assertTrue(testSprite.getY() == 250);
	}
	
	private void testDirectionHint(Iterator it)
	{
		Sprite testSprite = (Sprite)it.next();

		assertTrue(testSprite instanceof DirectionHint);		
		assertTrue(testSprite.getLayer().equals("platforms"));
		assertTrue(testSprite.getX() == 500);
		assertTrue(testSprite.getY() == 150);
		assertTrue(((DirectionHint)testSprite).getDirection()  == MovingActor.NORTH_EAST);

		//check that all the directions come out
		testSprite = (Sprite)it.next();
		assertTrue(((DirectionHint)testSprite).getDirection()  == MovingActor.EAST);
		testSprite = (Sprite)it.next();
		assertTrue(((DirectionHint)testSprite).getDirection()  == MovingActor.SOUTH_EAST);
		testSprite = (Sprite)it.next();
		assertTrue(((DirectionHint)testSprite).getDirection()  == MovingActor.SOUTH);
		testSprite = (Sprite)it.next();
		assertTrue(((DirectionHint)testSprite).getDirection()  == MovingActor.SOUTH_WEST);
		testSprite = (Sprite)it.next();
		assertTrue(((DirectionHint)testSprite).getDirection()  == MovingActor.WEST);
		testSprite = (Sprite)it.next();
		assertTrue(((DirectionHint)testSprite).getDirection()  == MovingActor.NORTH_WEST);
		testSprite = (Sprite)it.next();
		assertTrue(((DirectionHint)testSprite).getDirection()  == MovingActor.NORTH);
	}
}
