package keenGame.test;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import junit.framework.TestCase;
import keenGame.XmlElement;

public class XmlElementTest extends TestCase
{
	private XmlElement testXmlElement;
	private Element testElement;

	/**
	 * Constructor for XmlElementTest.
	 * @param arg0
	 */
	public XmlElementTest(String arg0)
	{
		super(arg0);
	}

	/**
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception
	{
		super.setUp();
		
		Document xmlDoc = null;
		try
		{
			xmlDoc = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
		}
		catch(ParserConfigurationException e)
		{
			System.err.println("WFT");
			System.exit(0);
		}
		
		Element rootElement = xmlDoc.createElement("sprite");
		//set width and height
		rootElement.setAttribute("width",Integer.toString(100));
		rootElement.setAttribute("height",Integer.toString(200));
		
    	//x
    	Element xElement = xmlDoc.createElement("x");
    	xElement.appendChild(xmlDoc.createTextNode(Integer.toString(50)));
    	rootElement.appendChild(xElement);
    	
    	//y
    	Element yElement = xmlDoc.createElement("y");
    	yElement.appendChild(xmlDoc.createTextNode(Integer.toString(55)));
    	rootElement.appendChild(yElement);
    	 	
    	//layer
    	Element spriteLayer = xmlDoc.createElement("spriteLayer");
    	spriteLayer.appendChild(xmlDoc.createTextNode("thisIsALayer"));
    	rootElement.appendChild(spriteLayer);

    	Element hitLayer = xmlDoc.createElement("hitLayers");
    	rootElement.appendChild(hitLayer);

    	Element layer = xmlDoc.createElement("layer");
    	layer.appendChild(xmlDoc.createTextNode("foo"));
    	hitLayer.appendChild(layer);
    	layer = xmlDoc.createElement("layer");
    	layer.appendChild(xmlDoc.createTextNode("blah"));
    	hitLayer.appendChild(layer);
    	layer = xmlDoc.createElement("layer");
    	layer.appendChild(xmlDoc.createTextNode("stuff"));
    	hitLayer.appendChild(layer);

		testXmlElement = new XmlElement(rootElement);
		testElement = rootElement;
	}

	public void testGetElement()
	{
		assertTrue(testElement == testXmlElement.getElement());
	}

	public void testGetName()
	{
		assertTrue(testXmlElement.getName().equals("sprite"));
		assertTrue(testXmlElement.getChild("x").getName().equals("x"));
		assertTrue(testXmlElement.getChild("hitLayers").getName().equals("hitLayers"));
	}

	public void testGetAttribute()
	{
		assertTrue(testXmlElement.getAttribute("width").equals("100"));
		assertTrue(testXmlElement.getAttribute("height").equals("200"));
		assertTrue(testXmlElement.getAttribute("something") == null);
	}

	public void testGetAttributes()
	{
		String [] attributes = testXmlElement.getAttributes();
		
		assertTrue(attributes.length == 2);
		assertTrue(attributes[0].equals("width"));
		assertTrue(attributes[1].equals("height"));
	}

	public void testGetChild()
	{
		assertTrue(testXmlElement.getChild("hitLayers").getName().equals("hitLayers"));
		assertTrue(testXmlElement.getChild("blah") == null);
	}

	public void testGetChildren()
	{
		XmlElement [] children = testXmlElement.getChild("hitLayers").getChildren();
		
		assertTrue(children.length == 3);
	}

	public void testGetText()
	{
		XmlElement [] children = testXmlElement.getChild("hitLayers").getChildren();
		
		assertTrue(children[0].getText().equals("foo"));
		assertTrue(children[1].getText().equals("blah"));
		assertTrue(children[2].getText().equals("stuff"));
	}

	public void testGetValue()
	{
		assertTrue(testXmlElement.getChild("x").getValue() == 50);
		assertTrue(testXmlElement.getChild("y").getValue() == 55);
	}

}
